@@ -1,5 +1,5 @@
 /*
-    Copyright 2008-2013
+    Copyright 2008-2022
         Matthias Ehmann,
         Michael Gerhaeuser,
         Carsten Miller,
@@ -67,9 +67,18 @@ define([
 
     "use strict";
 
-    /**
-     * Chart plotting
-     */
+     /**
+      *
+      * The Chart class is a basic class for the chart object.
+      * @class Creates a new basic chart object. Do not use this constructor to create a chart.
+      * Use {@link JXG.Board#create} with type {@link Chart} instead.
+      * @constructor
+      * @augments JXG.GeometryElement
+      * @param {String,JXG.Board} board The board the new chart is drawn on.
+      * @param {Array} parent data arrays for the chart
+      * @param {Object} attributes Javascript object containing attributes like name, id and colors.
+      *
+      */
     JXG.Chart = function (board, parents, attributes) {
         this.constructor(board, attributes);
 
@@ -148,9 +157,19 @@ define([
 
         return this.elements;
     };
+
     JXG.Chart.prototype = new GeometryElement();
 
     JXG.extend(JXG.Chart.prototype, /** @lends JXG.Chart.prototype */ {
+        /**
+         * Create line chart defined by two data arrays.
+         *
+         * @param  {String,JXG.Board} board      The board the chart is drawn on
+         * @param  {Array} x          Array of x-coordinates
+         * @param  {Array} y          Array of y-coordinates
+         * @param  {Object} attributes  Javascript object containing attributes like colors
+         * @returns {JXG.Curve}       JSXGraph curve
+         */
         drawLine: function (board, x, y, attributes) {
             // we don't want the line chart to be filled
             attributes.fillcolor = 'none';
@@ -159,6 +178,16 @@ define([
             return board.create('curve', [x, y], attributes);
         },
 
+        /**
+         * Create line chart that consists of a natural spline curve
+         * defined by two data arrays.
+         *
+         * @param  {String,JXG.Board} board      The board the chart is drawn on
+         * @param  {Array} x          Array of x-coordinates
+         * @param  {Array} y          Array of y-coordinates
+         * @param  {Object} attributes Javascript object containing attributes like colors
+         * @returns {JXG.Curve}       JSXGraph (natural) spline curve
+         */
         drawSpline: function (board, x, y, attributes) {
             // we don't want the spline chart to be filled
             attributes.fillColor = 'none';
@@ -167,6 +196,17 @@ define([
             return board.create('spline', [x, y], attributes);
         },
 
+        /**
+         * Create line chart where the curve is given by a regression polynomial
+         * defined by two data arrays. The degree of the polynomial is supplied
+         * through the attribute "degree" in attributes.
+         *
+         * @param  {String,JXG.Board} board      The board the chart is drawn on
+         * @param  {Array} x          Array of x-coordinates
+         * @param  {Array} y          Array of y-coordinates
+         * @param  {Object} attributes Javascript object containing attributes like colors
+         * @returns {JXG.Curve}    JSXGraph function graph object
+         */
         drawFit: function (board, x, y, attributes) {
             var deg = attributes.degree;
 
@@ -179,18 +219,32 @@ define([
             return board.create('functiongraph', [Numerics.regressionPolynomial(deg, x, y)], attributes);
         },
 
+        /**
+         * Create bar chart defined by two data arrays.
+         * Attributes to change the layout of the bar chart are:
+         * <ul>
+         * <li> width (optional)
+         * <li> dir: 'horizontal' or 'vertical'
+         * <li> colors: array of colors
+         * <li> labels: array of labels
+         * </ul>
+         *
+         * @param  {String,JXG.Board} board      The board the chart is drawn on
+         * @param  {Array} x          Array of x-coordinates
+         * @param  {Array} y          Array of y-coordinates
+         * @param  {Object} attributes Javascript object containing attributes like colors
+         * @returns {Array}    Array of JXG polygons defining the bars
+         */
         drawBar: function (board, x, y, attributes) {
-            var i, strwidth, fill, fs, text, w, xp0, xp1, xp2, yp, colors,
+            var i, strwidth, text, w, xp0, xp1, xp2, yp, colors,
                 pols = [],
                 p = [],
-                attr,
-
+                attr, attrSub,
                 makeXpFun = function (i, f) {
                     return function () {
                         return x[i]() - f * w;
                     };
                 },
-
                 hiddenPoint = {
                     fixed: true,
                     withLabel: false,
@@ -198,13 +252,11 @@ define([
                     name: ''
                 };
 
-            if (!Type.exists(attributes.fillopacity)) {
-                attributes.fillopacity = 0.6;
-            }
+            attr = Type.copyAttributes(attributes, board.options, 'chart');
 
             // Determine the width of the bars
-            if (attributes && attributes.width) {  // width given
-                w = attributes.width;
+            if (attr && attr.width) {  // width given
+                w = attr.width;
             } else {
                 if (x.length <= 1) {
                     w = 1;
@@ -218,43 +270,41 @@ define([
                 w *= 0.8;
             }
 
-            fill = attributes.fillcolor;
-
-            attr = Type.copyAttributes(attributes, board.options, 'chart', 'label');
-            fs = parseFloat(attr.fontsize);
+            attrSub = Type.copyAttributes(attributes, board.options, 'chart', 'label');
 
             for (i = 0; i < x.length; i++) {
                 if (Type.isFunction(x[i])) {
                     xp0 = makeXpFun(i, -0.5);
-
                     xp1 = makeXpFun(i, 0);
-
                     xp2 = makeXpFun(i, 0.5);
                 } else {
                     xp0 = x[i] - w * 0.5;
                     xp1 = x[i];
                     xp2 = x[i] + w * 0.5;
                 }
+                if (Type.isFunction(y[i])) {
+                    yp = y[i]();
+                } else {
+                    yp = y[i];
+                }
                 yp = y[i];
-                if (attributes.dir === 'horizontal') {  // horizontal bars
+
+                if (attr.dir === 'horizontal') {  // horizontal bars
                     p[0] = board.create('point', [0, xp0], hiddenPoint);
                     p[1] = board.create('point', [yp, xp0], hiddenPoint);
                     p[2] = board.create('point', [yp, xp2], hiddenPoint);
                     p[3] = board.create('point', [0, xp2], hiddenPoint);
 
-                    if (Type.exists(attributes.labels) && Type.exists(attributes.labels[i])) {
-                        strwidth = attributes.labels[i].toString().length;
-                        strwidth = 2 * strwidth * fs / board.unitX;
+                    if (Type.exists(attr.labels) && Type.exists(attr.labels[i])) {
+                        attrSub.anchorY = 'middle';
+                        text = board.create('text', [
+                            yp,
+                            xp1,
+                            attr.labels[i]], attrSub);
+                        text.visProp.anchorx = (function(txt) { return function() {
+                            return (txt.X() >= 0) ? 'left' : 'right';
+                        }; })(text);
 
-                        if (yp >= 0) {
-                            // Static offset for label
-                            yp += fs * 0.5 / board.unitX;
-                        } else {
-                            // Static offset for label
-                            yp -= fs * strwidth / board.unitX;
-                        }
-                        xp1 -= fs * 0.2 / board.unitY;
-                        text = board.create('text', [yp, xp1, attributes.labels[i]].toString(), attr);
                     }
                 } else { // vertical bars
                     p[0] = board.create('point', [xp0, 0], hiddenPoint);
@@ -262,31 +312,30 @@ define([
                     p[2] = board.create('point', [xp2, yp], hiddenPoint);
                     p[3] = board.create('point', [xp2, 0], hiddenPoint);
 
-                    if (Type.exists(attributes.labels) && Type.exists(attributes.labels[i])) {
-                        strwidth = attributes.labels[i].toString().length;
-                        strwidth = 0.6 * strwidth * fs / board.unitX;
+                    if (Type.exists(attr.labels) && Type.exists(attr.labels[i])) {
+                        attrSub.anchorX = 'middle';
+
+                        text = board.create('text', [
+                            xp1,
+                            yp,
+                            attr.labels[i]], attrSub);
+
+                        text.visProp.anchory = (function(txt) {
+                            return function() {
+                                    return (txt.Y() >= 0) ? 'bottom' : 'top';
+                                };
+                            })(text);
 
-                        if (yp >= 0) {
-                            // Static offset for label
-                            yp += fs * 0.5 / board.unitY;
-                        } else {
-                            // Static offset for label
-                            yp -= fs / board.unitY;
-                        }
-                        text = board.create('text', [xp1 - strwidth * 0.5, yp, attributes.labels[i].toString()], attr);
                     }
                 }
 
-                attributes.withlines = false;
-
-                if (Type.isArray(attributes.colors)) {
-                    colors = attributes.colors;
-                    attributes.fillcolor = colors[i % colors.length];
+                if (Type.isArray(attr.colors)) {
+                    colors = attr.colors;
+                    attr.fillcolor = colors[i % colors.length];
                 }
 
-                pols[i] = board.create('polygon', p, attributes);
-
-                if (Type.exists(attributes.labels) && Type.exists(attributes.labels[i])) {
+                pols[i] = board.create('polygon', p, attr);
+                if (Type.exists(attr.labels) && Type.exists(attr.labels[i])) {
                     pols[i].text = text;
                 }
             }
@@ -294,6 +343,20 @@ define([
             return pols;
         },
 
+        /**
+         * Create chart consisting of JSXGraph points.
+         * Attributes to change the layout of the point chart are:
+         * <ul>
+         * <li> fixed (Boolean)
+         * <li> infoboxArray (Array): Texts for the infobox
+         * </ul>
+         *
+         * @param  {String,JXG.Board} board      The board the chart is drawn on
+         * @param  {Array} x          Array of x-coordinates
+         * @param  {Array} y          Array of y-coordinates
+         * @param  {Object} attributes Javascript object containing attributes like colors
+         * @returns {Array} Array of JSXGraph points
+         */
         drawPoints: function (board, x, y, attributes) {
             var i,
                 points = [],
@@ -310,6 +373,23 @@ define([
             return points;
         },
 
+        /**
+         * Create pie chart.
+         * Attributes to change the layout of the pie chart are:
+         * <ul>
+         * <li> labels: array of labels
+         * <li> colors: (Array)
+         * <li> highlightColors (Array)
+         * <li> radius
+         * <li> center (coordinate array)
+         * <li> highlightOnSector (Boolean)
+         * </ul>
+         *
+         * @param  {String,JXG.Board} board      The board the chart is drawn on
+         * @param  {Array} y          Array of x-coordinates
+         * @param  {Object} attributes Javascript object containing attributes like colors
+         * @returns {Object}  with keys: "{sectors, points, midpoint}"
+         */
         drawPie: function (board, y, attributes) {
             var i, center,
                 p = [],
@@ -326,7 +406,8 @@ define([
 
                 makeRadPointFun = function (j, fun, xc) {
                     return function () {
-                        var s, t = 0, i, rad;
+                        var s, i, rad,
+                            t = 0;
 
                         for (i = 0; i <= j; i++) {
                             t += parseFloat(Type.evaluate(y[i]));
@@ -347,15 +428,15 @@ define([
                         dy = -this.point1.coords.usrCoords[2] + this.point2.coords.usrCoords[2];
 
                     if (Type.exists(this.label)) {
-                        this.label.rendNode.style.fontSize = (s * this.label.visProp.fontsize) + 'px';
-                        this.label.prepareUpdate().update().updateRenderer();
+                        this.label.rendNode.style.fontSize = (s * Type.evaluate(this.label.visProp.fontsize)) + 'px';
+                        this.label.fullUpdate();
                     }
 
                     this.point2.coords = new Coords(Const.COORDS_BY_USER, [
                         this.point1.coords.usrCoords[1] + dx * f,
                         this.point1.coords.usrCoords[2] + dy * f
                     ], this.board);
-                    this.prepareUpdate().update().updateRenderer();
+                    this.fullUpdate();
                 },
 
                 highlightFun = function () {
@@ -438,10 +519,35 @@ define([
             return {sectors: sector, points: p, midpoint: center};
         },
 
-        /*
-         * labelArray=[ row1, row2, row3 ]
-         * paramArray=[ paramx, paramy, paramz ]
-         * parents=[[x1, y1, z1], [x2, y2, z2], [x3, y3, z3]]
+        /**
+         * Create radar chart.
+         * Attributes to change the layout of the pie chart are:
+         * <ul>
+         * <li> paramArray: labels for axes, [ paramx, paramy, paramz ]
+         * <li> startShiftRatio: 0 <= offset from chart center <=1
+         * <li> endShiftRatio:  0 <= offset from chart radius <=1
+         * <li> startShiftArray: Adjust offsets per each axis
+         * <li> endShiftArray: Adjust offsets per each axis
+         * <li> startArray: Values for inner circle. Default values: minimums
+         * <li> start: one value to overwrite all startArray values
+         * <li> endArray: Values for outer circle, maximums by default
+         * <li> end: one value to overwrite all endArray values
+         * <li> labelArray
+         * <li> polyStrokeWidth
+         * <li> colors
+         * <li> highlightcolors
+         * <li> labelArray: [ row1, row2, row3 ]
+         * <li> radius
+         * <li> legendPosition
+         * <li> showCircles
+         * <li> circleLabelArray
+         * <li> circleStrokeWidth
+         * </ul>
+         *
+         * @param  {String,JXG.Board} board      The board the chart is drawn on
+         * @param  {Array} parents    Array of coordinates, e.g. [[x1, y1, z1], [x2, y2, z2], [x3, y3, z3]]
+         * @param  {Object} attributes Javascript object containing attributes like colors
+         * @returns {Object} with keys "{circles, lines, points, midpoint, polygons}"
          */
         drawRadar: function (board, parents, attributes) {
             var i, j, paramArray, numofparams, maxes, mins,
@@ -455,7 +561,7 @@ define([
 
                 get_anchor = function () {
                     var x1, x2, y1, y2,
-                        relCoords = this.visProp.label.offset.slice(0);
+                        relCoords = Type.evaluate(this.visProp.label.offset).slice(0);
 
                     x1 = this.point1.X();
                     x2 = this.point2.X();
@@ -487,25 +593,21 @@ define([
                 };
 
             if (len <= 0) {
-                JXG.debug("No data");
-                return;
+                throw new Error('JSXGraph radar chart: no data');
             }
             // labels for axes
             paramArray = attributes.paramarray;
             if (!Type.exists(paramArray)) {
-                JXG.debug("Need paramArray attribute");
-                return;
+                throw new Error('JSXGraph radar chart: need paramArray attribute');
             }
             numofparams = paramArray.length;
             if (numofparams <= 1) {
-                JXG.debug("Need more than 1 param");
-                return;
+                throw new Error('JSXGraph radar chart: need more than one param in paramArray');
             }
 
             for (i = 0; i < len; i++) {
                 if (numofparams !== parents[i].length) {
-                    JXG.debug("Use data length equal to number of params (" + parents[i].length + " != " + numofparams + ")");
-                    return;
+                    throw new Error('JSXGraph radar chart: use data length equal to number of params (' + parents[i].length + ' != ' + numofparams + ')');
                 }
             }
 
@@ -571,23 +673,19 @@ define([
             }
 
             if (sshifts.length !== numofparams) {
-                JXG.debug("Start shifts length is not equal to number of parameters");
-                return;
+                throw new Error('JSXGraph radar chart: start shifts length is not equal to number of parameters');
             }
 
             if (eshifts.length !== numofparams) {
-                JXG.debug("End shifts length is not equal to number of parameters");
-                return;
+                throw new Error('JSXGraph radar chart: end shifts length is not equal to number of parameters');
             }
 
             if (starts.length !== numofparams) {
-                JXG.debug("Starts length is not equal to number of parameters");
-                return;
+                throw new Error('JSXGraph radar chart: starts length is not equal to number of parameters');
             }
 
             if (ends.length !== numofparams) {
-                JXG.debug("Ends length is not equal to number of parameters");
-                return;
+                throw new Error('JSXGraph radar chart: snds length is not equal to number of parameters');
             }
 
             // labels for legend
@@ -699,8 +797,7 @@ define([
                 ncircles = clabelArray.length;
 
                 if (ncircles < 2) {
-                    JXG.debug("Too less circles");
-                    return;
+                    throw new Error('JSXGraph radar chart: too less circles in circleLabelArray');
                 }
 
                 pcircles = [];
@@ -739,18 +836,15 @@ define([
             };
         },
 
-        /**
-         * Then, the update function of the renderer
-         * is called.  Since a chart is only an abstract element,
-         * containing other elements, this function is empty.
-         */
+         /**
+          * Uses the boards renderer to update the chart.
+          * @private
+          */
         updateRenderer: function () {
             return this;
         },
 
-        /**
-         * Update of the defining points
-         */
+         // documented in base/element
         update: function () {
             if (this.needsUpdate) {
                 this.updateDataArray();
@@ -760,22 +854,267 @@ define([
         },
 
         /**
-         * For dynamic charts update
-         * can be used to compute new entries
+         * Template for dynamic charts update.
+         * This method is used to compute new entries
          * for the arrays this.dataX and
-         * this.dataY. It is used in @see update.
+         * this.dataY. It is used in update.
          * Default is an empty method, can be overwritten
          * by the user.
+         *
+         * @returns {JXG.Chart} Reference to this chart object.
          */
-        updateDataArray: function () {}
+        updateDataArray: function () { return this; }
     });
 
-    JXG.createChart = function (board, parents, attributes) {
-        var data, row, i, j, col, charts = [], w, x, showRows, attr,
-            originalWidth, name, strokeColor, fillColor, hStrokeColor, hFillColor, len,
-            table = Env.isBrowser ? document.getElementById(parents[0]) : null;
+    /**
+     * @class Constructor for a chart.
+     * @pseudo
+     * @description
+     * @name Chart
+     * @augments JXG.Chart
+     * @constructor
+     * @type JXG.Chart
+     * @throws {Exception} If the element cannot be constructed with the given parent objects an exception is thrown.
+     * @param {Array} x Array of x-coordinates (default case, see below for alternatives)
+     * @param {Array} y Array of y-coordinates (default case, see below for alternatives)
+     * <p>
+     * The parent array may be of one of the following forms:
+     * <ol>
+     * <li> Parents array looks like [number, number, number, ...]. It is interpreted as array of y-coordinates.
+     * The x coordinates are automatically set to [1, 2, ...]
+     * <li> Parents array looks like [[number, number, number, ...]]. The content is interpreted as array of y-coordinates.
+     * The x coordinates are automatically set to [1, 2, ...]x coordinates are automatically set to [1, 2, ...]
+     * Default case: [[x0,x1,x2,...],[y1,y2,y3,...]]
+     * </ol>
+     *
+     * The attribute value for the key 'chartStyle' determines the type(s) of the chart. 'chartStyle' is a comma
+     * separated list of strings of the possible chart types
+     * 'bar', 'fit', 'line',  'pie', 'point', 'radar', 'spline'.
+     *
+     * @see JXG.Chart#drawBar
+     * @see JXG.Chart#drawFit
+     * @see JXG.Chart#drawLine
+     * @see JXG.Chart#drawPie
+     * @see JXG.Chart#drawPoints
+     * @see JXG.Chart#drawRadar
+     * @see JXG.Chart#drawSpline
+     *
+     * @example
+     *   board = JXG.JSXGraph.initBoard('jxgbox', {boundingbox:[-0.5,8,9,-2],axis:true});
+     *
+     *   var f = [4, 2, -1, 3, 6, 7, 2];
+     *   var chart = board.create('chart', f,
+     *                 {chartStyle:'bar',
+     *                  width:0.8,
+     *                  labels:f,
+     *                  colorArray:['#8E1B77','#BE1679','#DC1765','#DA2130','#DB311B','#DF4917','#E36317','#E87F1A',
+     *                              '#F1B112','#FCF302','#C1E212'],
+     *                  label: {fontSize:30, display:'internal', anchorX:'left', rotate:90}
+     *             });
+     *
+     * </pre><div id="JXG1528c395-9fa4-4210-ada6-7fc5652ed920" class="jxgbox" style="width: 300px; height: 300px;"></div>
+     * <script type="text/javascript">
+     *     (function() {
+     *         var board = JXG.JSXGraph.initBoard('JXG1528c395-9fa4-4210-ada6-7fc5652ed920',
+     *             {boundingbox: [-0.5,8,9,-2], axis: true, showcopyright: false, shownavigation: false});
+     *                 var f = [4,2,-1,3,6,7,2];
+     *                 var chart = board.create('chart', f,
+     *                     {chartStyle:'bar',
+     *                      width:0.8,
+     *                      labels:f,
+     *                      colorArray:['#8E1B77','#BE1679','#DC1765','#DA2130','#DB311B','#DF4917','#E36317','#E87F1A',
+     *                                  '#F1B112','#FCF302','#C1E212'],
+     *                      label: {fontSize:30, display:'internal', anchorX:'left', rotate:90}
+     *                 });
+     *
+     *     })();
+     *
+     * </script><pre>
+     *
+     * @example
+     *   board = JXG.JSXGraph.initBoard('jxgbox', {boundingbox: [-1, 9, 13, -3], axis:true});
+     *
+     *   var s = board.create('slider', [[4,7],[8,7],[1,1,1.5]], {name:'S', strokeColor:'black', fillColor:'white'});
+     *   var f = [function(){return (s.Value()*4.5).toFixed(2);},
+     *                      function(){return (s.Value()*(-1)).toFixed(2);},
+     *                      function(){return (s.Value()*3).toFixed(2);},
+     *                      function(){return (s.Value()*2).toFixed(2);},
+     *                      function(){return (s.Value()*(-0.5)).toFixed(2);},
+     *                      function(){return (s.Value()*5.5).toFixed(2);},
+     *                      function(){return (s.Value()*2.5).toFixed(2);},
+     *                      function(){return (s.Value()*(-0.75)).toFixed(2);},
+     *                      function(){return (s.Value()*3.5).toFixed(2);},
+     *                      function(){return (s.Value()*2).toFixed(2);},
+     *                      function(){return (s.Value()*(-1.25)).toFixed(2);}
+     *                      ];
+     *   var chart = board.create('chart', [f],
+     *                                             {chartStyle:'bar',width:0.8,labels:f,
+     *                                              colorArray:['#8E1B77','#BE1679','#DC1765','#DA2130','#DB311B','#DF4917','#E36317','#E87F1A',
+     *                                                          '#F1B112','#FCF302','#C1E212']});
+     *
+     *   var dataArr = [4,1,3,2,5,6.5,1.5,2,0.5,1.5,-1];
+     *   var chart2 = board.create('chart', dataArr, {chartStyle:'line,point'});
+     *   chart2[0].setAttribute('strokeColor:black','strokeWidth:2pt');
+     *   for(var i=0; i<11;i++) {
+     *            chart2[1][i].setAttribute({strokeColor:'black',fillColor:'white',face:'[]', size:4, strokeWidth:'2pt'});
+     *   }
+     *   board.unsuspendUpdate();
+     *
+     * </pre><div id="JXG22deb158-48c6-41c3-8157-b88b4b968a55" class="jxgbox" style="width: 300px; height: 300px;"></div>
+     * <script type="text/javascript">
+     *     (function() {
+     *         var board = JXG.JSXGraph.initBoard('JXG22deb158-48c6-41c3-8157-b88b4b968a55',
+     *             {boundingbox: [-1, 9, 13, -3], axis: true, showcopyright: false, shownavigation: false});
+     *                 var s = board.create('slider', [[4,7],[8,7],[1,1,1.5]], {name:'S', strokeColor:'black', fillColor:'white'});
+     *                 var f = [function(){return (s.Value()*4.5).toFixed(2);},
+     *                          function(){return (s.Value()*(-1)).toFixed(2);},
+     *                          function(){return (s.Value()*3).toFixed(2);},
+     *                          function(){return (s.Value()*2).toFixed(2);},
+     *                          function(){return (s.Value()*(-0.5)).toFixed(2);},
+     *                          function(){return (s.Value()*5.5).toFixed(2);},
+     *                          function(){return (s.Value()*2.5).toFixed(2);},
+     *                          function(){return (s.Value()*(-0.75)).toFixed(2);},
+     *                          function(){return (s.Value()*3.5).toFixed(2);},
+     *                          function(){return (s.Value()*2).toFixed(2);},
+     *                          function(){return (s.Value()*(-1.25)).toFixed(2);}
+     *                          ];
+     *                 var chart = board.create('chart', [f],
+     *                                                 {chartStyle:'bar',width:0.8,labels:f,
+     *                                                  colorArray:['#8E1B77','#BE1679','#DC1765','#DA2130','#DB311B','#DF4917','#E36317','#E87F1A',
+     *                                                              '#F1B112','#FCF302','#C1E212']});
+     *
+     *                 var dataArr = [4,1,3,2,5,6.5,1.5,2,0.5,1.5,-1];
+     *                 var chart2 = board.create('chart', dataArr, {chartStyle:'line,point'});
+     *                 chart2[0].setAttribute('strokeColor:black','strokeWidth:2pt');
+     *                 for(var i=0; i<11;i++) {
+     *                     chart2[1][i].setAttribute({strokeColor:'black',fillColor:'white',face:'[]', size:4, strokeWidth:'2pt'});
+     *                 }
+     *                 board.unsuspendUpdate();
+     *
+     *     })();
+     *
+     * </script><pre>
+     *
+      * @example
+     *         var dataArr = [4, 1.2, 3, 7, 5, 4, 1.54, function () { return 2; }];
+     *         var a = board.create('chart', dataArr, {
+     *                 chartStyle:'pie', colors:['#B02B2C','#3F4C6B','#C79810','#D15600'],
+     *                 fillOpacity:0.9,
+     *                 center:[5,2],
+     *                 strokeColor:'#ffffff',
+     *                 strokeWidth:6,
+     *                 highlightBySize:true,
+     *                 highlightOnSector:true
+     *             });
+     *
+     * </pre><div id="JXG1180b7dd-b048-436a-a5ad-87ffa82d5aff" class="jxgbox" style="width: 300px; height: 300px;"></div>
+     * <script type="text/javascript">
+     *     (function() {
+     *         var board = JXG.JSXGraph.initBoard('JXG1180b7dd-b048-436a-a5ad-87ffa82d5aff',
+     *             {boundingbox: [0, 8, 12, -4], axis: true, showcopyright: false, shownavigation: false});
+     *             var dataArr = [4, 1.2, 3, 7, 5, 4, 1.54, function () { return 2; }];
+     *             var a = board.create('chart', dataArr, {
+     *                     chartStyle:'pie', colors:['#B02B2C','#3F4C6B','#C79810','#D15600'],
+     *                     fillOpacity:0.9,
+     *                     center:[5,2],
+     *                     strokeColor:'#ffffff',
+     *                     strokeWidth:6,
+     *                     highlightBySize:true,
+     *                     highlightOnSector:true
+     *                 });
+     *
+     *     })();
+     *
+     * </script><pre>
+     *
+     * @example
+     *             board = JXG.JSXGraph.initBoard('jxgbox', {boundingbox: [-12, 12, 20, -12], axis: false});
+     *             board.suspendUpdate();
+     *             // See labelArray and paramArray
+     *             var dataArr = [[23, 14, 15.0], [60, 8, 25.0], [0, 11.0, 25.0], [10, 15, 20.0]];
+     *
+     *             var a = board.create('chart', dataArr, {
+     *                 chartStyle:'radar',
+     *                 colorArray:['#0F408D','#6F1B75','#CA147A','#DA2228','#E8801B','#FCF302','#8DC922','#15993C','#87CCEE','#0092CE'],
+     *                 //fillOpacity:0.5,
+     *                 //strokeColor:'black',
+     *                 //strokeWidth:1,
+     *                 //polyStrokeWidth:1,
+     *                 paramArray:['Speed','Flexibility', 'Costs'],
+     *                 labelArray:['Ruby','JavaScript', 'PHP', 'Python'],
+     *                 //startAngle:Math.PI/4,
+     *                 legendPosition:'right',
+     *                 //"startShiftRatio": 0.1,
+     *                 //endShiftRatio:0.1,
+     *                 //startShiftArray:[0,0,0],
+     *                 //endShiftArray:[0.5,0.5,0.5],
+     *                 start:0
+     *                 //end:70,
+     *                 //startArray:[0,0,0],
+     *                 //endArray:[7,7,7],
+     *                 //radius:3,
+     *                 //showCircles:true,
+     *                 //circleLabelArray:[1,2,3,4,5],
+     *                 //highlightColorArray:['#E46F6A','#F9DF82','#F7FA7B','#B0D990','#69BF8E','#BDDDE4','#92C2DF','#637CB0','#AB91BC','#EB8EBF'],
+     *             });
+     *             board.unsuspendUpdate();
+     *
+     * </pre><div id="JXG985fbbe6-0488-4073-b73b-cb3ebaea488a" class="jxgbox" style="width: 300px; height: 300px;"></div>
+     * <script type="text/javascript">
+     *     (function() {
+     *         var board = JXG.JSXGraph.initBoard('JXG985fbbe6-0488-4073-b73b-cb3ebaea488a',
+     *             {boundingbox: [-12, 12, 20, -12], axis: false, showcopyright: false, shownavigation: false});
+     *                 board.suspendUpdate();
+     *                 // See labelArray and paramArray
+     *                 var dataArr = [[23, 14, 15.0], [60, 8, 25.0], [0, 11.0, 25.0], [10, 15, 20.0]];
+     *
+     *                 var a = board.create('chart', dataArr, {
+     *                     chartStyle:'radar',
+     *                     colorArray:['#0F408D','#6F1B75','#CA147A','#DA2228','#E8801B','#FCF302','#8DC922','#15993C','#87CCEE','#0092CE'],
+     *                     //fillOpacity:0.5,
+     *                     //strokeColor:'black',
+     *                     //strokeWidth:1,
+     *                     //polyStrokeWidth:1,
+     *                     paramArray:['Speed','Flexibility', 'Costs'],
+     *                     labelArray:['Ruby','JavaScript', 'PHP', 'Python'],
+     *                     //startAngle:Math.PI/4,
+     *                     legendPosition:'right',
+     *                     //"startShiftRatio": 0.1,
+     *                     //endShiftRatio:0.1,
+     *                     //startShiftArray:[0,0,0],
+     *                     //endShiftArray:[0.5,0.5,0.5],
+     *                     start:0
+     *                     //end:70,
+     *                     //startArray:[0,0,0],
+     *                     //endArray:[7,7,7],
+     *                     //radius:3,
+     *                     //showCircles:true,
+     *                     //circleLabelArray:[1,2,3,4,5],
+     *                     //highlightColorArray:['#E46F6A','#F9DF82','#F7FA7B','#B0D990','#69BF8E','#BDDDE4','#92C2DF','#637CB0','#AB91BC','#EB8EBF'],
+     *                 });
+     *                 board.unsuspendUpdate();
+     *
+     *     })();
+     *
+     * </script><pre>
+     *
+     * For more examples see
+     * <ul>
+     * <li><a href="https://jsxgraph.org/wiki/index.php/Charts_from_HTML_tables_-_tutorial">JSXgraph wiki: Charts from HTML tables - tutorial</a>
+     * <li><a href="https://jsxgraph.org/wiki/index.php/Pie_chart">JSXgraph wiki: Pie chart</a>
+     * <li><a href="https://jsxgraph.org/wiki/index.php/Different_chart_styles">JSXgraph wiki: Various chart styles</a>
+     * <li><a href="https://jsxgraph.org/wiki/index.php/Dynamic_bar_chart">JSXgraph wiki: Dynamic bar chart</a>
+     * </ul>
+     */
+     JXG.createChart = function (board, parents, attributes) {
+         var data, row, i, j, col,
+             charts = [],
+             w, x, showRows, attr,
+             originalWidth, name, strokeColor, fillColor,
+             hStrokeColor, hFillColor, len,
+             table = Env.isBrowser ? board.document.getElementById(parents[0]) : null;
 
-        if ((parents.length === 1) && (typeof parents[0] === 'string')) {
+        if ((parents.length === 1) && (Type.isString(parents[0]))) {
             if (Type.exists(table)) {
                 // extract the data
                 attr = Type.copyAttributes(attributes, board.options, 'chart');
@@ -881,8 +1220,20 @@ define([
 
     /**
      * Legend for chart
+     * TODO
      *
-     **/
+     * The Legend class is a basic class for legends.
+     * @class Creates a new Lgend object. Do not use this constructor to create a legend.
+     * Use {@link JXG.Board#create} with type {@link Legend} instead.
+     * <p>
+     * The legend object consists of segements with labels. These lines can be
+     * access with the property "lines" of the element.
+     * @constructor
+     * @augments JXG.GeometryElement
+     * @param {String,JXG.Board} board The board the new legend is drawn on.
+     * @param {Array} coords Coordinates of the left top point of the legend.
+     * @param  {Object} attributes Attributes of the legend
+     */
     JXG.Legend = function (board, coords, attributes) {
         var attr;
 
@@ -910,8 +1261,16 @@ define([
             throw new Error('JSXGraph: Unknown legend style: ' + this.style);
         }
     };
+
     JXG.Legend.prototype = new GeometryElement();
 
+    /**
+     * Draw a vertical legend.
+     *
+     * @private
+     * @param  {String,JXG.Board} board      The board the legend is drawn on
+     * @param  {Object} attributes Attributes of the legend
+     */
     JXG.Legend.prototype.drawVerticalLegend = function (board, attributes) {
         var i,
             line_length = attributes.linelength || 1,
@@ -923,12 +1282,12 @@ define([
             };
 
         for (i = 0; i < this.label_array.length; i++) {
-            this.myAtts.strokecolor = this.color_array[i];
-            this.myAtts.highlightstrokecolor = this.color_array[i];
             this.myAtts.name = this.label_array[i];
+            this.myAtts.strokecolor = this.color_array[i % this.color_array.length];
+            this.myAtts.highlightstrokecolor = this.color_array[i % this.color_array.length];
             this.myAtts.label = {
                 offset: [10, 0],
-                strokeColor: this.color_array[i],
+                strokeColor: this.color_array[i % this.color_array.length ],
                 strokeWidth: this.myAtts.strokewidth
             };
 
@@ -938,22 +1297,73 @@ define([
                 this.myAtts);
 
             this.lines[i].getLabelAnchor = getLabelAnchor;
-
+            this.lines[i].prepareUpdate().update().updateVisibility(Type.evaluate(this.lines[i].visProp.visible)).updateRenderer();
         }
     };
 
+    /**
+     * @class This element is used to provide a constructor for a chart legend.
+     * Parameter is a pair of coordinates. The label names and  the label colors are
+     * supplied in the attributes:
+     * <ul>
+     * <li> labels (Array): array of strings containing label names
+     * <li> labelArray (Array): alternative array for label names (has precedence over 'labels')
+     * <li> colors (Array): array of color values
+     * <li> colorArray (Array): alternative array for color values (has precedence over 'colors')
+     * <li> legendStyle or style: at the time being only 'vertical' is supported.
+     * <li> rowHeight.
+     * </ul>
+     *
+     * @pseudo
+     * @description
+     * @name Legend
+     * @augments JXG.Legend
+     * @constructor
+     * @type JXG.Legend
+     * @throws {Exception} If the element cannot be constructed with the given parent objects an exception is thrown.
+     * @param {Number} x Horizontal coordinate of the left top point of the legend
+     * @param {Number} y Vertical coordinate of the left top point of the legend
+     *
+     * @example
+     * var board = JXG.JSXGraph.initBoard('jxgbox', {axis:true,boundingbox:[-4,48.3,12.0,-2.3]});
+     * var x       = [-3,-2,-1,0,1,2,3,4,5,6,7,8];
+     * var dataArr = [4,7,7,27,33,37,46,22,11,4,1,0];
+     *
+     * colors = ['green', 'yellow', 'red', 'blue'];
+     * board.create('chart', [x,dataArr], {chartStyle:'bar', width:1.0, labels:dataArr, colors: colors} );
+     * board.create('legend', [8, 45], {labels:dataArr, colors: colors, strokeWidth:5} );
+     *
+     * </pre><div id="JXGeeb588d9-a4fd-41bf-93f4-cd6f7a016682" class="jxgbox" style="width: 300px; height: 300px;"></div>
+     * <script type="text/javascript">
+     *     (function() {
+     *         var board = JXG.JSXGraph.initBoard('JXGeeb588d9-a4fd-41bf-93f4-cd6f7a016682',
+     *             {boundingbox: [-4,48.3,12.0,-2.3], axis: true, showcopyright: false, shownavigation: false});
+     *     var x       = [-3,-2,-1,0,1,2,3,4,5,6,7,8];
+     *     var dataArr = [4,7,7,27,33,37,46,22,11,4,1,0];
+     *
+     *     colors = ['green', 'yellow', 'red', 'blue'];
+     *     board.create('chart', [x,dataArr], {chartStyle:'bar', width:1.0, labels:dataArr, colors: colors} );
+     *     board.create('legend', [8, 45], {labels:dataArr, colors: colors, strokeWidth:5} );
+     *
+     *     })();
+     *
+     * </script><pre>
+     *
+     *
+     */
     JXG.createLegend = function (board, parents, attributes) {
         //parents are coords of left top point of the legend
         var start_from = [0, 0];
 
-        if (Type.exists(parents)) {
-            if (parents.length === 2) {
+        if (Type.exists(parents) && parents.length === 2) {
                 start_from = parents;
-            }
+        } else {
+            throw new Error('JSXGraph: Legend element needs two numbers as parameters');
         }
 
         return new JXG.Legend(board, start_from, attributes);
     };
+
     JXG.registerElement('legend', JXG.createLegend);
 
     return {
