@@ -1,5 +1,5 @@
 /*
-    Copyright 2008-2013
+    Copyright 2008-2022
         Matthias Ehmann,
         Michael Gerhaeuser,
         Carsten Miller,
@@ -51,14 +51,11 @@
  * @fileoverview The geometry object Circle is defined in this file. Circle stores all
  * style and functional properties that are required to draw and move a circle on
  * a board.
- * @author graphjs
- * @version 0.1
  */
 
 define([
-    'jxg', 'base/element', 'base/coords', 'base/constants', 'parser/geonext', 'math/geometry', 'math/statistics',
-    'utils/type', 'base/transformation', 'base/point'
-], function (JXG, GeometryElement, Coords, Const, GeonextParser, Geometry, Statistics, Type, Transform, Point) {
+    'jxg', 'base/element', 'base/coords', 'base/constants', 'element/conic', 'parser/geonext', 'utils/type'
+], function (JXG, GeometryElement, Coords, Const, Conic, GeonextParser, Type) {
 
     "use strict";
 
@@ -96,7 +93,7 @@ define([
          * <li><b>'pointRadius'</b> which means the circle is defined by its center and its radius given in user units or as term.</li>
          * <li><b>'pointLine'</b> which means the circle is defined by its center and its radius given by the distance from the startpoint and the endpoint of the line.</li>
          * <li><b>'pointCircle'</b> which means the circle is defined by its center and its radius given by the radius of another circle.</li></ul>
-         * @type string
+         * @type String
          * @see #center
          * @see #point2
          * @see #radius
@@ -144,6 +141,8 @@ define([
          */
         this.circle = null;
 
+        this.points = [];
+
         if (method === 'twoPoints') {
             this.point2 = board.select(par2);
             this.radius = this.Radius();
@@ -151,7 +150,7 @@ define([
             this.gxtterm = par2;
             // Converts GEONExT syntax into JavaScript syntax and generally ensures that the radius is a function
             this.updateRadius = Type.createFunction(par2, this.board, null, true);
-            // First evaluation of the graph
+            // First evaluation of the radius function
             this.updateRadius();
         } else if (method === 'pointLine') {
             // dann ist p2 die Id eines Objekts vom Typ Line!
@@ -172,7 +171,12 @@ define([
         this.elType = 'circle';
         this.createLabel();
 
-        this.center.addChild(this);
+        if (Type.exists(this.center._is_new)) {
+            this.addChild(this.center);
+            delete this.center._is_new;
+        } else {
+            this.center.addChild(this);
+        }
 
         if (method === 'pointRadius') {
             this.notifyParents(par2);
@@ -181,12 +185,19 @@ define([
         } else if (method === 'pointCircle') {
             this.circle.addChild(this);
         } else if (method === 'twoPoints') {
-            this.point2.addChild(this);
+            if (Type.exists(this.point2._is_new)) {
+                this.addChild(this.point2);
+                delete this.point2._is_new;
+            } else {
+                this.point2.addChild(this);
+            }
         }
 
         this.methodMap = Type.deepCopy(this.methodMap, {
             setRadius: 'setRadius',
             getRadius: 'getRadius',
+            Area: 'Area',
+            area: 'Area',
             radius: 'Radius',
             center: 'center',
             line: 'line',
@@ -206,13 +217,28 @@ define([
          * @private
          */
         hasPoint: function (x, y) {
-            var prec = this.board.options.precision.hasPoint / (this.board.unitX),
+            var prec, type,
                 mp = this.center.coords.usrCoords,
                 p = new Coords(Const.COORDS_BY_SCREEN, [x, y], this.board),
                 r = this.Radius(),
-                dist = Math.sqrt((mp[1] - p.usrCoords[1]) * (mp[1] - p.usrCoords[1]) + (mp[2] - p.usrCoords[2]) * (mp[2] - p.usrCoords[2]));
+                dx, dy, dist;
 
-            if (this.visProp.hasinnerpoints) {
+
+            if (Type.isObject(Type.evaluate(this.visProp.precision))) {
+                type = this.board._inputDevice;
+                prec = Type.evaluate(this.visProp.precision[type]);
+            } else {
+                // 'inherit'
+                prec = this.board.options.precision.hasPoint;
+            }
+            dx = mp[1] - p.usrCoords[1];
+            dy = mp[2] - p.usrCoords[2];
+            dist = Math.sqrt(dx * dx + dy * dy);
+            // We have to use usrCoords, since Radius is available in usrCoords only.
+            prec += Type.evaluate(this.visProp.strokewidth) * 0.5;
+            prec /= Math.sqrt(this.board.unitX * this.board.unitY);
+
+            if (Type.evaluate(this.visProp.hasinnerpoints)) {
                 return (dist < r + prec);
             }
 
@@ -298,7 +324,7 @@ define([
 
                 rsq = '((' + p1 + ')-(' + m1 + '))^2 + ((' + p2 + ')-(' + m2 + '))^2';
             } else if (this.method === "pointRadius") {
-                if (typeof this.radius === 'number') {
+                if (Type.isNumber(this.radius)) {
                     rsq = (this.radius * this.radius).toString();
                 }
             } else if (this.method === "pointLine") {
@@ -320,8 +346,10 @@ define([
          * Uses the boards renderer to update the circle.
          */
         update: function () {
+            var x, y, z, r, c, i;
+
             if (this.needsUpdate) {
-                if (this.visProp.trace) {
+                if (Type.evaluate(this.visProp.trace)) {
                     this.cloneToBackground(true);
                 }
 
@@ -335,6 +363,24 @@ define([
 
                 this.updateStdform();
                 this.updateQuadraticform();
+
+                // Approximate the circle by 4 Bezier segments
+                // This will be used for intersections of type curve / circle.
+                // See https://spencermortensen.com/articles/bezier-circle/
+                z = this.center.coords.usrCoords[0];
+                x = this.center.coords.usrCoords[1] / z;
+                y = this.center.coords.usrCoords[2] / z;
+                z /= z;
+                r = this.Radius();
+                c = 0.551915024494;
+
+                this.numberPoints = 13;
+                this.dataX = [x + r, x + r, x + r * c, x, x - r * c, x - r, x - r, x - r, x - r * c, x, x + r * c, x + r, x + r];
+                this.dataY = [y, y + r * c, y + r, y + r, y + r, y + r * c, y, y - r * c, y - r, y - r, y - r, y - r * c, y];
+                this.bezierDegree = 3;
+                for (i = 0; i < this.numberPoints; i++) {
+                    this.points[i] = new Coords(Const.COORDS_BY_USER, [this.dataX[i], this.dataY[i]], this.board);
+                }
             }
 
             return this;
@@ -366,6 +412,12 @@ define([
             this.stdform[4] = this.Radius();
             this.stdform[1] = -this.center.coords.usrCoords[1];
             this.stdform[2] = -this.center.coords.usrCoords[2];
+            if (!isFinite(this.stdform[4])) {
+                this.stdform[0] = Type.exists(this.point2) ? -(
+                    this.stdform[1] * this.point2.coords.usrCoords[1] +
+                    this.stdform[2] * this.point2.coords.usrCoords[2]
+                ) : 0;
+            }
             this.normalize();
         },
 
@@ -374,38 +426,48 @@ define([
          * @private
          */
         updateRenderer: function () {
-            var wasReal;
+            // var wasReal;
 
-            if (this.needsUpdate && this.visProp.visible) {
-                wasReal = this.isReal;
+            if (!this.needsUpdate) {
+                return this;
+            }
+
+            if (this.visPropCalc.visible) {
+                // wasReal = this.isReal;
                 this.isReal = (!isNaN(this.center.coords.usrCoords[1] + this.center.coords.usrCoords[2] + this.Radius())) && this.center.isReal;
 
-                if (this.isReal) {
-                    if (wasReal !== this.isReal) {
-                        this.board.renderer.show(this);
-
-                        if (this.hasLabel && this.label.visProp.visible) {
-                            this.board.renderer.show(this.label);
-                        }
-                    }
-                    this.board.renderer.updateEllipse(this);
-                } else {
-                    if (wasReal !== this.isReal) {
-                        this.board.renderer.hide(this);
-
-                        if (this.hasLabel && this.label.visProp.visible) {
-                            this.board.renderer.hide(this.label);
-                        }
-                    }
+                if (//wasReal &&
+                    !this.isReal) {
+                    this.updateVisibility(false);
                 }
-                this.needsUpdate = false;
+            }
+
+            // Update the position
+            if (this.visPropCalc.visible) {
+                this.board.renderer.updateEllipse(this);
             }
 
             // Update the label if visible.
-            if (this.hasLabel && this.label.visProp.visible && this.isReal) {
+            if (this.hasLabel && this.visPropCalc.visible && this.label &&
+                this.label.visPropCalc.visible && this.isReal) {
+
                 this.label.update();
                 this.board.renderer.updateText(this.label);
             }
+
+            // Update rendNode display
+            this.setDisplayRendNode();
+            // if (this.visPropCalc.visible !== this.visPropOld.visible) {
+            //     this.board.renderer.display(this, this.visPropCalc.visible);
+            //     this.visPropOld.visible = this.visPropCalc.visible;
+            //
+            //     if (this.hasLabel) {
+            //         this.board.renderer.display(this.label, this.label.visPropCalc.visible);
+            //     }
+            // }
+
+            this.needsUpdate = false;
+            return this;
         },
 
         /**
@@ -415,7 +477,7 @@ define([
          * @private
          */
         notifyParents: function (contentStr) {
-            if (typeof contentStr === 'string') {
+            if (Type.isString(contentStr)) {
                 GeonextParser.findDependencies(this, contentStr, this.board);
             }
         },
@@ -444,8 +506,9 @@ define([
             }
 
             if (this.method === 'twoPoints') {
-                if (Geometry.distance(this.point2.coords.usrCoords, [0, 0, 0]) === 0 ||
-                        Geometry.distance(this.center.coords.usrCoords, [0, 0, 0]) === 0) {
+                if (Type.cmpArrays(this.point2.coords.usrCoords, [0, 0, 0]) ||
+                        Type.cmpArrays(this.center.coords.usrCoords, [0, 0, 0])) {
+
                     return NaN;
                 }
 
@@ -468,6 +531,7 @@ define([
          * @deprecated
          */
         getRadius: function () {
+            JXG.deprecated('Circle.getRadius()', 'Circle.Radius()');
             return this.Radius();
         },
 
@@ -480,28 +544,29 @@ define([
         getLabelAnchor: function () {
             var x, y,
                 r = this.Radius(),
-                c = this.center.coords.usrCoords;
+                c = this.center.coords.usrCoords,
+                SQRTH = 7.07106781186547524401E-1;      // sqrt(2)/2
 
-            switch (this.visProp.label.position) {
+            switch (Type.evaluate(this.visProp.label.position)) {
             case 'lft':
                 x = c[1] - r;
                 y = c[2];
                 break;
             case 'llft':
-                x = c[1] - Math.sqrt(0.5) * r;
-                y = c[2] - Math.sqrt(0.5) * r;
+                x = c[1] - SQRTH * r;
+                y = c[2] - SQRTH * r;
                 break;
             case 'rt':
                 x = c[1] + r;
                 y = c[2];
                 break;
             case 'lrt':
-                x = c[1] + Math.sqrt(0.5) * r;
-                y = c[2] - Math.sqrt(0.5) * r;
+                x = c[1] + SQRTH * r;
+                y = c[2] - SQRTH * r;
                 break;
             case 'urt':
-                x = c[1] + Math.sqrt(0.5) * r;
-                y = c[2] + Math.sqrt(0.5) * r;
+                x = c[1] + SQRTH * r;
+                y = c[2] + SQRTH * r;
                 break;
             case 'top':
                 x = c[1];
@@ -513,15 +578,14 @@ define([
                 break;
             default:
                 // includes case 'ulft'
-                x = c[1] - Math.sqrt(0.5) * r;
-                y = c[2] + Math.sqrt(0.5) * r;
+                x = c[1] - SQRTH * r;
+                y = c[2] + SQRTH * r;
                 break;
             }
 
             return new Coords(Const.COORDS_BY_USER, [x, y], this.board);
         },
 
-
         // documented in geometry element
         cloneToBackground: function () {
             var er,
@@ -546,6 +610,9 @@ define([
 
             this.numTraces++;
             Type.clearVisPropOld(copy);
+            copy.visPropCalc = {
+                visible: Type.evaluate(copy.visProp.visible)
+            };
 
             er = this.board.renderer.enhancedRendering;
             this.board.renderer.enhancedRendering = true;
@@ -577,66 +644,27 @@ define([
             return this;
         },
 
-        // see geometryelement.js
+        // see element.js
         snapToGrid: function () {
-            if (this.visProp.snaptogrid) {
-                this.center.snapToGrid();
+            var forceIt = Type.evaluate(this.visProp.snaptogrid);
 
-                if (this.method === 'twoPoints') {
-                    this.point2.snapToGrid();
-                }
+            this.center.handleSnapToGrid(forceIt, true);
+            if (this.method === 'twoPoints') {
+                this.point2.handleSnapToGrid(forceIt, true);
             }
 
             return this;
         },
 
-        /**
-         * Sets the position of the circle by translating the center and - in case of {@link JXG.Circle#method} equals
-         * 'twoPoints' - the point on the circle by the amount given in the coords parameter.
-         * @param {Number} method Either {@link JXG#COORDS_BY_SCREEN} or {@link JXG#COORDS_BY_USER}.
-         * @param {Array} coords
-         * @returns {JXG.Circle}
-         */
-        setPosition: function (method, coords) {
-            var t;
+        // see element.js
+        snapToPoints: function () {
+            var forceIt = Type.evaluate(this.visProp.snaptopoints);
 
-            coords = new Coords(method, coords, this.board);
-            t = this.board.create('transform', coords.usrCoords.slice(1), {type: 'translate'});
-            this.addTransform(t);
-
-            return this;
-        },
-
-        /**
-         * Sets x and y coordinate and calls the circle's update() method.
-         * @param {number} method The type of coordinates used here. Possible values are {@link JXG.COORDS_BY_USER} and {@link JXG.COORDS_BY_SCREEN}.
-         * @param {Array} coords coordinate in screen/user units
-         * @param {Array} oldcoords previous coordinate in screen/user units
-         * @returns {JXG.Circle} this element
-         */
-        setPositionDirectly: function (method, coords, oldcoords) {
-            var i, p, diffc,
-                len = this.parents.length;
-
-            coords = new Coords(method, coords, this.board);
-            oldcoords = new Coords(method, oldcoords, this.board);
-
-            diffc = Statistics.subtract(coords.usrCoords, oldcoords.usrCoords);
-
-            for (i = 0; i < len; i++) {
-                if (!this.board.select(this.parents[i]).draggable()) {
-                    return this;
-                }
+            this.center.handleSnapToPoints(forceIt);
+            if (this.method === 'twoPoints') {
+                this.point2.handleSnapToPoints(forceIt);
             }
 
-            for (i = 0; i < len; i++) {
-                p = this.board.select(this.parents[i]);
-                // p.coords.setCoordinates(Const.COORDS_BY_USER, Statistics.add(p.coords.usrCoords, diffc));  // This missed snapToPoints
-                p.setPositionDirectly(Const.COORDS_BY_USER, Statistics.add(p.coords.usrCoords, diffc));
-            }
-
-            this.prepareUpdate().update();
-
             return this;
         },
 
@@ -661,7 +689,7 @@ define([
         /**
          * Treat the circle as parametric curve and calculates its Z coordinate.
          * @param {Number} t ignored
-         * @return {Number} 1.0
+         * @returns {Number} 1.0
          */
         Z: function (t) {
             return 1.0;
@@ -683,17 +711,37 @@ define([
             return 1.0;
         },
 
+        /**
+         * Circle area
+         * @returns {Number} area of the circle.
+         */
         Area: function () {
             var r = this.Radius();
 
             return r * r * Math.PI;
         },
 
+        /**
+         * Get bounding box of the circle.
+         * @returns {Array} [x1, y1, x2, y2]
+         */
         bounds: function () {
             var uc = this.center.coords.usrCoords,
                 r = this.Radius();
 
             return [uc[1] - r, uc[2] + r, uc[1] + r, uc[2] - r];
+        },
+
+        /**
+         * Get data to construct this element. Data consists of the parent elements
+         * and static data like radius.
+         * @returns {Array} data necessary to construct this element
+         */
+        getParents: function() {
+            if (this.parents.length === 1) {  // i.e. this.method === 'pointRadius'
+                return this.parents.concat(this.radius);
+            }
+            return this.parents;
         }
     });
 
@@ -708,56 +756,131 @@ define([
      * @constructor
      * @type JXG.Circle
      * @throws {Exception} If the element cannot be constructed with the given parent objects an exception is thrown.
-     * @param {JXG.Point_number,JXG.Point,JXG.Line,JXG.Circle} center,radius The center must be given as a {@link JXG.Point}, but the radius can be given
+     * @param {JXG.Point_number,JXG.Point,JXG.Line,JXG.Circle} center,radius The center must be given as a {@link JXG.Point}, see {@link JXG.providePoints}, but the radius can be given
      * as a number (which will create a circle with a fixed radius), another {@link JXG.Point}, a {@link JXG.Line} (the distance of start and end point of the
      * line will determine the radius), or another {@link JXG.Circle}.
      * @example
      * // Create a circle providing two points
-     * var p1 = board.create('point', [2.0, 2.0]);
-     * var p2 = board.create('point', [2.0, 0.0]);
-     * var c1 = board.create('circle', [p1, p2]);
+     * var p1 = board.create('point', [2.0, 2.0]),
+     *     p2 = board.create('point', [2.0, 0.0]),
+     *     c1 = board.create('circle', [p1, p2]);
      *
      * // Create another circle using the above circle
-     * var p3 = board.create('point', [3.0, 2.0]);
-     * var c2 = board.create('circle', [p3, c1]);
-     * </pre><div id="5f304d31-ef20-4a8e-9c0e-ea1a2b6c79e0" style="width: 400px; height: 400px;"></div>
+     * var p3 = board.create('point', [3.0, 2.0]),
+     *     c2 = board.create('circle', [p3, c1]);
+     * </pre><div class="jxgbox" id="JXG5f304d31-ef20-4a8e-9c0e-ea1a2b6c79e0" style="width: 400px; height: 400px;"></div>
      * <script type="text/javascript">
-     *   var cex1_board = JXG.JSXGraph.initBoard('5f304d31-ef20-4a8e-9c0e-ea1a2b6c79e0', {boundingbox: [-1, 9, 9, -1], axis: true, showcopyright: false, shownavigation: false});
-     *   var cex1_p1 = cex1_board.create('point', [2.0, 2.0]);
-     *   var cex1_p2 = cex1_board.create('point', [2.0, 0.0]);
-     *   var cex1_c1 = cex1_board.create('circle', [cex1_p1, cex1_p2]);
-     *   var cex1_p3 = cex1_board.create('point', [3.0, 2.0]);
-     *   var cex1_c2 = cex1_board.create('circle', [cex1_p3, cex1_c1]);
+     * (function() {
+     *   var cex1_board = JXG.JSXGraph.initBoard('JXG5f304d31-ef20-4a8e-9c0e-ea1a2b6c79e0', {boundingbox: [-1, 9, 9, -1], axis: true, showcopyright: false, shownavigation: false});
+     *       cex1_p1 = cex1_board.create('point', [2.0, 2.0]),
+     *       cex1_p2 = cex1_board.create('point', [2.0, 0.0]),
+     *       cex1_c1 = cex1_board.create('circle', [cex1_p1, cex1_p2]),
+     *       cex1_p3 = cex1_board.create('point', [3.0, 2.0]),
+     *       cex1_c2 = cex1_board.create('circle', [cex1_p3, cex1_c1]);
+     * })();
      * </script><pre>
+     * @example
+     * // Create a circle providing two points
+     * var p1 = board.create('point', [2.0, 2.0]),
+     *     c1 = board.create('circle', [p1, 3]);
+     *
+     * // Create another circle using the above circle
+     * var c2 = board.create('circle', [function() { return [p1.X(), p1.Y() + 1];}, function() { return c1.Radius(); }]);
+     * </pre><div class="jxgbox" id="JXG54165f60-93b9-441d-8979-ac5d0f193020" style="width: 400px; height: 400px;"></div>
+     * <script type="text/javascript">
+     * (function() {
+     * var board = JXG.JSXGraph.initBoard('JXG54165f60-93b9-441d-8979-ac5d0f193020', {boundingbox: [-1, 9, 9, -1], axis: true, showcopyright: false, shownavigation: false});
+     * var p1 = board.create('point', [2.0, 2.0]);
+     * var c1 = board.create('circle', [p1, 3]);
+     *
+     * // Create another circle using the above circle
+     * var c2 = board.create('circle', [function() { return [p1.X(), p1.Y() + 1];}, function() { return c1.Radius(); }]);
+     * })();
+     * </script><pre>
+     * @example
+     * var li = board.create('line', [1,1,1], {strokeColor: '#aaaaaa'});
+     * var reflect = board.create('transform', [li], {type: 'reflect'});
+     *
+     * var c1 = board.create('circle', [[-2,-2], [-2, -1]], {center: {visible:true}});
+     * var c2 = board.create('circle', [c1, reflect]);
+     *      * </pre><div id="JXGa2a5a870-5dbb-11e8-9fb9-901b0e1b8723" class="jxgbox" style="width: 300px; height: 300px;"></div>
+     * <script type="text/javascript">
+     *     (function() {
+     *         var board = JXG.JSXGraph.initBoard('JXGa2a5a870-5dbb-11e8-9fb9-901b0e1b8723',
+     *             {boundingbox: [-8, 8, 8,-8], axis: true, showcopyright: false, shownavigation: false});
+     *             var li = board.create('line', [1,1,1], {strokeColor: '#aaaaaa'});
+     *             var reflect = board.create('transform', [li], {type: 'reflect'});
+     *
+     *             var c1 = board.create('circle', [[-2,-2], [-2, -1]], {center: {visible:true}});
+     *             var c2 = board.create('circle', [c1, reflect]);
+     *     })();
+     *
+     * </script><pre>
+     *
+     * @example
+     * var t = board.create('transform', [2, 1.5], {type: 'scale'});
+     * var c1 = board.create('circle', [[1.3, 1.3], [0, 1.3]], {strokeColor: 'black', center: {visible:true}});
+     * var c2 = board.create('circle', [c1, t], {strokeColor: 'black'});
+     *
+     * </pre><div id="JXG0686a222-6339-11e8-9fb9-901b0e1b8723" class="jxgbox" style="width: 300px; height: 300px;"></div>
+     * <script type="text/javascript">
+     *     (function() {
+     *         var board = JXG.JSXGraph.initBoard('JXG0686a222-6339-11e8-9fb9-901b0e1b8723',
+     *             {boundingbox: [-8, 8, 8,-8], axis: true, showcopyright: false, shownavigation: false});
+     *     var t = board.create('transform', [2, 1.5], {type: 'scale'});
+     *     var c1 = board.create('circle', [[1.3, 1.3], [0, 1.3]], {strokeColor: 'black', center: {visible:true}});
+     *     var c2 = board.create('circle', [c1, t], {strokeColor: 'black'});
+     *
+     *     })();
+     *
+     * </script><pre>
+     *
      */
     JXG.createCircle = function (board, parents, attributes) {
-        var el, p, i, attr,
-            isDraggable = true;
+        var el, p, i, attr, obj,
+            isDraggable = true,
+            point_style = ['center', 'point2'];
 
         p = [];
+        obj = board.select(parents[0]);
+        if (Type.isObject(obj) && obj.elementClass === Const.OBJECT_CLASS_CIRCLE &&
+            Type.isTransformationOrArray(parents[1])) {
+
+            attr = Type.copyAttributes(attributes, board.options, 'circle');
+            // if (!Type.exists(attr.type) || attr.type.toLowerCase() !== 'euclidean') {
+            //     // Create a circle element from a circle and a Euclidean transformation
+            //     el = JXG.createCircle(board, [obj.center, function() { return obj.Radius(); }], attr);
+            // } else {
+                // Create a conic element from a circle and a projective transformation
+                el = Conic.createEllipse(board, [obj.center, obj.center, function() { return 2 * obj.Radius(); }], attr);
+            // }
+            el.addTransform(parents[1]);
+            return el;
+
+        }
+        // Circle defined by points
         for (i = 0; i < parents.length; i++) {
-            // Point
-            if (Type.isPoint(parents[i])) {
-                p[i] = parents[i];
-            // Coordinates
-            } else if (Type.isArray(parents[i]) && parents[i].length > 1) {
-                attr = Type.copyAttributes(attributes, board.options, 'circle', 'center');
-                p[i] = board.create('point', parents[i], attr);
-            // Something else (number, function, string)
+            if (Type.isPointType(board, parents[i])) {
+                p = p.concat(Type.providePoints(board, [parents[i]], attributes, 'circle', [point_style[i]]));
+                if (p[p.length - 1] === false) {
+                    throw new Error('JSXGraph: Can\'t create circle from this type. Please provide a point type.');
+                }
             } else {
-                p[i] = parents[i];
+                p.push(parents[i]);
             }
         }
 
         attr = Type.copyAttributes(attributes, board.options, 'circle');
 
-        if (parents.length === 2 && Type.isPoint(p[0]) && Type.isPoint(p[1])) {
+        if (p.length === 2 && Type.isPoint(p[0]) && Type.isPoint(p[1])) {
             // Point/Point
             el = new JXG.Circle(board, 'twoPoints', p[0], p[1], attr);
-        } else if ((Type.isNumber(p[0]) || Type.isFunction(p[0]) || Type.isString(p[0])) && Type.isPoint(p[1])) {
+        } else if ((Type.isNumber(p[0]) || Type.isFunction(p[0]) || Type.isString(p[0])) &&
+                    Type.isPoint(p[1])) {
             // Number/Point
             el = new JXG.Circle(board, 'pointRadius', p[1], p[0], attr);
-        } else if ((Type.isNumber(p[1]) || Type.isFunction(p[1]) || Type.isString(p[1])) && Type.isPoint(p[0])) {
+        } else if ((Type.isNumber(p[1]) || Type.isFunction(p[1]) || Type.isString(p[1])) &&
+                Type.isPoint(p[0])) {
             // Point/Number
             el = new JXG.Circle(board, 'pointRadius', p[0], p[1], attr);
         } else if ((p[0].elementClass === Const.OBJECT_CLASS_CIRCLE) && Type.isPoint(p[1])) {
@@ -780,22 +903,21 @@ define([
             } else {
                 throw new Error('JSXGraph: Can\'t create circle with three points. Please include the circumcircle element (element/composition).');
             }
+
         } else {
             throw new Error("JSXGraph: Can't create circle with parent types '" +
                 (typeof parents[0]) + "' and '" + (typeof parents[1]) + "'." +
-                "\nPossible parent types: [point,point], [point,number], [point,function], [point,circle], [point,point,point]");
+                "\nPossible parent types: [point,point], [point,number], [point,function], [point,circle], [point,point,point], [circle,transformation]");
         }
 
         el.isDraggable = isDraggable;
-        el.parents = [];
-
-        for (i = 0; i < parents.length; i++) {
-            if (parents[i].id) {
-                el.parents.push(parents[i].id);
+        el.setParents(p);
+        el.elType = 'circle';
+        for (i = 0; i < p.length; i++) {
+            if (Type.isPoint(p[i])) {
+                el.inherits.push(p[i]);
             }
         }
-
-        el.elType = 'circle';
         return el;
     };
 
