@@ -1,9 +1,10 @@
 /*
-    Copyright 2008-2013
+    Copyright 2008-2022
         Matthias Ehmann,
         Michael Gerhaeuser,
         Carsten Miller,
         Bianca Valentin,
+        Andreas Walter,
         Alfred Wassermann,
         Peter Wilfahrt
 
@@ -29,7 +30,6 @@
     and <http://opensource.org/licenses/MIT/>.
  */
 
-
 /*global JXG: true, define: true*/
 /*jslint nomen: true, plusplus: true*/
 
@@ -38,34 +38,45 @@
  */
 
 define(['jxg'], function (JXG) {
+    'use strict';
 
-    "use strict";
-
-    var major = 0,
-        minor = 98,
-        patch = 0,
-        add = '',
+    var major = 1,
+        minor = 4,
+        patch = 2,
+        add = '', //'dev'
         version = major + '.' + minor + '.' + patch + (add ? '-' + add : ''),
         constants;
 
     constants = /** @lends JXG */ {
-        // copyright, version, ...
-
         /**
-         * Represents the currently used JSXGraph version.
-         * @type {String}
+         * Constant: the currently used JSXGraph version.
+         *
+         * @name JXG.version
+         * @type String
          */
         version: version,
 
         /**
-         * The small gray version indicator in the top left corner of every JSXGraph board (if
+         * Constant: the small gray version indicator in the top left corner of every JSXGraph board (if
          * showCopyright is not set to false on board creation).
+         *
+         * @name JXG.licenseText
          * @type String
          */
-        licenseText: 'JSXGraph v' + version + ' Copyright (C) see http://jsxgraph.org',
+        licenseText: 'JSXGraph v' + version + ' Copyright (C) see https://jsxgraph.org',
 
-        // coords
+        /**
+         *  Constant: user coordinates relative to the coordinates system defined by the bounding box.
+         *  @name JXG.COORDS_BY_USER
+         *  @type Number
+         */
         COORDS_BY_USER: 0x0001,
+
+        /**
+         *  Constant: screen coordinates in pixel relative to the upper left corner of the div element.
+         *  @name JXG.COORDS_BY_SCREEN
+         *  @type Number
+         */
         COORDS_BY_SCREEN: 0x0002,
 
         // object types
@@ -94,6 +105,19 @@ define(['jxg'], function (JXG) {
         OBJECT_TYPE_OPROJECT: 23,
         OBJECT_TYPE_GRID: 24,
         OBJECT_TYPE_TANGENT: 25,
+        OBJECT_TYPE_HTMLSLIDER: 26,
+        OBJECT_TYPE_CHECKBOX: 27,
+        OBJECT_TYPE_INPUT: 28,
+        OBJECT_TYPE_BUTTON: 29,
+        OBJECT_TYPE_TRANSFORMATION: 30,
+        OBJECT_TYPE_FOREIGNOBJECT: 31,
+        OBJECT_TYPE_VIEW3D: 32,
+
+        // IMPORTANT:
+        // ----------
+        // For being able to differentiate between the (sketchometry specific) SPECIAL_OBJECT_TYPEs and
+        // (core specific) OBJECT_TYPEs, the non-sketchometry types MUST NOT be changed
+        // to values > 100.
 
         // object classes
         OBJECT_CLASS_POINT: 1,
@@ -102,13 +126,27 @@ define(['jxg'], function (JXG) {
         OBJECT_CLASS_CURVE: 4,
         OBJECT_CLASS_AREA: 5,
         OBJECT_CLASS_OTHER: 6,
+        OBJECT_CLASS_TEXT: 7,
 
         // SketchReader constants
         GENTYPE_ABC: 1, // unused
         GENTYPE_AXIS: 2,
         GENTYPE_MID: 3,
+
+        /**
+         * @ignore
+         * @deprecated, now use {@link JXG.GENTYPE_REFLECTION_ON_LINE}
+         *
+         */
         GENTYPE_REFLECTION: 4,
-        GENTYPE_MIRRORPOINT: 5,
+        /**
+         * @ignore
+         * @deprecated, now use {@link JXG.GENTYPE_REFLECTION_ON_POINT}
+         */
+        GENTYPE_MIRRORELEMENT: 5,
+
+        GENTYPE_REFLECTION_ON_LINE: 4,
+        GENTYPE_REFLECTION_ON_POINT: 5,
         GENTYPE_TANGENT: 6,
         GENTYPE_PARALLEL: 7,
         GENTYPE_BISECTORLINES: 8,
@@ -119,7 +157,11 @@ define(['jxg'], function (JXG) {
         GENTYPE_GLIDER: 13,
         GENTYPE_INTERSECTION: 14,
         GENTYPE_CIRCLE: 15,
+        /**
+         * @ignore @deprecated NOT USED ANY MORE SINCE SKETCHOMETRY 2.0 (only for old constructions needed)
+         */
         GENTYPE_CIRCLE2POINTS: 16,
+
         GENTYPE_LINE: 17,
         GENTYPE_TRIANGLE: 18,
         GENTYPE_QUADRILATERAL: 19,
@@ -137,15 +179,28 @@ define(['jxg'], function (JXG) {
         GENTYPE_RULER: 31,
         GENTYPE_SLOPETRIANGLE: 32,
         GENTYPE_PERPSEGMENT: 33,
-        // 34 ... 39 // unused ...
+        GENTYPE_LABELMOVEMENT: 34,
+        GENTYPE_VECTOR: 35,
+        GENTYPE_NONREFLEXANGLE: 36,
+        GENTYPE_REFLEXANGLE: 37,
+        GENTYPE_PATH: 38,
+        GENTYPE_DERIVATIVE: 39,
+        // 40 // unused ...
         GENTYPE_DELETE: 41,
         GENTYPE_COPY: 42,
         GENTYPE_MIRROR: 43,
         GENTYPE_ROTATE: 44,
         GENTYPE_ABLATION: 45,
         GENTYPE_MIGRATE: 46,
-//        GENTYPE_TRANSFORM: 47, // unused
-        // 48 ... 50 // unused ...
+        GENTYPE_VECTORCOPY: 47,
+        GENTYPE_POLYGONCOPY: 48,    /**
+        * Constants
+        * @name Constants
+        * @namespace
+        */
+
+        //        GENTYPE_TRANSFORM: 48, // unused
+        // 49 ... 50 // unused ...
 
         // IMPORTANT:
         // ----------
@@ -180,10 +235,14 @@ define(['jxg'], function (JXG) {
         GENTYPE_CTX_INTVAL: 75,
         GENTYPE_CTX_POINT1: 76,
         GENTYPE_CTX_POINT2: 77,
-        GENTYPE_CTX_LABELSTICKY: 78
+        GENTYPE_CTX_LABELSTICKY: 78,
+        GENTYPE_CTX_TYPE_I: 79,
+        GENTYPE_CTX_HASINNERPOINTS: 80,
+        GENTYPE_CTX_SNAPWIDTH: 81,
+        GENTYPE_CTX_SNAPTOGRID: 82
     };
 
-    JXG.extend(JXG, constants);
+    JXG.extendConstants(JXG, constants);
 
     return constants;
 });
