@@ -1,5 +1,5 @@
 /*
-    Copyright 2008-2013
+    Copyright 2008-2022
         Matthias Ehmann,
         Michael Gerhaeuser,
         Carsten Miller,
@@ -52,8 +52,8 @@
  */
 
 define([
-    'jxg', 'base/constants', 'base/element', 'utils/type', 'base/curve', 'base/point', 'base/line', 'base/transformation'
-], function (JXG, Const, GeometryElement, Type, Curve, Point, Line, Transform) {
+    'jxg', 'base/constants', 'base/element', 'utils/type'
+], function (JXG, Const, GeometryElement, Type) {
 
     "use strict";
 
@@ -72,6 +72,61 @@ define([
      * [[x, y]]
      * @param {Object} attributes Attributes to change the visual properties of the turtle object
      * All angles are in degrees.
+     *
+     * @example
+     *
+     * //creates a figure 8 animation
+     * var board = JXG.JSXGraph.initBoard('jxgbox',{boundingbox: [-250, 250, 250, -250]});
+     * var t = board.create('turtle',[0, 0], {strokeOpacity:0.5});
+     * t.setPenSize(3);
+     * t.right(90);
+     * var alpha = 0;
+     *
+     * var run = function() {
+     *  t.forward(2);
+     *  if (Math.floor(alpha / 360) % 2 === 0) {
+     *   t.left(1);        // turn left by 1 degree
+     *  } else {
+     *   t.right(1);       // turn right by 1 degree
+     *  }
+     *  alpha += 1;
+     *
+     *  if (alpha < 1440) {  // stop after two rounds
+     *   setTimeout(run, 20);
+     *  }
+     * }
+     *
+     *run();
+     *
+     * </pre><div class="jxgbox" id="JXG14167b1c-2ad3-11e5-8dd9-901b0e1b8723" style="width: 300px; height: 300px;"></div>
+     * <script type="text/javascript">
+     *     (function() {
+     *         var brd = JXG.JSXGraph.initBoard('JXG14167b1c-2ad3-11e5-8dd9-901b0e1b8723',
+     *             {boundingbox: [-250, 250, 250, -250], axis: true, showcopyright: false, shownavigation: false});
+     *               var t = brd.create('turtle',[0, 0], {strokeOpacity:0.5});
+     *               t.setPenSize(3);
+     *               t.right(90);
+     *               var alpha = 0;
+     *
+     *              var run = function() {
+     *              t.forward(2);
+     *             if (Math.floor(alpha / 360) % 2 === 0) {
+     *                t.left(1);        // turn left by 1 degree
+     *              } else {
+     *                   t.right(1);       // turn right by 1 degree
+     *             }
+     *             alpha += 1;
+     *
+     *             if (alpha < 1440) {  // stop after two rounds
+     *                 setTimeout(run, 20);
+     *               }
+     *             }
+     *
+     *          run();
+     *
+     *     })();
+     *
+     * </script><pre>
      */
     JXG.Turtle = function (board, parents, attributes) {
         var x, y, dir;
@@ -196,6 +251,11 @@ define([
             this.arrow = this.board.create('line', [this.turtle, this.turtle2], this.visProp.arrow);
             this.objects.push(this.arrow);
 
+            this.subs = {
+                arrow: this.arrow
+            };
+            this.inherits.push(this.arrow);
+
             this.right(90 - dir);
             this.board.update();
         },
@@ -573,52 +633,53 @@ define([
         },
 
         /**
-         * Alias for {@link #forward}
+         * Alias for {@link JXG.Turtle#forward}
          */
         fd: function (len) { return this.forward(len); },
         /**
-         * Alias for {@link #back}
+         * Alias for {@link JXG.Turtle#back}
          */
         bk: function (len) { return this.back(len); },
         /**
-         * Alias for {@link #left}
+         * Alias for {@link JXG.Turtle#left}
          */
         lt: function (angle) { return this.left(angle); },
         /**
-         * Alias for {@link #right}
+         * Alias for {@link JXG.Turtle#right}
          */
         rt: function (angle) { return this.right(angle); },
         /**
-         * Alias for {@link #penUp}
+         * Alias for {@link JXG.Turtle#penUp}
          */
         pu: function () { return this.penUp(); },
         /**
-         * Alias for {@link #penDown}
+         * Alias for {@link JXG.Turtle#penDown}
          */
         pd: function () { return this.penDown(); },
         /**
-         * Alias for {@link #hideTurtle}
+         * Alias for {@link JXG.Turtle#hideTurtle}
          */
         ht: function () { return this.hideTurtle(); },
         /**
-         * Alias for {@link #showTurtle}
+         * Alias for {@link JXG.Turtle#showTurtle}
          */
         st: function () { return this.showTurtle(); },
         /**
-         * Alias for {@link #clearScreen}
+         * Alias for {@link JXG.Turtle#clearScreen}
          */
         cs: function () { return this.clearScreen(); },
         /**
-         * Alias for {@link #pushTurtle}
+         * Alias for {@link JXG.Turtle#pushTurtle}
          */
         push: function () { return this.pushTurtle(); },
         /**
-         * Alias for {@link #popTurtle}
+         * Alias for {@link JXG.Turtle#popTurtle}
          */
         pop: function () { return this.popTurtle(); },
 
         /**
-         * the "co"-coordinate of the turtle curve at position t is returned.
+         * The "co"-coordinate of the turtle curve at position t is returned.
+         *
          * @param {Number} t parameter
          * @param {String} co. Either 'X' or 'Y'.
          * @returns {Number} x-coordinate of the turtle position or x-coordinate of turtle at position t
@@ -727,7 +788,14 @@ define([
     });
 
     /**
-     * Creates a new turtle
+     * @class This element is used to provide a constructor for a turtle.
+     * @pseudo
+     * @description  Creates a new turtle
+     * @name Turtle
+     * @augments JXG.Turtle
+     * @constructor
+     * @type JXG.Turtle
+     *
      * @param {JXG.Board} board The board the turtle is put on.
      * @param {Array} parents
      * @param {Object} attributes Object containing properties for the element such as stroke-color and visibility. See {@link JXG.GeometryElement#setAttribute}
