@@ -1,5 +1,5 @@
 /*
-    Copyright 2008-2013
+    Copyright 2008-2022
         Matthias Ehmann,
         Michael Gerhaeuser,
         Carsten Miller,
@@ -53,15 +53,15 @@
  */
 
 define([
-    'jxg', 'math/geometry', 'math/math', 'base/coords', 'base/circle', 'utils/type', 'base/constants',
-    'base/curve', 'element/composition'
-], function (JXG, Geometry, Mat, Coords, Circle, Type, Const, Curve, Compositions) {
+    'jxg', 'math/geometry', 'math/math', 'base/coords', 'base/circle', 'utils/type', 'base/constants'
+], function (JXG, Geometry, Mat, Coords, Circle, Type, Const) {
 
     "use strict";
 
     /**
      * @class An arc is a segment of the circumference of a circle. It is defined by a center, one point that
      * defines the radius, and a third point that defines the angle of the arc.
+     *
      * @pseudo
      * @name Arc
      * @augments Curve
@@ -77,43 +77,55 @@ define([
      * var p3 = board.create('point', [3.5, 1.0]);
      *
      * var a = board.create('arc', [p1, p2, p3]);
-     * </pre><div id="114ef584-4a5e-4686-8392-c97501befb5b" style="width: 300px; height: 300px;"></div>
+     * board.create('text',[1,6,function(){return 'arclength: '+Math.round(a.Value()*100)/100}])
+     * </pre><div class="jxgbox" id="JXG114ef584-4a5e-4686-8392-c97501befb5b" style="width: 300px; height: 300px;"></div>
      * <script type="text/javascript">
      * (function () {
-     *   var board = JXG.JSXGraph.initBoard('114ef584-4a5e-4686-8392-c97501befb5b', {boundingbox: [-1, 7, 7, -1], axis: true, showcopyright: false, shownavigation: false}),
+     *   var board = JXG.JSXGraph.initBoard('JXG114ef584-4a5e-4686-8392-c97501befb5b', {boundingbox: [-1, 7, 7, -1], axis: true, showcopyright: false, shownavigation: false}),
      *       p1 = board.create('point', [2.0, 2.0]),
      *       p2 = board.create('point', [1.0, 0.5]),
      *       p3 = board.create('point', [3.5, 1.0]),
      *
      *       a = board.create('arc', [p1, p2, p3]);
+     *       board.create('text',[1,6,function(){return 'arclength: '+Math.round(a.Value()*100)/100}])
      * })();
      * </script><pre>
+     *
+     * @example
+     * var t = board.create('transform', [2, 1.5], {type: 'scale'});
+     * var a1 = board.create('arc', [[1, 1], [0, 1], [1, 0]], {strokeColor: 'red'});
+     * var a2 = board.create('curve', [a1, t], {strokeColor: 'red'});
+     *
+     * </pre><div id="JXG1949da46-6339-11e8-9fb9-901b0e1b8723" class="jxgbox" style="width: 300px; height: 300px;"></div>
+     * <script type="text/javascript">
+     *     (function() {
+     *         var board = JXG.JSXGraph.initBoard('JXG1949da46-6339-11e8-9fb9-901b0e1b8723',
+     *             {boundingbox: [-8, 8, 8,-8], axis: true, showcopyright: false, shownavigation: false});
+     *     var t = board.create('transform', [2, 1.5], {type: 'scale'});
+     *     var a1 = board.create('arc', [[1, 1], [0, 1], [1, 0]], {strokeColor: 'red'});
+     *     var a2 = board.create('curve', [a1, t], {strokeColor: 'red'});
+     *
+     *     })();
+     *
+     * </script><pre>
+     *
      */
     JXG.createArc = function (board, parents, attributes) {
-        var el, attr, i;
+        var el, attr, points;
 
-
-        // this method is used to create circumccirclearcs, too. if a circumcirclearc is created we get a fourth
-        // point, that's why we need to check that case, too.
-        if (parents.length < 3 || parents[0].elementClass !== Const.OBJECT_CLASS_POINT || parents[1].elementClass !== Const.OBJECT_CLASS_POINT ||
-                parents[2].elementClass !== Const.OBJECT_CLASS_POINT || (parents[3] && parents[3].elementClass !== Const.OBJECT_CLASS_POINT)) {
+        points = Type.providePoints(board, parents, attributes, 'arc', ['center', 'radiusPoint', 'anglePoint']);
+        if (points === false || points.length < 3) {
             throw new Error("JSXGraph: Can't create Arc with parent types '" +
                 (typeof parents[0]) + "' and '" + (typeof parents[1]) + "' and '" +
                 (typeof parents[2]) + "'." +
-                "\nPossible parent types: [point,point,point]");
+                "\nPossible parent types: [point,point,point], [arc, transformation]");
         }
 
         attr = Type.copyAttributes(attributes, board.options, 'arc');
         el = board.create('curve', [[0], [0]], attr);
 
         el.elType = 'arc';
-
-        el.parents = [];
-        for (i = 0; i < parents.length; i++) {
-            if (parents[i].id) {
-                el.parents.push(parents[i].id);
-            }
-        }
+        el.setParents(points);
 
         /**
          * documented in JXG.GeometryElement
@@ -127,7 +139,7 @@ define([
          * @name center
          * @type JXG.Point
          */
-        el.center = board.select(parents[0]);
+        el.center = points[0];
 
         /**
          * Point defining the arc's radius.
@@ -135,7 +147,7 @@ define([
          * @name radiuspoint
          * @type JXG.Point
          */
-        el.radiuspoint = board.select(parents[1]);
+        el.radiuspoint = points[1];
         el.point2 = el.radiuspoint;
 
         /**
@@ -144,45 +156,62 @@ define([
          * @name anglepoint
          * @type JXG.Point
          */
-        el.anglepoint = board.select(parents[2]);
+        el.anglepoint = points[2];
         el.point3 = el.anglepoint;
 
         // Add arc as child to defining points
-        el.center.addChild(el);
-        el.radiuspoint.addChild(el);
-        el.anglepoint.addChild(el);
+        // or vice versa if the points are provided as coordinates
+        if (Type.exists(el.center._is_new)) {
+            el.addChild(el.center);
+            delete el.center._is_new;
+        } else {
+            el.center.addChild(el);
+        }
+        if (Type.exists(el.radiuspoint._is_new)) {
+            el.addChild(el.radiuspoint);
+            delete el.radiuspoint._is_new;
+        } else {
+            el.radiuspoint.addChild(el);
+        }
+        if (Type.exists(el.anglepoint._is_new)) {
+            el.addChild(el.anglepoint);
+            delete el.anglepoint._is_new;
+        } else {
+            el.anglepoint.addChild(el);
+        }
 
         // should be documented in options
         el.useDirection = attr.usedirection;
 
         // documented in JXG.Curve
         el.updateDataArray = function () {
-            var ar, phi, v, det, p0c, p1c, p2c,
+            var ar, phi, det, p0c, p1c, p2c,
                 sgn = 1,
                 A = this.radiuspoint,
                 B = this.center,
-                C = this.anglepoint;
+                C = this.anglepoint,
+                ev_s = Type.evaluate(this.visProp.selection);
 
             phi = Geometry.rad(A, B, C);
-            if ((this.visProp.type === 'minor' && phi > Math.PI) ||
-                    (this.visProp.type === 'major' && phi < Math.PI)) {
+            if ((ev_s === 'minor' && phi > Math.PI) ||
+                    (ev_s === 'major' && phi < Math.PI)) {
                 sgn = -1;
             }
 
             // This is true for circumCircleArcs. In that case there is
             // a fourth parent element: [center, point1, point3, point2]
             if (this.useDirection) {
-                p0c = parents[1].coords.usrCoords;
-                p1c = parents[3].coords.usrCoords;
-                p2c = parents[2].coords.usrCoords;
+                p0c = points[1].coords.usrCoords;
+                p1c = points[3].coords.usrCoords;
+                p2c = points[2].coords.usrCoords;
                 det = (p0c[1] - p2c[1]) * (p0c[2] - p1c[2]) - (p0c[2] - p2c[2]) * (p0c[1] - p1c[1]);
 
                 if (det < 0) {
-                    this.radiuspoint = parents[1];
-                    this.anglepoint = parents[2];
+                    this.radiuspoint = points[1];
+                    this.anglepoint = points[2];
                 } else {
-                    this.radiuspoint = parents[2];
-                    this.anglepoint = parents[1];
+                    this.radiuspoint = points[2];
+                    this.anglepoint = points[1];
                 }
             }
 
@@ -220,17 +249,41 @@ define([
          * @returns {Number}
          */
         el.getRadius = function () {
+            JXG.deprecated('Arc.getRadius()', 'Arc.Radius()');
             return this.Radius();
         };
 
+        /**
+         * Returns the length of the arc.
+         * @memberOf Arc.prototype
+         * @name Value
+         * @function
+         * @returns {Number} The arc length
+         */
+        el.Value = function () {
+            return this.Radius() * Geometry.rad(this.radiuspoint, this.center, this.anglepoint);
+        };
+
         // documented in geometry element
         el.hasPoint = function (x, y) {
             var dist, checkPoint,
-                has, angle, alpha, beta,
+                has,
                 invMat, c,
-                prec = this.board.options.precision.hasPoint / this.board.unitX,
+                prec, type,
                 r = this.Radius();
 
+            if (Type.evaluate(this.visProp.hasinnerpoints)) {
+                return this.hasPointSector(x, y);
+            }
+
+            if (Type.isObject(Type.evaluate(this.visProp.precision))) {
+                type = this.board._inputDevice;
+                prec = Type.evaluate(this.visProp.precision[type]);
+            } else {
+                // 'inherit'
+                prec = this.board.options.precision.hasPoint;
+            }
+            prec /= Math.min(this.board.unitX, this.board.unitY);
             checkPoint = new Coords(Const.COORDS_BY_SCREEN, [x, y], this.board);
 
             if (this.transformations.length > 0) {
@@ -247,22 +300,11 @@ define([
 
             /**
              * At that point we know that the user has touched the circle line.
+             * Now, we have to check, if the user has hit the arc path.
              */
             if (has) {
-                angle = Geometry.rad(this.radiuspoint, this.center, checkPoint.usrCoords.slice(1));
-                alpha = 0.0;
-                beta = Geometry.rad(this.radiuspoint, this.center, this.anglepoint);
-
-                if ((this.visProp.type === 'minor' && beta > Math.PI) ||
-                        (this.visProp.type === 'major' && beta < Math.PI)) {
-                    alpha = beta;
-                    beta = 2 * Math.PI;
-                }
-                if (angle < alpha || angle > beta) {
-                    has = false;
-                }
+                has = Geometry.coordsOnArc(this, checkPoint);
             }
-
             return has;
         };
 
@@ -276,27 +318,14 @@ define([
          * @returns {Boolean} True if (x,y) is within the sector defined by the arc, False otherwise.
          */
         el.hasPointSector = function (x, y) {
-            var angle, alpha, beta,
-                checkPoint = new Coords(Const.COORDS_BY_SCREEN, [x, y], this.board),
+            var checkPoint = new Coords(Const.COORDS_BY_SCREEN, [x, y], this.board),
                 r = this.Radius(),
                 dist = this.center.coords.distance(Const.COORDS_BY_USER, checkPoint),
                 has = (dist < r);
 
             if (has) {
-                angle = Geometry.rad(this.radiuspoint, this.center, checkPoint.usrCoords.slice(1));
-                alpha = 0;
-                beta = Geometry.rad(this.radiuspoint, this.center, this.anglepoint);
-
-                if ((this.visProp.type === 'minor' && beta > Math.PI) ||
-                        (this.visProp.type === 'major' && beta < Math.PI)) {
-                    alpha = beta;
-                    beta = 2 * Math.PI;
-                }
-                if (angle < alpha || angle > beta) {
-                    has = false;
-                }
+                has = Geometry.coordsOnArc(this, checkPoint);
             }
-
             return has;
         };
 
@@ -307,21 +336,24 @@ define([
 
         // documented in geometry element
         el.getLabelAnchor = function () {
-            var coords, vecx, vecy, len,
+            var coords, vec, vecx, vecy, len,
                 angle = Geometry.rad(this.radiuspoint, this.center, this.anglepoint),
                 dx = 10 / this.board.unitX,
                 dy = 10 / this.board.unitY,
                 p2c = this.point2.coords.usrCoords,
                 pmc = this.center.coords.usrCoords,
                 bxminusax = p2c[1] - pmc[1],
-                byminusay = p2c[2] - pmc[2];
+                byminusay = p2c[2] - pmc[2],
+                ev_s = Type.evaluate(this.visProp.selection),
+                l_vp = this.label ? this.label.visProp : this.visProp.label;
 
-            if (Type.exists(this.label)) {
-                this.label.relativeCoords = new Coords(Const.COORDS_BY_SCREEN, [0, 0], this.board);
-            }
+            // If this is uncommented, the angle label can not be dragged
+            //if (Type.exists(this.label)) {
+            //    this.label.relativeCoords = new Coords(Const.COORDS_BY_SCREEN, [0, 0], this.board);
+            //}
 
-            if ((this.visProp.type === 'minor' && angle > Math.PI) ||
-                    (this.visProp.type === 'major' && angle < Math.PI)) {
+            if ((ev_s === 'minor' && angle > Math.PI) ||
+                    (ev_s === 'major' && angle < Math.PI)) {
                 angle = -(2 * Math.PI - angle);
             }
 
@@ -336,8 +368,11 @@ define([
             len = Math.sqrt(vecx * vecx + vecy * vecy);
             vecx = vecx * (len + dx) / len;
             vecy = vecy * (len + dy) / len;
+            vec = [pmc[1] + vecx, pmc[2] + vecy];
 
-            return new Coords(Const.COORDS_BY_USER, [pmc[1] + vecx, pmc[2] + vecy], this.board);
+            l_vp.position = Geometry.calcLabelQuadrant(Geometry.rad([1,0],[0,0],vec));
+
+            return new Coords(Const.COORDS_BY_USER, vec, this.board);
         };
 
         // documentation in jxg.circle
@@ -351,7 +386,8 @@ define([
             radius: 'Radius',
             center: 'center',
             radiuspoint: 'radiuspoint',
-            anglepoint: 'anglepoint'
+            anglepoint: 'anglepoint',
+            Value: 'Value'
         });
 
         el.prepareUpdate().update();
@@ -376,10 +412,10 @@ define([
      * var p2 = board.create('point', [1.0, 0.5]);
      *
      * var a = board.create('semicircle', [p1, p2]);
-     * </pre><div id="5385d349-75d7-4078-b732-9ae808db1b0e" style="width: 300px; height: 300px;"></div>
+     * </pre><div class="jxgbox" id="JXG5385d349-75d7-4078-b732-9ae808db1b0e" style="width: 300px; height: 300px;"></div>
      * <script type="text/javascript">
      * (function () {
-     *   var board = JXG.JSXGraph.initBoard('5385d349-75d7-4078-b732-9ae808db1b0e', {boundingbox: [-1, 7, 7, -1], axis: true, showcopyright: false, shownavigation: false}),
+     *   var board = JXG.JSXGraph.initBoard('JXG5385d349-75d7-4078-b732-9ae808db1b0e', {boundingbox: [-1, 7, 7, -1], axis: true, showcopyright: false, shownavigation: false}),
      *       p1 = board.create('point', [4.5, 2.0]),
      *       p2 = board.create('point', [1.0, 0.5]),
      *
@@ -388,37 +424,37 @@ define([
      * </script><pre>
      */
     JXG.createSemicircle = function (board, parents, attributes) {
-        var el, mp, attr;
+        var el, mp, attr, points;
 
         // we need 2 points
-        if ((Type.isPoint(parents[0])) && (Type.isPoint(parents[1]))) {
-            attr = Type.copyAttributes(attributes, board.options, 'semicircle', 'midpoint');
-            mp = board.create('midpoint', [parents[0], parents[1]], attr);
-
-            mp.dump = false;
-
-            attr = Type.copyAttributes(attributes, board.options, 'semicircle');
-            el = board.create('arc', [mp, parents[1], parents[0]], attr);
-
-            el.elType = 'semicircle';
-            el.parents = [parents[0].id, parents[1].id];
-            el.subs = {
-                midpoint: mp
-            };
-
-            /**
-             * The midpoint of the two defining points.
-             * @memberOf Semicircle.prototype
-             * @name midpoint
-             * @type Midpoint
-             */
-            el.midpoint = el.center = mp;
-        } else {
+        points = Type.providePoints(board, parents, attributes, 'point');
+        if (points === false || points.length !== 2) {
             throw new Error("JSXGraph: Can't create Semicircle with parent types '" +
                 (typeof parents[0]) + "' and '" + (typeof parents[1]) + "'." +
                 "\nPossible parent types: [point,point]");
         }
 
+        attr = Type.copyAttributes(attributes, board.options, 'semicircle', 'center');
+        mp = board.create('midpoint', points, attr);
+        mp.dump = false;
+
+        attr = Type.copyAttributes(attributes, board.options, 'semicircle');
+        el = board.create('arc', [mp, points[1], points[0]], attr);
+        el.elType = 'semicircle';
+        el.setParents([points[0].id, points[1].id]);
+        el.subs = {
+            midpoint: mp
+        };
+        el.inherits.push(mp);
+
+        /**
+         * The midpoint of the two defining points.
+         * @memberOf Semicircle.prototype
+         * @name midpoint
+         * @type Midpoint
+         */
+        el.midpoint = el.center = mp;
+
         return el;
     };
 
@@ -441,11 +477,11 @@ define([
      * var p2 = board.create('point', [1.0, 0.5]);
      * var p3 = board.create('point', [3.5, 1.0]);
      *
-     * var a = board.create('arc', [p1, p2, p3]);
-     * </pre><div id="87125fd4-823a-41c1-88ef-d1a1369504e3" style="width: 300px; height: 300px;"></div>
+     * var a = board.create('circumcirclearc', [p1, p2, p3]);
+     * </pre><div class="jxgbox" id="JXG87125fd4-823a-41c1-88ef-d1a1369504e3" style="width: 300px; height: 300px;"></div>
      * <script type="text/javascript">
      * (function () {
-     *   var board = JXG.JSXGraph.initBoard('87125fd4-823a-41c1-88ef-d1a1369504e3', {boundingbox: [-1, 7, 7, -1], axis: true, showcopyright: false, shownavigation: false}),
+     *   var board = JXG.JSXGraph.initBoard('JXG87125fd4-823a-41c1-88ef-d1a1369504e3', {boundingbox: [-1, 7, 7, -1], axis: true, showcopyright: false, shownavigation: false}),
      *       p1 = board.create('point', [2.0, 2.0]),
      *       p2 = board.create('point', [1.0, 0.5]),
      *       p3 = board.create('point', [3.5, 1.0]),
@@ -455,38 +491,39 @@ define([
      * </script><pre>
      */
     JXG.createCircumcircleArc = function (board, parents, attributes) {
-        var el, mp, attr;
+        var el, mp, attr, points;
 
         // We need three points
-        if ((Type.isPoint(parents[0])) && (Type.isPoint(parents[1])) && (Type.isPoint(parents[2]))) {
-            attr = Type.copyAttributes(attributes, board.options, 'circumcirclearc', 'center');
-            mp = board.create('circumcenter', [parents[0], parents[1], parents[2]], attr);
-
-            mp.dump = false;
-
-            attr = Type.copyAttributes(attributes, board.options, 'circumcirclearc');
-            attr.usedirection = true;
-            el = board.create('arc', [mp, parents[0], parents[2], parents[1]], attr);
-
-            el.elType = 'circumcirclearc';
-            el.parents = [parents[0].id, parents[1].id, parents[2].id];
-            el.subs = {
-                center: mp
-            };
-
-            /**
-             * The midpoint of the circumcircle of the three points defining the circumcircle arc.
-             * @memberOf CircumcircleArc.prototype
-             * @name center
-             * @type Circumcenter
-             */
-            el.center = mp;
-        } else {
+        points = Type.providePoints(board, parents, attributes, 'point');
+        if (points === false || points.length !== 3) {
             throw new Error("JSXGraph: create Circumcircle Arc with parent types '" +
                 (typeof parents[0]) + "' and '" + (typeof parents[1]) + "' and '" + (typeof parents[2]) + "'." +
                 "\nPossible parent types: [point,point,point]");
         }
 
+        attr = Type.copyAttributes(attributes, board.options, 'circumcirclearc', 'center');
+        mp = board.create('circumcenter', points, attr);
+        mp.dump = false;
+
+        attr = Type.copyAttributes(attributes, board.options, 'circumcirclearc');
+        attr.usedirection = true;
+        el = board.create('arc', [mp, points[0], points[2], points[1]], attr);
+
+        el.elType = 'circumcirclearc';
+        el.setParents([points[0].id, points[1].id, points[2].id]);
+        el.subs = {
+            center: mp
+        };
+        el.inherits.push(mp);
+
+        /**
+         * The midpoint of the circumcircle of the three points defining the circumcircle arc.
+         * @memberOf CircumcircleArc.prototype
+         * @name center
+         * @type Circumcenter
+         */
+        el.center = mp;
+
         return el;
     };
 
@@ -511,10 +548,10 @@ define([
      * var p3 = board.create('point', [3.5, 1.0]);
      *
      * var a = board.create('arc', [p1, p2, p3]);
-     * </pre><div id="af27ddcc-265f-428f-90dd-d31ace945800" style="width: 300px; height: 300px;"></div>
+     * </pre><div class="jxgbox" id="JXG64ba7ca2-8728-45f3-96e5-3c7a4414de2f" style="width: 300px; height: 300px;"></div>
      * <script type="text/javascript">
      * (function () {
-     *   var board = JXG.JSXGraph.initBoard('af27ddcc-265f-428f-90dd-d31ace945800', {boundingbox: [-1, 7, 7, -1], axis: true, showcopyright: false, shownavigation: false}),
+     *   var board = JXG.JSXGraph.initBoard('JXG64ba7ca2-8728-45f3-96e5-3c7a4414de2f', {boundingbox: [-1, 7, 7, -1], axis: true, showcopyright: false, shownavigation: false}),
      *       p1 = board.create('point', [2.0, 2.0]),
      *       p2 = board.create('point', [1.0, 0.5]),
      *       p3 = board.create('point', [3.5, 1.0]),
@@ -525,7 +562,7 @@ define([
      */
 
     JXG.createMinorArc = function (board, parents, attributes) {
-        attributes.type = 'minor';
+        attributes.selection = 'minor';
         return JXG.createArc(board, parents, attributes);
     };
 
@@ -536,7 +573,7 @@ define([
      * 180 degrees (pi radians). It is defined by a center, one point that
      * defines the radius, and a third point that defines the angle of the arc.
      * @pseudo
-     * @name MinorArc
+     * @name MajorArc
      * @augments Curve
      * @constructor
      * @type JXG.Curve
@@ -549,11 +586,11 @@ define([
      * var p2 = board.create('point', [1.0, 0.5]);
      * var p3 = board.create('point', [3.5, 1.0]);
      *
-     * var a = board.create('arc', [p1, p2, p3]);
-     * </pre><div id="83c6561f-7561-4047-b98d-036248a00932" style="width: 300px; height: 300px;"></div>
+     * var a = board.create('minorarc', [p1, p2, p3]);
+     * </pre><div class="jxgbox" id="JXG17a10d38-5629-40a4-b150-f41806edee9f" style="width: 300px; height: 300px;"></div>
      * <script type="text/javascript">
      * (function () {
-     *   var board = JXG.JSXGraph.initBoard('83c6561f-7561-4047-b98d-036248a00932', {boundingbox: [-1, 7, 7, -1], axis: true, showcopyright: false, shownavigation: false}),
+     *   var board = JXG.JSXGraph.initBoard('JXG17a10d38-5629-40a4-b150-f41806edee9f', {boundingbox: [-1, 7, 7, -1], axis: true, showcopyright: false, shownavigation: false}),
      *       p1 = board.create('point', [2.0, 2.0]),
      *       p2 = board.create('point', [1.0, 0.5]),
      *       p3 = board.create('point', [3.5, 1.0]),
@@ -563,7 +600,7 @@ define([
      * </script><pre>
      */
     JXG.createMajorArc = function (board, parents, attributes) {
-        attributes.type = 'major';
+        attributes.selection = 'major';
         return JXG.createArc(board, parents, attributes);
     };
 
