@@ -1,5 +1,5 @@
 /*
-    Copyright 2008-2013
+    Copyright 2008-2022
         Matthias Ehmann,
         Michael Gerhaeuser,
         Carsten Miller,
@@ -10,20 +10,20 @@
     This file is part of JSXGraph.
 
     JSXGraph is free software dual licensed under the GNU LGPL or MIT License.
-    
+
     You can redistribute it and/or modify it under the terms of the
-    
+
       * GNU Lesser General Public License as published by
         the Free Software Foundation, either version 3 of the License, or
         (at your option) any later version
       OR
       * MIT License: https://github.com/jsxgraph/jsxgraph/blob/master/LICENSE.MIT
-    
+
     JSXGraph is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU Lesser General Public License for more details.
-    
+
     You should have received a copy of the GNU Lesser General Public License and
     the MIT License along with JSXGraph. If not, see <http://www.gnu.org/licenses/>
     and <http://opensource.org/licenses/MIT/>.
@@ -50,8 +50,8 @@
  */
 
 define([
-    'jxg', 'base/constants', 'base/coords', 'math/math', 'math/numerics', 'math/geometry', 'utils/type', 'base/point', 'base/curve'
-], function (JXG, Const, Coords, Mat, Numerics, Geometry, Type, Point, Curve) {
+    'jxg', 'base/constants', 'base/coords', 'math/math', 'math/numerics', 'math/geometry', 'utils/type'
+], function (JXG, Const, Coords, Mat, Numerics, Geometry, Type) {
 
     "use strict";
 
@@ -61,7 +61,7 @@ define([
      * @pseudo
      * @description
      * @name Ellipse
-     * @augments JXG.Curve
+     * @augments Conic
      * @constructor
      * @type JXG.Curve
      * @throws {Exception} If the element cannot be constructed with the given parent objects an exception is thrown.
@@ -69,21 +69,55 @@ define([
      * numbers describing the coordinates of a point. In the latter case the point will be constructed automatically as a fixed invisible point.
      * @param {JXG.Point,array_JXG.Point,array_number,function} point1,point2,number Parent elements can be two elements either of type {@link JXG.Point} or array of
      * numbers describing the coordinates of a point. The third parameter is a number/function which defines the length of the major axis
-     * Optional parameters four and five are numbers which define the curve length (e.g. start/end). Default values are -pi and pi.
+     * @param {Number} start (Optional) parameter of the curve start, default: 0.
+     * @param {Number} end (Optional) parameter for the curve end, default: 2&pi;.
      * @example
      * // Create an Ellipse by three points
      * var A = board.create('point', [-1,4]);
      * var B = board.create('point', [-1,-4]);
      * var C = board.create('point', [1,1]);
      * var el = board.create('ellipse',[A,B,C]);
-     * </pre><div id="a4d7fb6f-8708-4e45-87f2-2379ae2bd2c0" style="width: 300px; height: 300px;"></div>
+     * </pre><div class="jxgbox" id="JXGa4d7fb6f-8708-4e45-87f2-2379ae2bd2c0" style="width: 300px; height: 300px;"></div>
      * <script type="text/javascript">
-     *   var glex1_board = JXG.JSXGraph.initBoard('a4d7fb6f-8708-4e45-87f2-2379ae2bd2c0', {boundingbox:[-6,6,6,-6], keepaspectratio:true, showcopyright: false, shownavigation: false});
+     *   (function() {
+     *   var glex1_board = JXG.JSXGraph.initBoard('JXGa4d7fb6f-8708-4e45-87f2-2379ae2bd2c0', {boundingbox:[-6,6,6,-6], keepaspectratio:true, showcopyright: false, shownavigation: false});
      *   var A = glex1_board.create('point', [-1,4]);
      *   var B = glex1_board.create('point', [-1,-4]);
      *   var C = glex1_board.create('point', [1,1]);
      *   var el = glex1_board.create('ellipse',[A,B,C]);
+     * })();
      * </script><pre>
+     *
+     * @example
+     * // Create an elliptical arc
+     * var p1 = board.create('point', [-1, 2]);
+     * var p2 = board.create('point', [ 1, 2]);
+     * var p3 = board.create('point', [0, 3]);
+     * 
+     * var ell = board.create('ellipse', [
+     *   p1, p2, p3, 0, Math.PI], {
+     *   lastArrow: {type: 7}
+     * });
+     * 
+     * </pre><div id="JXG950f7c07-27a4-4c67-9505-c73c22ce9345" class="jxgbox" style="width: 300px; height: 300px;"></div>
+     * <script type="text/javascript">
+     *     (function() {
+     *         var board = JXG.JSXGraph.initBoard('JXG950f7c07-27a4-4c67-9505-c73c22ce9345',
+     *             {boundingbox: [-8, 8, 8,-8], axis: true, showcopyright: false, shownavigation: false});
+     *     var p1 = board.create('point', [-1, 2]);
+     *     var p2 = board.create('point', [ 1, 2]);
+     *     var p3 = board.create('point', [0, 3]);
+     *     
+     *     var ell = board.create('ellipse', [
+     *       p1, p2, p3, 0, Math.PI], {
+     *       lastArrow: {type: 7}
+     *     });
+     * 
+     *     })();
+     * 
+     * </script><pre>
+     * 
+*
      */
     JXG.createEllipse = function (board, parents, attributes) {
         var polarForm, curve, M, C, majorAxis, i,
@@ -91,6 +125,7 @@ define([
             // focus 1 and focus 2
             F = [],
             attr_foci = Type.copyAttributes(attributes, board.options, 'conic', 'foci'),
+            attr_center = Type.copyAttributes(attributes, board.options, 'conic', 'center'),
             attr_curve = Type.copyAttributes(attributes, board.options, 'conic');
 
         // The foci and the third point are either points or coordinate arrays.
@@ -102,7 +137,7 @@ define([
             } else if (Type.isPoint(parents[i])) {
                 F[i] = board.select(parents[i]);
             // given by function
-            } else if ((typeof parents[i] === 'function') && (parents[i]().elementClass === Const.OBJECT_CLASS_POINT)) {
+            } else if (Type.isFunction(parents[i]) && Type.isPoint(parents[i]()) ) {
                 F[i] = parents[i]();
             // focus i given by point name
             } else if (Type.isString(parents[i])) {
@@ -117,7 +152,7 @@ define([
         // length of major axis
         if (Type.isNumber(parents[2])) {
             majorAxis = Type.createFunction(parents[2], board);
-        } else if ((typeof parents[2] === 'function') && (Type.isNumber(parents[2]()))) {
+        } else if (Type.isFunction(parents[2]) && Type.isNumber(parents[2]())) {
             majorAxis = parents[2];
         } else {
             // point on ellipse
@@ -127,7 +162,7 @@ define([
             } else if (parents[2].length > 1) {
                 C = board.create('point', parents[2], attr_foci);
             // given by function
-            } else if ((typeof parents[2] === 'function') && (parents[2]().elementClass === Const.OBJECT_CLASS_POINT)) {
+            } else if (Type.isFunction(parents[2]) && Type.isPoint(parents[2]()) ) {
                 C = parents[2]();
             // focus i given by point name
             } else if (Type.isString(parents[2])) {
@@ -160,7 +195,7 @@ define([
             function () {
                 return (F[0].Y() + F[1].Y()) * 0.5;
             }
-        ], attr_foci);
+        ], attr_center);
 
         curve = board.create('curve', [
             function (x) {
@@ -224,11 +259,18 @@ define([
             return F[0].Y() + Math.sin(beta + phi) * b;
         };
 
-        curve.midpoint = M;
+        curve.midpoint = curve.center = M;
         curve.type = Const.OBJECT_TYPE_CONIC;
+        curve.subs = {
+                center: curve.center
+            };
+        curve.inherits.push(curve.center, F[0], F[1]);
+        if (Type.isPoint(C)) {
+            curve.inherits.push(C);
+        }
 
         /**
-         * Checks whether (x,y) is near the ellipse line or inside of the ellipse 
+         * Checks whether (x,y) is near the ellipse line or inside of the ellipse
          * (in case JXG.Options.conic#hasInnerPoints is true).
          * @param {Number} x Coordinate in x direction, screen coordinates.
          * @param {Number} y Coordinate in y direction, screen coordinates.
@@ -238,7 +280,7 @@ define([
         curve.hasPoint =  function (x, y) {
             var ac, bc, r, p, dist;
 
-            if (this.visProp.hasinnerpoints) {
+            if (Type.evaluate(this.visProp.hasinnerpoints)) {
                 ac = F[0].coords;
                 bc = F[1].coords;
                 r = this.majorAxis();
@@ -260,12 +302,7 @@ define([
         if (Type.isPoint(C)) {
             C.addChild(curve);
         }
-        curve.parents = [];
-        for (i = 0; i < parents.length; i++) {
-            if (parents[i].id) {
-                curve.parents.push(parents[i].id);
-            }
-        }
+        curve.setParents(parents);
 
         return curve;
     };
@@ -276,7 +313,7 @@ define([
      * @pseudo
      * @description
      * @name Hyperbola
-     * @augments JXG.Curve
+     * @augments Conic
      * @constructor
      * @type JXG.Curve
      * @throws {Exception} If the element cannot be constructed with the given parent objects an exception is thrown.
@@ -284,20 +321,23 @@ define([
      * numbers describing the coordinates of a point. In the latter case the point will be constructed automatically as a fixed invisible point.
      * @param {JXG.Point,array_JXG.Point,array_number,function} point1,point2,number Parent elements can be two elements either of type {@link JXG.Point} or array of
      * numbers describing the coordinates of a point. The third parameter is a number/function which defines the length of the major axis
-     * Optional parameters four and five are numbers which define the curve length (e.g. start/end). Default values are -pi and pi.
+     * @param {Number} start (Optional) parameter of the curve start, default: -&pi;.
+     * @param {Number} end (Optional) parameter for the curve end, default: &pi;.
      * @example
      * // Create an Hyperbola by three points
      * var A = board.create('point', [-1,4]);
      * var B = board.create('point', [-1,-4]);
      * var C = board.create('point', [1,1]);
      * var el = board.create('hyperbola',[A,B,C]);
-     * </pre><div id="cf99049d-a3fe-407f-b936-27d76550f8c4" style="width: 300px; height: 300px;"></div>
+     * </pre><div class="jxgbox" id="JXGcf99049d-a3fe-407f-b936-27d76550f8c4" style="width: 300px; height: 300px;"></div>
      * <script type="text/javascript">
-     *   var glex1_board = JXG.JSXGraph.initBoard('cf99049d-a3fe-407f-b936-27d76550f8c4', {boundingbox:[-6,6,6,-6], keepaspectratio:true, showcopyright: false, shownavigation: false});
+     *   (function(){
+     *   var glex1_board = JXG.JSXGraph.initBoard('JXGcf99049d-a3fe-407f-b936-27d76550f8c4', {boundingbox:[-6,6,6,-6], keepaspectratio:true, showcopyright: false, shownavigation: false});
      *   var A = glex1_board.create('point', [-1,4]);
      *   var B = glex1_board.create('point', [-1,-4]);
      *   var C = glex1_board.create('point', [1,1]);
      *   var el = glex1_board.create('hyperbola',[A,B,C]);
+     * })();
      * </script><pre>
      */
     JXG.createHyperbola = function (board, parents, attributes) {
@@ -305,6 +345,7 @@ define([
             // focus 1 and focus 2
             F = [],
             attr_foci = Type.copyAttributes(attributes, board.options, 'conic', 'foci'),
+            attr_center = Type.copyAttributes(attributes, board.options, 'conic', 'center'),
             attr_curve = Type.copyAttributes(attributes, board.options, 'conic');
 
         // The foci and the third point are either points or coordinate arrays.
@@ -316,7 +357,7 @@ define([
             } else if (Type.isPoint(parents[i])) {
                 F[i] = board.select(parents[i]);
             // given by function
-            } else if ((typeof parents[i] === 'function') && (parents[i]().elementClass === Const.OBJECT_CLASS_POINT)) {
+            } else if (Type.isFunction(parents[i]) && Type.isPoint(parents[i]()) ) {
                 F[i] = parents[i]();
             // focus i given by point name
             } else if (Type.isString(parents[i])) {
@@ -331,7 +372,7 @@ define([
         // length of major axis
         if (Type.isNumber(parents[2])) {
             majorAxis = Type.createFunction(parents[2], board);
-        } else if ((typeof parents[2] === 'function') && (Type.isNumber(parents[2]()))) {
+        } else if (Type.isFunction(parents[2]) && Type.isNumber(parents[2]())) {
             majorAxis = parents[2];
         } else {
             // point on ellipse
@@ -341,7 +382,7 @@ define([
             } else if (parents[2].length > 1) {
                 C = board.create('point', parents[2], attr_foci);
             // given by function
-            } else if ((typeof parents[2] === 'function') && (parents[2]().elementClass === Const.OBJECT_CLASS_POINT)) {
+            } else if (Type.isFunction(parents[2]) && Type.isPoint(parents[2]())) {
                 C = parents[2]();
             // focus i given by point name
             } else if (Type.isString(parents[2])) {
@@ -374,7 +415,7 @@ define([
             function () {
                 return (F[0].Y() + F[1].Y()) * 0.5;
             }
-        ], attr_foci);
+        ], attr_center);
 
         curve = board.create('curve', [
             function (x) {
@@ -412,7 +453,7 @@ define([
 
         /** @ignore */
         curve.X = function (phi, suspendUpdate) {
-            var r = this.majorAxis(),
+            var r = majorAxis(),
                 c = F[1].Dist(F[0]),
                 b = 0.5 * (c * c - r * r) / (c * Math.cos(phi) + r),
                 beta = Math.atan2(F[1].Y() - F[0].Y(), F[1].X() - F[0].X());
@@ -426,7 +467,7 @@ define([
 
         /** @ignore */
         curve.Y = function (phi, suspendUpdate) {
-            var r = this.majorAxis(),
+            var r = majorAxis(),
                 c = F[1].Dist(F[0]),
                 b = 0.5 * (c * c - r * r) / (c * Math.cos(phi) + r),
                 beta = Math.atan2(F[1].Y() - F[0].Y(), F[1].X() - F[0].X());
@@ -434,7 +475,14 @@ define([
             return F[0].Y() + Math.sin(beta + phi) * b;
         };
 
-        curve.midpoint = M;
+        curve.midpoint = curve.center = M;
+        curve.subs = {
+            center: curve.center
+        };
+        curve.inherits.push(curve.center, F[0], F[1]);
+        if (Type.isPoint(C)) {
+            curve.inherits.push(C);
+        }
         curve.type = Const.OBJECT_TYPE_CONIC;
 
         M.addChild(curve);
@@ -446,12 +494,7 @@ define([
         if (Type.isPoint(C)) {
             C.addChild(curve);
         }
-        curve.parents = [];
-        for (i = 0; i < parents.length; i++) {
-            if (parents[i].id) {
-                curve.parents.push(parents[i].id);
-            }
-        }
+        curve.setParents(parents);
 
         return curve;
     };
@@ -461,11 +504,11 @@ define([
      * @pseudo
      * @description
      * @name Parabola
-     * @augments JXG.Curve
+     * @augments Conic
      * @constructor
      * @type JXG.Curve
      * @throws {Exception} If the element cannot be constructed with the given parent objects an exception is thrown.
-     * @param {JXG.Point,array_JXG.Line} point,line Parent elements are a point and a line.
+     * @param {JXG.Point,array_JXG.Line} point,line Parent elements are a point and a line or a pair of coordinates.
      * Optional parameters three and four are numbers which define the curve length (e.g. start/end). Default values are -pi and pi.
      * @example
      * // Create a parabola by a point C and a line l.
@@ -474,35 +517,54 @@ define([
      * var l = board.create('line', [A,B]);
      * var C = board.create('point', [1,1]);
      * var el = board.create('parabola',[C,l]);
-     * </pre><div id="524d1aae-217d-44d4-ac58-a19c7ab1de36" style="width: 300px; height: 300px;"></div>
+     * </pre><div class="jxgbox" id="JXG524d1aae-217d-44d4-ac58-a19c7ab1de36" style="width: 300px; height: 300px;"></div>
      * <script type="text/javascript">
-     *   var glex1_board = JXG.JSXGraph.initBoard('524d1aae-217d-44d4-ac58-a19c7ab1de36', {boundingbox:[-6,6,6,-6], keepaspectratio:true, showcopyright: false, shownavigation: false});
+     * (function() {
+     *   var glex1_board = JXG.JSXGraph.initBoard('JXG524d1aae-217d-44d4-ac58-a19c7ab1de36', {boundingbox:[-6,6,6,-6], keepaspectratio:true, showcopyright: false, shownavigation: false});
      *   var A = glex1_board.create('point', [-1,4]);
      *   var B = glex1_board.create('point', [-1,-4]);
      *   var l = glex1_board.create('line', [A,B]);
      *   var C = glex1_board.create('point', [1,1]);
      *   var el = glex1_board.create('parabola',[C,l]);
+     * })();
      * </script><pre>
+     *
+     * @example
+     * var par = board.create('parabola',[[3.25, 0], [[0.25, 1],[0.25, 0]]]);
+     *
+     * </pre><div id="JXG09252542-b77a-4990-a109-66ffb649a472" class="jxgbox" style="width: 300px; height: 300px;"></div>
+     * <script type="text/javascript">
+     *     (function() {
+     *         var board = JXG.JSXGraph.initBoard('JXG09252542-b77a-4990-a109-66ffb649a472',
+     *             {boundingbox: [-8, 8, 8,-8], axis: true, showcopyright: false, shownavigation: false});
+     *     var par = board.create('parabola',[[3.25, 0], [[0.25, 1],[0.25, 0]]]);
+     *
+     *     })();
+     *
+     * </script><pre>
+     *
      */
     JXG.createParabola = function (board, parents, attributes) {
-        var polarForm, curve, M, i,
+        var polarForm, curve, M,
             // focus
             F1 = parents[0],
             // directrix
             l = parents[1],
             attr_foci = Type.copyAttributes(attributes, board.options, 'conic', 'foci'),
-            attr_curve = Type.copyAttributes(attributes, board.options, 'conic');
+            attr_center = Type.copyAttributes(attributes, board.options, 'conic', 'center'),
+            attr_curve = Type.copyAttributes(attributes, board.options, 'conic'),
+            attr_line;
 
         // focus 1 given by coordinates
         if (parents[0].length > 1) {
             F1 = board.create('point', parents[0], attr_foci);
-        // focus i given by point
+        // focus 1 given by point
         } else if (Type.isPoint(parents[0])) {
             F1 = board.select(parents[0]);
         // given by function
-        } else if ((typeof parents[0] === 'function') && (parents[0]().elementClass === Const.OBJECT_CLASS_POINT)) {
+        } else if (Type.isFunction(parents[0]) && Type.isPoint(parents[0]()) ) {
             F1 = parents[0]();
-        // focus i given by point name
+        // focus 1 given by point name
         } else if (Type.isString(parents[0])) {
             F1 = board.select(parents[0]);
         } else {
@@ -511,24 +573,32 @@ define([
                 "\nPossible parent types: [point,line]");
         }
 
+        // Create line if given as array of two points.
+        if (Type.isArray(l) && l.length === 2) {
+            attr_line = Type.copyAttributes(attributes, board.options, 'conic', 'line');
+            l = board.create('line', l, attr_line);
+        }
+
         // to
         if (!Type.exists(parents[3])) {
-            parents[3] = 10;
+            parents[3] = 2 * Math.PI;
         }
 
         // from
         if (!Type.exists(parents[2])) {
-            parents[2] = -10;
+            parents[2] = 0;
         }
 
         M = board.create('point', [
             function () {
+                /*
                 var v = [0, l.stdform[1], l.stdform[2]];
-
                 v = Mat.crossProduct(v, F1.coords.usrCoords);
                 return Geometry.meetLineLine(v, l.stdform, 0, board).usrCoords;
+                */
+                return Geometry.projectPointToLine(F1, l, board).usrCoords;
             }
-        ], attr_foci);
+        ], attr_center);
 
         /** @ignore */
         curve = board.create('curve', [
@@ -539,6 +609,12 @@ define([
                 return 0;
             }, parents[2], parents[3]], attr_curve);
 
+        curve.midpoint = curve.center = M;
+        curve.subs = {
+            center: curve.center
+        };
+        curve.inherits.push(curve.center);
+
         /** @ignore */
         polarForm = function (t, suspendUpdate) {
             var a, b, c, ab, px, py;
@@ -568,6 +644,12 @@ define([
                 B = l.point2.coords.usrCoords,
                 M = F1.coords.usrCoords;
 
+            // Handle the case if one of the two defining points of the line is an ideal point
+            if (A[0] === 0) {
+                A = [1, B[1] + l.stdform[2], B[2] - l.stdform[1]];
+            } else if (B[0] === 0) {
+                B = [1, A[1] + l.stdform[2], A[2] - l.stdform[1]];
+            }
             det = ((B[1] - A[1]) * (M[2] - A[2]) - (B[2] - A[2]) * (M[1] - A[1]) >= 0) ? 1 : -1;
             a = det * d / (1 - Math.sin(phi));
 
@@ -587,6 +669,12 @@ define([
                 B = l.point2.coords.usrCoords,
                 M = F1.coords.usrCoords;
 
+            // Handle the case if one of the two defining points of the line is an ideal point
+            if (A[0] === 0) {
+                A = [1, B[1] + l.stdform[2], B[2] - l.stdform[1]];
+            } else if (B[0] === 0) {
+                B = [1, A[1] + l.stdform[2], A[2] - l.stdform[1]];
+            }
             det = ((B[1] - A[1]) * (M[2] - A[2]) - (B[2] - A[2]) * (M[1] - A[1]) >= 0) ? 1 : -1;
             a = det * d / (1 - Math.sin(phi));
 
@@ -598,23 +686,24 @@ define([
 
         if (Type.isPoint(F1)) {
             F1.addChild(curve);
+            curve.inherits.push(F1);
         }
 
         l.addChild(curve);
-        curve.parents = [];
-
-        for (i = 0; i < parents.length; i++) {
-            if (parents[i].id) {
-                curve.parents.push(parents[i].id);
-            }
-        }
+        curve.setParents(parents);
 
         return curve;
     };
 
     /**
      *
-     * @class This element is used to provide a constructor for a generic conic section uniquely defined by five points.
+     * @class This element is used to provide a constructor for a generic conic section uniquely defined by five points or
+     * a conic defined by the coefficients of the equation
+     * <p><i>Ax<sup>2</sup>+ Bxy+Cy<sup>2</sup> + Dx + Ey + F = 0</i></p>.
+     * Then the parameters are as follows:
+     * <pre>
+     *     board.create('conic', [A, C, F, B/2, D/2, E/2]);
+     * </pre>
      * @pseudo
      * @description
      * @name Conic
@@ -623,7 +712,7 @@ define([
      * @type JXG.Conic
      * @throws {Exception} If the element cannot be constructed with the given parent objects an exception is thrown.
      * @param {JXG.Point,Array_JXG.Point,Array_JXG.Point,Array_JXG.Point,Array_JXG.Point,Array} a,b,c,d,e Parent elements are five points.
-     * @param {Number_Number_Number_Number_Number_Number} a_00,a_11,a_22,a_01,a_12,a_22 6 numbers
+     * @param {Number_Number_Number_Number_Number_Number} a_00,a_11,a_22,a_01,a_02,a_12 6 numbers, i.e. A, C, F, B/2, D/2, E/2
      * @example
      * // Create a conic section through the points A, B, C, D, and E.
      *  var A = board.create('point', [1,5]);
@@ -632,16 +721,34 @@ define([
      *  var D = board.create('point', [0,0]);
      *  var E = board.create('point', [-1,5]);
      *  var conic = board.create('conic',[A,B,C,D,E]);
-     * </pre><div id="2d79bd6a-db9b-423c-9cba-2497f0b06320" style="width: 300px; height: 300px;"></div>
+     * </pre><div class="jxgbox" id="JXG2d79bd6a-db9b-423c-9cba-2497f0b06320" style="width: 300px; height: 300px;"></div>
      * <script type="text/javascript">
-     *   var glex1_board = JXG.JSXGraph.initBoard('2d79bd6a-db9b-423c-9cba-2497f0b06320', {boundingbox:[-6,6,6,-6], keepaspectratio:true, showcopyright: false, shownavigation: false});
+     * (function(){
+     *   var glex1_board = JXG.JSXGraph.initBoard('JXG2d79bd6a-db9b-423c-9cba-2497f0b06320', {boundingbox:[-6,6,6,-6], keepaspectratio:true, showcopyright: false, shownavigation: false});
      *   var A = glex1_board.create('point', [1,5]);
      *   var B = glex1_board.create('point', [1,2]);
      *   var C = glex1_board.create('point', [2,0]);
      *   var D = glex1_board.create('point', [0,0]);
      *   var E = glex1_board.create('point', [-1,5]);
      *   var conic = glex1_board.create('conic',[A,B,C,D,E]);
+     * })();
      * </script><pre>
+     *
+     * @example
+     * // Parameters: A, C, F, B/2, D/2, E/2
+     * var conic = board.create('conic', [1, 2, -4, 0, 0, 0]s);
+     *
+     * </pre><div id="JXG8576a04a-52d8-4a7e-8d54-e32443910b97" class="jxgbox" style="width: 300px; height: 300px;"></div>
+     * <script type="text/javascript">
+     *     (function() {
+     *         var board = JXG.JSXGraph.initBoard('JXG8576a04a-52d8-4a7e-8d54-e32443910b97',
+     *             {boundingbox: [-8, 8, 8,-8], axis: true, showcopyright: false, shownavigation: false});
+     *     // Parameters: A, C, F, B/2, D/2, E/2
+     *     var conic = board.create('conic', [1, 2, -4, 0, 0, 0]s);
+     *     })();
+     *
+     * </script><pre>
+     *
      */
     JXG.createConic = function (board, parents, attributes) {
         var polarForm, curve, fitConic, degconic, sym,
@@ -659,7 +766,8 @@ define([
             ],
             points = [],
             p = [],
-            attr_foci = Type.copyAttributes(attributes, board.options, 'conic', 'foci'),
+            attr_point = Type.copyAttributes(attributes, board.options, 'conic', 'point'),
+            attr_center = Type.copyAttributes(attributes, board.options, 'conic', 'center'),
             attr_curve = Type.copyAttributes(attributes, board.options, 'conic');
 
         if (parents.length === 5) {
@@ -674,12 +782,12 @@ define([
             for (i = 0; i < 5; i++) {
                 // point i given by coordinates
                 if (parents[i].length > 1) {
-                    points[i] = board.create('point', parents[i], attr_foci);
+                    points[i] = board.create('point', parents[i], attr_point);
                 // point i given by point
                 } else if (Type.isPoint(parents[i])) {
                     points[i] = board.select(parents[i]);
                 // given by function
-                } else if ((typeof parents[i] === 'function') && (parents[i]().elementClass === Const.OBJECT_CLASS_POINT)) {
+                } else if (Type.isFunction(parents[i]) && Type.isPoint(parents[i]()) ) {
                     points[i] = parents[i]();
                 // point i given by point name
                 } else if (Type.isString(parents[i])) {
@@ -695,10 +803,6 @@ define([
              *   [A3,A1,A5],
              *   [A4,A5,A2]].
              * Our notation (z,x,y):
-             *  [[-A2   , A4*2.0, A5*0.5],
-             *   [A4*2.0,    -A0, A3*0.5],
-             *   [A5*0.5, A3*0.5,    -A1]]
-             * New: (z,x,y):
              *  [[A2, A4, A5],
              *   [A4, A0, A3],
              *   [A5, A3, A1]]
@@ -847,7 +951,7 @@ define([
                 v = Mat.matVecMult(rotationMatrix, [Math.sin(phi) / c, Math.cos(phi) / a, 1 / b]);
             }
 
-            if (JXG.exists(v)) {
+            if (Type.exists(v)) {
                 // Normalize
                 v[1] /= v[0];
                 v[2] /= v[0];
@@ -880,9 +984,15 @@ define([
                     m[0][1] * m[1][2] - m[1][1] * m[0][2]
                 ];
             }
-        ], attr_foci);
+        ], attr_center);
 
         curve.type = Const.OBJECT_TYPE_CONIC;
+        curve.center = curve.midpoint;
+        curve.subs = {
+            center: curve.center
+        };
+        curve.inherits.push(curve.center);
+        curve.inherits = curve.inherits.concat(points);
 
         if (givenByPoints) {
             for (i = 0; i < 5; i++) {
@@ -890,14 +1000,9 @@ define([
                     points[i].addChild(curve);
                 }
             }
-            curve.parents = [];
-            for (i = 0; i < parents.length; i++) {
-                if (parents[i].id) {
-                    curve.parents.push(parents[i].id);
-                }
-            }
+            curve.setParents(parents);
         }
-        curve.addChild(curve.midpoint);
+        curve.addChild(curve.center);
 
         return curve;
     };
