@@ -1,5 +1,5 @@
 /*
-    Copyright 2008-2013
+    Copyright 2008-2022
         Matthias Ehmann,
         Michael Gerhaeuser,
         Carsten Miller,
@@ -10,20 +10,20 @@
     This file is part of JSXGraph.
 
     JSXGraph is free software dual licensed under the GNU LGPL or MIT License.
-    
+
     You can redistribute it and/or modify it under the terms of the
-    
+
       * GNU Lesser General Public License as published by
         the Free Software Foundation, either version 3 of the License, or
         (at your option) any later version
       OR
       * MIT License: https://github.com/jsxgraph/jsxgraph/blob/master/LICENSE.MIT
-    
+
     JSXGraph is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU Lesser General Public License for more details.
-    
+
     You should have received a copy of the GNU Lesser General Public License and
     the MIT License along with JSXGraph. If not, see <http://www.gnu.org/licenses/>
     and <http://opensource.org/licenses/MIT/>.
@@ -40,10 +40,9 @@
 /**
  * @fileoverview In this file the namespace JXG.Math is defined, which is the base namespace
  * for namespaces like Math.Numerics, Math.Algebra, Math.Statistics etc.
- * @author graphjs
  */
 
-define(['jxg'], function (JXG) {
+define(['jxg', 'utils/type'], function (JXG, Type) {
 
     "use strict";
 
@@ -89,13 +88,28 @@ define(['jxg'], function (JXG) {
         /**
          * eps defines the closeness to zero. If the absolute value of a given number is smaller
          * than eps, it is considered to be equal to zero.
-         * @type number
+         * @type Number
          */
         eps: 0.000001,
 
+        /**
+         * Determine the relative difference between two numbers.
+         * @param  {Number} a First number
+         * @param  {Number} b Second number
+         * @returns {Number}  Relative difference between a and b: |a-b| / max(|a|, |b|)
+         */
+        relDif: function(a, b) {
+            var c = Math.abs(a),
+                d = Math.abs(b);
+
+            d = Math.max(c, d);
+
+            return (d === 0.0) ? 0.0 : Math.abs(a - b) / d;
+        },
+
         /**
          * The JavaScript implementation of the % operator returns the symmetric modulo.
-         * They are both identical if a >= 0 and m >= 0 but the results differ if a or m < 0.
+         * mod and "%" are both identical if a >= 0 and m >= 0 but the results differ if a or m < 0.
          * @param {Number} a
          * @param {Number} m
          * @returns {Number} Mathematical modulo <tt>a mod m</tt>
@@ -403,7 +417,7 @@ define(['jxg'], function (JXG) {
             var i,
                 s = 0;
 
-            if ((n === undef) || (typeof n !== 'number')) {
+            if (n === undef || !Type.isNumber(n)) {
                 n = a.length;
             }
 
@@ -431,6 +445,27 @@ define(['jxg'], function (JXG) {
                 c1[0] * c2[1] - c1[1] * c2[0]];
         },
 
+        /**
+         * Euclidean norm of a vector.
+         *
+         * @param {Array} a Array containing a vector.
+         * @param {Number} n (Optional) length of the array.
+         * @returns {Number} Euclidean norm of the vector.
+         */
+        norm: function(a, n) {
+            var i, sum = 0.0;
+
+            if (n === undef || !Type.isNumber(n)) {
+                n = a.length;
+            }
+
+            for (i = 0; i < n; i++) {
+                sum += a[i] * a[i];
+            }
+
+            return Math.sqrt(sum);
+        },
+
         /**
          * Compute the factorial of a positive integer. If a non-integer value
          * is given, the fraction will be ignored.
@@ -485,22 +520,114 @@ define(['jxg'], function (JXG) {
 
         /**
          * Calculates the cosine hyperbolicus of x.
+         * @function
          * @param {Number} x The number the cosine hyperbolicus will be calculated of.
          * @returns {Number} Cosine hyperbolicus of the given value.
          */
-        cosh: function (x) {
+        cosh: Math.cosh || function (x) {
             return (Math.exp(x) + Math.exp(-x)) * 0.5;
         },
 
         /**
          * Sine hyperbolicus of x.
+         * @function
          * @param {Number} x The number the sine hyperbolicus will be calculated of.
          * @returns {Number} Sine hyperbolicus of the given value.
          */
-        sinh: function (x) {
+        sinh: Math.sinh || function (x) {
             return (Math.exp(x) - Math.exp(-x)) * 0.5;
         },
 
+        /**
+         * Hyperbolic arc-cosine of a number.
+         *
+         * @param {Number} x
+         * @returns {Number}
+         */
+        acosh: Math.acosh || function(x) {
+            return Math.log(x + Math.sqrt(x * x - 1));
+        },
+
+        /**
+         * Hyperbolic arcsine of a number
+         * @param {Number} x
+         * @returns {Number}
+         */
+        asinh: Math.asinh || function(x) {
+            if (x === -Infinity) {
+                return x;
+            }
+            return Math.log(x + Math.sqrt(x * x + 1));
+        },
+
+        /**
+         * Computes the cotangent of x.
+         * @function
+         * @param {Number} x The number the cotangent will be calculated of.
+         * @returns {Number} Cotangent of the given value.
+         */
+        cot: function (x) {
+            return 1 / Math.tan(x);
+        },
+
+        /**
+         * Computes the inverse cotangent of x.
+         * @param {Number} x The number the inverse cotangent will be calculated of.
+         * @returns {Number} Inverse cotangent of the given value.
+         */
+        acot: function (x) {
+            return ((x >= 0) ? (0.5) : (-0.5)) * Math.PI - Math.atan(x);
+        },
+
+        /**
+         * Compute n-th real root of a real number. n must be strictly positive integer.
+         * If n is odd, the real n-th root exists and is negative.
+         * For n even, for negative valuees of x NaN is returned
+         * @param  {Number} x radicand. Must be non-negative, if n even.
+         * @param  {Number} n index of the root. must be strictly positive integer.
+         * @returns {Number} returns real root or NaN
+         *
+         * @example
+         * nthroot(16, 4): 2
+         * nthroot(-27, 3): -3
+         * nthroot(-4, 2): NaN
+         */
+        nthroot: function(x, n) {
+            var inv = 1 / n;
+
+            if (n <= 0 || Math.floor(n) !== n) {
+                return NaN;
+            }
+
+            if (x === 0.0) {
+                return 0.0;
+            }
+
+            if (x > 0) {
+                return Math.exp(inv * Math.log(x));
+            }
+
+            // From here on, x is negative
+            if (n % 2 === 1) {
+                return -Math.exp(inv * Math.log(-x));
+            }
+
+            // x negative, even root
+            return NaN;
+        },
+
+        /**
+         * Computes cube root of real number
+         * Polyfill for Math.cbrt().
+         *
+         * @function
+         * @param  {Number} x Radicand
+         * @returns {Number} Cube root of x.
+         */
+        cbrt: Math.cbrt || function(x) {
+            return this.nthroot(x, 3);
+        },
+
         /**
          * Compute base to the power of exponent.
          * @param {Number} base
@@ -512,26 +639,100 @@ define(['jxg'], function (JXG) {
                 if (exponent === 0) {
                     return 1;
                 }
-
                 return 0;
             }
 
+            // exponent is an integer
             if (Math.floor(exponent) === exponent) {
-                // a is an integer
                 return Math.pow(base, exponent);
             }
 
-            // a is not an integer
+            // exponent is not an integer
             if (base > 0) {
-                return Math.exp(exponent * Math.log(Math.abs(base)));
+                return Math.exp(exponent * Math.log(base));
             }
 
             return NaN;
         },
 
+        /**
+         * Compute base to the power of the rational exponent m / n.
+         * This function first reduces the fraction m/n and then computes
+         * JXG.Math.pow(base, m/n).
+         *
+         * This function is necessary to have the same results for e.g.
+         * (-8)^(1/3) = (-8)^(2/6) = -2
+         * @param {Number} base
+         * @param {Number} m numerator of exponent
+         * @param {Number} n denominator of exponent
+         * @returns {Number} base to the power of exponent.
+         */
+        ratpow: function(base, m, n) {
+            var g;
+            if (m === 0) {
+                return 1;
+            }
+            if (n === 0) {
+                return NaN;
+            }
+
+            g = this.gcd(m, n);
+            return this.nthroot(this.pow(base, m / g), n / g);
+        },
+
+        /**
+         * Logarithm to base 10.
+         * @param {Number} x
+         * @returns {Number} log10(x) Logarithm of x to base 10.
+         */
+        log10: function (x) {
+            return Math.log(x) / Math.log(10.0);
+        },
+
+        /**
+         * Logarithm to base 2.
+         * @param {Number} x
+         * @returns {Number} log2(x) Logarithm of x to base 2.
+         */
+        log2: function (x) {
+            return Math.log(x) / Math.log(2.0);
+        },
+
+        /**
+         * Logarithm to arbitrary base b. If b is not given, natural log is taken, i.e. b = e.
+         * @param {Number} x
+         * @param {Number} b base
+         * @returns {Number} log(x, b) Logarithm of x to base b, that is log(x)/log(b).
+         */
+        log: function (x, b) {
+            if (b !== undefined && Type.isNumber(b)) {
+                return Math.log(x) / Math.log(b);
+            }
+
+            return Math.log(x);
+        },
+
+        /**
+         * The sign() function returns the sign of a number, indicating whether the number is positive, negative or zero.
+         *
+         * @function
+         * @param  {Number} x A Number
+         * @returns {[type]}  This function has 5 kinds of return values,
+         *    1, -1, 0, -0, NaN, which represent "positive number", "negative number", "positive zero", "negative zero"
+         *    and NaN respectively.
+         */
+        sign: Math.sign || function(x) {
+            x = +x; // convert to a number
+            if (x === 0 || isNaN(x)) {
+                return x;
+            }
+            return x > 0 ? 1 : -1;
+        },
+
         /**
          * A square & multiply algorithm to compute base to the power of exponent.
          * Implementated by Wolfgang Riedl.
+         *
          * @param {Number} base
          * @param {Number} exponent
          * @returns {Number} Base to the power of exponent
@@ -563,6 +764,228 @@ define(['jxg'], function (JXG) {
             return this.pow(base, exponent);
         },
 
+        /**
+         * Greatest common divisor (gcd) of two numbers.
+         * @see <a href="http://rosettacode.org/wiki/Greatest_common_divisor#JavaScript">rosettacode.org</a>
+         *
+         * @param  {Number} a First number
+         * @param  {Number} b Second number
+         * @returns {Number}   gcd(a, b) if a and b are numbers, NaN else.
+         */
+        gcd: function (a, b) {
+            a = Math.abs(a);
+            b = Math.abs(b);
+
+            if (!(Type.isNumber(a) && Type.isNumber(b))) {
+                return NaN;
+            }
+            if (b > a) {
+                var temp = a;
+                a = b;
+                b = temp;
+            }
+
+            while (true) {
+                a %= b;
+                if (a === 0) { return b; }
+                b %= a;
+                if (b === 0) { return a; }
+            }
+        },
+
+        /**
+         * Least common multiple (lcm) of two numbers.
+         *
+         * @param  {Number} a First number
+         * @param  {Number} b Second number
+         * @returns {Number}   lcm(a, b) if a and b are numbers, NaN else.
+         */
+        lcm: function (a, b) {
+            var ret;
+
+            if (!(Type.isNumber(a) && Type.isNumber(b))) {
+                return NaN;
+            }
+
+            ret = a * b;
+            if (ret !== 0) {
+                return ret / this.gcd(a, b);
+            }
+
+            return 0;
+        },
+
+        /**
+         *  Error function, see {@link https://en.wikipedia.org/wiki/Error_function}.
+         *
+         * @see JXG.Math.PropFunc.erf
+         * @param  {Number} x
+         * @returns {Number}
+         */
+        erf: function(x) {
+            return this.ProbFuncs.erf(x);
+        },
+
+        /**
+         * Complementary error function, i.e. 1 - erf(x).
+         *
+         * @see JXG.Math.erf
+         * @see JXG.Math.PropFunc.erfc
+         * @param  {Number} x
+         * @returns {Number}
+         */
+         erfc: function(x) {
+            return this.ProbFuncs.erfc(x);
+        },
+
+        /**
+         * Inverse of error function
+         *
+         * @see JXG.Math.erf
+         * @see JXG.Math.PropFunc.erfi
+         * @param  {Number} x
+         * @returns {Number}
+         */
+         erfi: function(x) {
+            return this.ProbFuncs.erfi(x);
+        },
+
+        /**
+         * Normal distribution function
+         *
+         * @see JXG.Math.PropFunc.ndtr
+         * @param  {Number} x
+         * @returns {Number}
+         */
+         ndtr: function(x) {
+            return this.ProbFuncs.ndtr(x);
+        },
+
+        /**
+         * Inverse of normal distribution function
+         *
+         * @see JXG.Math.ndtr
+         * @see JXG.Math.PropFunc.ndtri
+         * @param  {Number} x
+         * @returns {Number}
+         */
+         ndtri: function(x) {
+            return this.ProbFuncs.ndtri(x);
+        },
+
+        /* ********************  Comparisons and logical operators ************** */
+
+        /**
+         * Logical test: a < b?
+         *
+         * @param {Number} a
+         * @param {Number} b
+         * @returns {Boolean}
+         */
+        lt: function(a, b) {
+            return a < b;
+        },
+
+        /**
+         * Logical test: a <= b?
+         *
+         * @param {Number} a
+         * @param {Number} b
+         * @returns {Boolean}
+         */
+        leq: function(a, b) {
+            return a <= b;
+        },
+
+        /**
+         * Logical test: a > b?
+         *
+         * @param {Number} a
+         * @param {Number} b
+         * @returns {Boolean}
+         */
+        gt: function(a, b) {
+            return a > b;
+        },
+
+        /**
+         * Logical test: a >= b?
+         *
+         * @param {Number} a
+         * @param {Number} b
+         * @returns {Boolean}
+         */
+        geq: function(a, b) {
+            return a >= b;
+        },
+
+        /**
+         * Logical test: a === b?
+         *
+         * @param {Number} a
+         * @param {Number} b
+         * @returns {Boolean}
+         */
+        eq: function(a, b) {
+            return a === b;
+        },
+
+        /**
+         * Logical test: a !== b?
+         *
+         * @param {Number} a
+         * @param {Number} b
+         * @returns {Boolean}
+         */
+        neq: function(a, b) {
+            return a !== b;
+        },
+
+        /**
+         * Logical operator: a && b?
+         *
+         * @param {Boolean} a
+         * @param {Boolean} b
+         * @returns {Boolean}
+         */
+        and: function(a, b) {
+            return a && b;
+        },
+
+        /**
+         * Logical operator: !a?
+         *
+         * @param {Boolean} a
+         * @returns {Boolean}
+         */
+        not: function(a) {
+            return !a;
+        },
+
+        /**
+         * Logical operator: a || b?
+         *
+         * @param {Boolean} a
+         * @param {Boolean} b
+         * @returns {Boolean}
+         */
+        or: function(a, b) {
+            return a || b;
+        },
+
+        /**
+         * Logical operator: either a or b?
+         *
+         * @param {Boolean} a
+         * @param {Boolean} b
+         * @returns {Boolean}
+         */
+        xor: function(a, b) {
+            return (a || b) && !(a && b);
+        },
+
+        /* *************************** Normalize *************************** */
+
         /**
          * Normalize the standard form [c, b0, b1, a, k, r, q0, q1].
          * @private
@@ -578,7 +1001,7 @@ define(['jxg'], function (JXG) {
             stdform[6] = -stdform[1] / a2;
             stdform[7] = -stdform[2] / a2;
 
-            if (r === Infinity || isNaN(r)) {
+            if (!isFinite(r)) {
                 n = Math.sqrt(stdform[1] * stdform[1] + stdform[2] * stdform[2]);
 
                 stdform[0] /= n;
@@ -630,8 +1053,37 @@ define(['jxg'], function (JXG) {
             }
 
             return v;
+        },
+
+        /**
+         * Theorem of Vieta: Given a set of simple zeroes x_0, ..., x_n
+         * of a polynomial f, compute the coefficients s_k, (k=0,...,n-1)
+         * of the polynomial of the form. See {@link https://de.wikipedia.org/wiki/Elementarsymmetrisches_Polynom}.
+         * <p>
+         *  f(x) = (x-x_0)*...*(x-x_n) =
+         *  x^n + sum_{k=1}^{n} (-1)^(k) s_{k-1} x^(n-k)
+         * </p>
+         * @param {Array} x Simple zeroes of the polynomial.
+         * @returns {Array} Coefficients of the polynomial.
+         *
+         */
+        Vieta: function(x) {
+            var n = x.length,
+                s = [],
+                m, k, y;
+
+            s = x.slice();
+            for (m = 1; m < n; ++m) {
+                y = s[m];
+                s[m] *= s[m - 1];
+                for (k = m - 1; k >= 1; --k) {
+                    s[k] += s[k - 1] * y;
+                }
+                s[0] += y;
+            }
+            return s;
         }
     };
 
     return JXG.Math;
-});
\ No newline at end of file
+});
