@@ -1,5 +1,5 @@
 /*
-    Copyright 2008-2013
+    Copyright 2008-2022
         Matthias Ehmann,
         Michael Gerhaeuser,
         Carsten Miller,
@@ -10,20 +10,20 @@
     This file is part of JSXGraph.
 
     JSXGraph is free software dual licensed under the GNU LGPL or MIT License.
-    
+
     You can redistribute it and/or modify it under the terms of the
-    
+
       * GNU Lesser General Public License as published by
         the Free Software Foundation, either version 3 of the License, or
         (at your option) any later version
       OR
       * MIT License: https://github.com/jsxgraph/jsxgraph/blob/master/LICENSE.MIT
-    
+
     JSXGraph is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU Lesser General Public License for more details.
-    
+
     You should have received a copy of the GNU Lesser General Public License and
     the MIT License along with JSXGraph. If not, see <http://www.gnu.org/licenses/>
     and <http://opensource.org/licenses/MIT/>.
@@ -58,7 +58,7 @@ define([
          * @returns {String} Converted expression.
          */
         replacePow: function (te) {
-            var count, pos, c,
+            var count, pos, c, previousIndex,
                 leftop, rightop, pre, p, left, i, right, expr;
 
             // delete all whitespace immediately before and after all ^ operators
@@ -66,7 +66,14 @@ define([
 
             //  Loop over all ^ operators
             i = te.indexOf('^');
-            while (i >= 0) {
+            previousIndex = -1;
+
+            while (i >= 0 && i < te.length - 1) {
+                if (previousIndex === i) {
+                    throw new Error("JSXGraph: Error while parsing expression '" + te + "'");
+                }
+                previousIndex = i;
+
                 // left and right are the substrings before, resp. after the ^ character
                 left = te.slice(0, i);
                 right = te.slice(i + 1);
@@ -98,12 +105,12 @@ define([
                         // Search for F or p.M before (...)^
                         pre = left.substring(0, pos + 1);
                         p = pos;
-                        while (p >= 0 && pre.substr(p, 1).match(/([\w\.]+)/)) {
+                        while (p >= 0 && pre.substr(p, 1).match(/([\w.]+)/)) {
                             leftop = RegExp.$1 + leftop;
                             p -= 1;
                         }
                         leftop += left.substring(pos + 1, left.length);
-                        leftop = leftop.replace(/([\(\)\+\*\%\^\-\/\]\[])/g, '\\$1');
+                        leftop = leftop.replace(/([()+*%^\-/\][])/g, '\\$1');
                     } else {
                         throw new Error("JSXGraph: Missing '(' in expression");
                     }
@@ -115,7 +122,7 @@ define([
                 // To the right of the ^ operator there also may be a function or method call
                 // or a term in parenthesis. Alos, ere we search for the closing
                 // parenthesis.
-                if (right.match(/^([\w\.]*\()/)) {
+                if (right.match(/^([\w.]*\()/)) {
                     count = 1;
                     pos = RegExp.$1.length;
 
@@ -132,7 +139,7 @@ define([
 
                     if (count === 0) {
                         rightop = right.substring(0, pos);
-                        rightop = rightop.replace(/([\(\)\+\*\%\^\-\/\[\]])/g, '\\$1');
+                        rightop = rightop.replace(/([()+*%^\-/[\]])/g, '\\$1');
                     } else {
                         throw new Error("JSXGraph: Missing ')' in expression");
                     }
@@ -373,7 +380,7 @@ define([
             expr = /(Dist)\(([\w_]+),([\w_]+)\)/g;
             term = term.replace(expr, 'dist($(\'$2\'), $(\'$3\'))');
 
-            expr = /(Deg)\(([\w_]+),([ \w\[\w_]+),([\w_]+)\)/g;
+            expr = /(Deg)\(([\w_]+),([ \w[\w_]+),([\w_]+)\)/g;
             term = term.replace(expr, 'deg($(\'$2\'),$(\'$3\'),$(\'$4\'))');
 
             // Search for Rad('gi23','gi24','gi25')
@@ -448,8 +455,8 @@ define([
             for (el in elements) {
                 if (elements.hasOwnProperty(el)) {
                     if (el !== me.name) {
-                        if (elements[el].type === Const.OBJECT_TYPE_TEXT) {
-                            if (!elements[el].visProp.islabel) {
+                        if (elements[el].elementClass === Const.OBJECT_CLASS_TEXT) {
+                            if (!Type.evaluate(elements[el].visProp.islabel)) {
                                 elmask = el.replace(/\[/g, '\\[');
                                 elmask = elmask.replace(/\]/g, '\\]');
 
