@@ -1,5 +1,5 @@
 /*
-    Copyright 2008-2013
+    Copyright 2008-2022
         Matthias Ehmann,
         Michael Gerhaeuser,
         Carsten Miller,
@@ -103,7 +103,7 @@
          * @returns {Object|Array|String} The gathered data
          */
         gEBTN: function (node, tag, idx, fc) {
-            var tmp;
+            var tmp = [];
 
             if (!JXG.exists(node || !node.getElementsByTagName)) {
                 return node;
@@ -115,7 +115,10 @@
             }
 
             idx = idx || 0;
-            tmp = node.getElementsByTagName(tag);
+
+            if (node.getElementsByTagName) {
+                tmp = node.getElementsByTagName(tag);
+            }
             if (tmp.length > 0) {
                 tmp = tmp[idx];
                 if (fc && tmp.firstChild) {
@@ -164,6 +167,7 @@
             gxtEl.labelColor = rgbo[0];
             gxtEl.withLabel = rgbo[1] > 0;
             gxtEl.labelOpacity = rgbo[1];
+
             gxtEl.colorDraft = JXG.rgba2rgbo(this.gEBTN(color, 'draft'))[0];
 
             // backwards compatibility
@@ -233,7 +237,7 @@
          * Transforms the Geonext properties to jsxgraph properties
          * @param {Object} gxtEl
          * @param {String} [type]
-         * @return {Object} gxtEl
+         * @returns {Object} gxtEl
          */
         transformProperties: function (gxtEl, type) {
             var i,
@@ -436,7 +440,7 @@
                             id: id,
                             anchor: el
                         });
-                        board.renderer.hide(el.label);
+                        board.renderer.display(el.label, false);
                         el.image = im;
                     } else if (el.elementClass === JXG.OBJECT_CLASS_CIRCLE) {
                         // A circle containing an image
@@ -588,6 +592,7 @@
                 gxtEl = this.transformProperties(gxtEl, 'point');
                 try {
                     gxtEl.parent = this.changeOriginIds(board, gxtEl.parent);
+                    gxtEl.isGeonext = true;
 
                     p = board.create('glider', [parseFloat(gxtEl.x), parseFloat(gxtEl.y), gxtEl.parent], gxtEl);
                     p.onPolygon = JXG.exists(gxtEl.onpolygon) && JXG.str2Bool(gxtEl.onpolygon);
@@ -684,9 +689,13 @@
                     gxtEl.outPoint = this.visualProperties(gxtEl.outPoint, xmlNode);
                     gxtEl.outPoint = this.firstLevelProperties(gxtEl.outPoint, xmlNode);
                     gxtEl.outPoint = this.transformProperties(gxtEl.outPoint);
-                    gxtEl.out.point = gxtEl.outPoint;
 
-                    board.create('arrowparallel', [gxtEl.defEl[1], gxtEl.defEl[0]], gxtEl.out);
+                    // construct this by hand, because arrowparallel uses projective geometry now
+                    p =  board.create('parallelpoint', [gxtEl.defEl[1], gxtEl.defEl[0]], gxtEl.outPoint);
+                    gxtEl.out.firstArrow = false;
+                    gxtEl.out.lastArrow = true;
+                    el = board.create('segment', [gxtEl.defEl[0], p], gxtEl.out);
+                    el.parallelpoint = p;
                     break;
 
                 // BISECTOR
@@ -901,14 +910,14 @@
                 // A.W.: Why do we need this?
                 /*
                  if (!gxtEl.visible) {
-                 p.setProperty({
+                 p.setAttribute({
                  fillColor: 'none',
                  highlightFillColor: 'none'
                  });
                  }
                  */
                 for (i = 0; i < p.borders.length; i++) {
-                    p.borders[i].setProperty(gxtEl.border[i]);
+                    p.borders[i].setAttribute(gxtEl.border[i]);
                 }
 
                 this.printDebugMessage('debug', gxtEl, Data.nodeName, 'OK');
@@ -973,6 +982,7 @@
                 gxtEl = this.firstLevelProperties(gxtEl, Data);
                 gxtEl = this.readNodes(gxtEl, Data, 'data');
                 gxtEl = this.transformProperties(gxtEl);
+                gxtEl.radius *= 1.0;
 
                 c = board.create('angle', [gxtEl.first, gxtEl.middle, gxtEl.last], gxtEl);
                 this.printDebugMessage('debug', gxtEl, Data.nodeName, 'OK');
@@ -1020,7 +1030,7 @@
                 }
                 gxtEl.parent = this.changeOriginIds(board, gxtEl.parent);
 
-                c = board.create('text', [gxtEl.x, gxtEl.y, gxtEl.mpStr], {
+                c = board.create('text', [parseFloat(gxtEl.x), parseFloat(gxtEl.y), gxtEl.mpStr], {
                     anchor: gxtEl.parent,
                     id: gxtEl.id,
                     name: gxtEl.name,
@@ -1112,6 +1122,7 @@
             xmlNode = this.gEBTN(boardData, 'coordinates', 0, false);
 
             tmp = this.readViewPort(xmlNode);
+
             if (tmp.length === 4) {
                 board.setBoundingBox(tmp, true);
             } else {
