@@ -1,5 +1,5 @@
 /*
-    Copyright 2008-2013
+    Copyright 2008-2022
         Matthias Ehmann,
         Michael Gerhaeuser,
         Carsten Miller,
@@ -412,7 +412,7 @@
         },
 
         setAttributes: function (o) {
-            o.setProperty({strokecolor: this.board.options.point.strokeColor, fillColor: this.board.options.point.fillColor});
+            o.setAttribute({strokecolor: this.board.options.point.strokeColor, fillColor: this.board.options.point.fillColor});
         },
 
         readParams: function (node) {
@@ -432,7 +432,7 @@
             if (!p.exists) {
                 p.exists = true;
                 p = this.board.create('point', p.coords, {name: p.id});
-                p.setProperty({strokecolor: 'red', fillColor: 'red'});
+                p.setAttribute({strokecolor: 'red', fillColor: 'red'});
             }
 
             return p;
@@ -599,14 +599,13 @@
             var param = this.readParams(node),
                 a = this.objects[param[0]].coords[0],
                 b = this.objects[param[0]].coords[1],
-                c = this.objects[param[0]].coords[2],
-                el = this.board.create('line', [c, a, b], {
-                    name: param[0],
-                    id: param[0],
-                    withLabel: true
-                });
+                c = this.objects[param[0]].coords[2];
 
-            this.objects[param[0]] = el;
+            this.objects[param[0]] = this.board.create('line', [c, a, b], {
+                name: param[0],
+                id: param[0],
+                withLabel: true
+            });
             this.objects[param[0]].exists = true;
         },
 
@@ -901,7 +900,7 @@
             var param = this.readParams(node),
                 el = this.board.select(param[1]);
 
-            el.setProperty({trace: true});
+            el.setAttribute({trace: true});
             this.objects[param[1]] = el;
             this.setAttributes(el);
         },
@@ -910,7 +909,7 @@
             var param = this.readParams(node),
                 el = this.board.select(param[1]);
 
-            el.setProperty({trace: true});
+            el.setAttribute({trace: true});
             this.objects[param[1]] = el;
             this.setAttributes(el);
         },
@@ -919,7 +918,7 @@
             var param = this.readParams(node),
                 el = this.board.select(param[1]);
 
-            el.setProperty({trace: true});
+            el.setAttribute({trace: true});
             this.objects[param[1]] = el;
             this.setAttributes(el);
         },
@@ -928,7 +927,7 @@
             var param = this.readParams(node),
                 el = this.board.select(param[1]);
 
-            el.setProperty({trace: true});
+            el.setAttribute({trace: true});
             this.objects[param[1]] = el;
             this.setAttributes(el);
         },
@@ -1028,7 +1027,7 @@
                         prop[key] = val;
                     }
                 }
-                el.setProperty(prop);
+                el.setAttribute(prop);
             } else {
                 JXG.debug('Display: not implemented' + node.nodeName);
             }
