@@ -1,5 +1,5 @@
 /*
-    Copyright 2008-2013
+    Copyright 2008-2022
         Matthias Ehmann,
         Michael Gerhaeuser,
         Carsten Miller,
@@ -31,7 +31,8 @@
 
 
 /*global JXG: true, define: true, AMprocessNode: true, MathJax: true, document: true */
-/*jslint nomen: true, plusplus: true, newcap:true*/
+/*jslint nomen: true, plusplus: true, newcap:true, unparam: true*/
+/*eslint no-unused-vars: "off"*/
 
 /* depends:
  jxg
@@ -52,8 +53,11 @@ define(['jxg', 'renderer/abstract'], function (JXG, AbstractRenderer) {
 
     /**
      * This renderer draws nothing. It is intended to be used in environments where none of our rendering engines
-     * are available, e.g. WebWorkers.
-     * @class JXG.AbstractRenderer
+     * are available, e.g. WebWorkers. All methods are empty.
+     * 
+     * @class JXG.NoRenderer
+     * @augments JXG.AbstractRenderer
+     * @see JXG.AbstractRenderer
      */
     JXG.NoRenderer = function () {
         /**
@@ -75,7 +79,7 @@ define(['jxg', 'renderer/abstract'], function (JXG, AbstractRenderer) {
         this.type = 'no';
     };
 
-    JXG.extend(JXG.NoRenderer.prototype, /** @lends JXG.AbstractRenderer.prototype */ {
+    JXG.extend(JXG.NoRenderer.prototype, /** @lends JXG.NoRenderer.prototype */ {
         /* ******************************** *
          *    Point drawing and updating    *
          * ******************************** */
@@ -150,17 +154,13 @@ define(['jxg', 'renderer/abstract'], function (JXG, AbstractRenderer) {
          * Update {@link Ticks} on a {@link JXG.Line}. This method is only a stub and has to be implemented
          * in any descendant renderer class.
          * @param {JXG.Line} element Reference of an line object, thats ticks have to be updated.
-         * @param {Number} dxMaj Number of pixels a major tick counts in x direction.
-         * @param {Number} dyMaj Number of pixels a major tick counts in y direction.
-         * @param {Number} dxMin Number of pixels a minor tick counts in x direction.
-         * @param {Number} dyMin Number of pixels a minor tick counts in y direction.
          * @see Line
          * @see Ticks
          * @see JXG.Line
          * @see JXG.Ticks
          * @see JXG.AbstractRenderer#drawTicks
          */
-        updateTicks: function (element, dxMaj, dyMaj, dxMin, dyMin) { /* stub */ },
+        updateTicks: function (element) { /* stub */ },
 
         /* **************************
          *    Curves
@@ -489,17 +489,34 @@ define(['jxg', 'renderer/abstract'], function (JXG, AbstractRenderer) {
          */
         setPropertyPrim: function (node, key, val) { /* stub */ },
 
+        /**
+         * Shows or hides an element on the canvas; Only a stub, requires implementation in the derived renderer.
+         * @param {JXG.GeometryElement} element Reference to the object that has to appear.
+         * @param {Boolean} value true to show the element, false to hide the element.
+         */
+        display: function (element, value) {
+            if (element) {
+                element.visPropOld.visible = value;
+            }
+        },
+
         /**
          * Shows a hidden element on the canvas; Only a stub, requires implementation in the derived renderer.
+         *
+         * Please use JXG.AbstractRenderer#display instead
          * @param {JXG.GeometryElement} element Reference to the object that has to appear.
          * @see JXG.AbstractRenderer#hide
+         * @deprecated
          */
         show: function (element) { /* stub */ },
 
         /**
          * Hides an element on the canvas; Only a stub, requires implementation in the derived renderer.
+         *
+         * Please use JXG.AbstractRenderer#display instead
          * @param {JXG.GeometryElement} element Reference to the geometry element that has to disappear.
          * @see JXG.AbstractRenderer#show
+         * @deprecated
          */
         hide: function (element) { /* stub */ },
 
@@ -543,6 +560,16 @@ define(['jxg', 'renderer/abstract'], function (JXG, AbstractRenderer) {
          */
         updateGradient: function (element) { /* stub */ },
 
+        /**
+         * Sets the transition duration (in milliseconds) for fill color and stroke
+         * color and opacity.
+         * @param {JXG.GeometryElement} element Reference of the object that wants a
+         *         new transition duration.
+         * @param {Number} duration (Optional) duration in milliseconds. If not given,
+         *        element.visProp.transitionDuration is taken. This is the default.
+         */
+        setObjectTransition: function (element, duration) { /* stub */ },
+
         /**
          * Sets an objects fill color.
          * @param {JXG.GeometryElement} element Reference of the object that wants a new fill color.
@@ -633,6 +660,9 @@ define(['jxg', 'renderer/abstract'], function (JXG, AbstractRenderer) {
 
     });
 
+    /**
+     * @ignore
+     */
     JXG.NoRenderer.prototype = new AbstractRenderer();
 
     return JXG.NoRenderer;
