@@ -1,5 +1,5 @@
 /*
-    Copyright 2008-2013
+    Copyright 2008-2022
         Matthias Ehmann,
         Michael Gerhaeuser,
         Carsten Miller,
@@ -51,16 +51,23 @@ define([
 
     /**
      * Uses VML to implement the rendering methods defined in {@link JXG.AbstractRenderer}.
-     * @class JXG.AbstractRenderer
+     * VML was used in very old Internet Explorer versions upto IE 8.
+     * 
+     * 
+     * @class JXG.VMLRenderer
      * @augments JXG.AbstractRenderer
      * @param {Node} container Reference to a DOM node containing the board.
      * @see JXG.AbstractRenderer
+     * @deprecated
      */
     JXG.VMLRenderer = function (container) {
         this.type = 'vml';
 
         this.container = container;
         this.container.style.overflow = 'hidden';
+        if (this.container.style.position === '') {
+            this.container.style.position = 'relative';
+        }
         this.container.onselectstart = function () {
             return false;
         };
@@ -96,7 +103,7 @@ define([
 
     JXG.VMLRenderer.prototype = new AbstractRenderer();
 
-    JXG.extend(JXG.VMLRenderer.prototype, /** @lends JXG.VMLRenderer */ {
+    JXG.extend(JXG.VMLRenderer.prototype, /** @lends JXG.VMLRenderer.prototype */ {
 
         /**
          * Sets attribute <tt>key</tt> of node <tt>node</tt> to <tt>value</tt>.
@@ -107,7 +114,7 @@ define([
          */
         _setAttr: function (node, key, val, iFlag) {
             try {
-                if (document.documentMode === 8) {
+                if (this.container.ownerDocument.documentMode === 8) {
                     node[key] = val;
                 } else {
                     node.setAttribute(key, val, iFlag);
@@ -129,7 +136,7 @@ define([
          * **************************/
 
         // documented in AbstractRenderer
-        updateTicks: function (ticks, dxMaj, dyMaj, dxMin, dyMin) {
+        updateTicks: function (ticks) {
             var i, len, c, x, y,
                 r = this.resolution,
                 tickArr = [];
@@ -140,7 +147,7 @@ define([
                 x = c[0];
                 y = c[1];
 
-                if (typeof x[0] === 'number' && typeof x[1] === 'number') {
+                if (Type.isNumber(x[0]) && Type.isNumber(x[1])) {
                     tickArr.push(' m ' + Math.round(r * x[0]) + ', ' + Math.round(r * y[0]) +
                         ' l ' + Math.round(r * x[1]) + ', ' + Math.round(r * y[1]) + ' ');
                 }
@@ -148,13 +155,13 @@ define([
 
             if (!Type.exists(ticks.rendNode)) {
                 ticks.rendNode = this.createPrim('path', ticks.id);
-                this.appendChildPrim(ticks.rendNode, ticks.visProp.layer);
+                this.appendChildPrim(ticks.rendNode, Type.evaluate(ticks.visProp.layer));
             }
 
             this._setAttr(ticks.rendNode, 'stroked', 'true');
-            this._setAttr(ticks.rendNode, 'strokecolor', ticks.visProp.strokecolor, 1);
-            this._setAttr(ticks.rendNode, 'strokeweight', ticks.visProp.strokewidth);
-            this._setAttr(ticks.rendNodeStroke, 'opacity', (ticks.visProp.strokeopacity * 100) + '%');
+            this._setAttr(ticks.rendNode, 'strokecolor', Type.evaluate(ticks.visProp.strokecolor), 1);
+            this._setAttr(ticks.rendNode, 'strokeweight', Type.evaluate(ticks.visProp.strokewidth));
+            this._setAttr(ticks.rendNodeStroke, 'opacity', (Type.evaluate(ticks.visProp.strokeopacity) * 100) + '%');
             this.updatePathPrim(ticks.rendNode, tickArr, ticks.board);
         },
 
@@ -162,7 +169,7 @@ define([
          *    Text related stuff
          * **************************/
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         displayCopyright: function (str, fontsize) {
             var node, t;
 
@@ -176,9 +183,9 @@ define([
             node.style.color = '#356AA0';
             node.style.fontFamily = 'Arial,Helvetica,sans-serif';
             this._setAttr(node, 'opacity', '30%');
-            node.style.filter = 'alpha(opacity = 30)';
+            node.style.filter = "progid:DXImageTransform.Microsoft.Matrix(M11='1.0', sizingMethod='auto expand', enabled = false) progid:DXImageTransform.Microsoft.Alpha(opacity = 30, enabled = true)";
 
-            t = document.createTextNode(str);
+            t = this.container.ownerDocument.createTextNode(str);
             node.appendChild(t);
             this.appendChildPrim(node, 0);
         },
@@ -188,43 +195,99 @@ define([
             var node;
             node = this.createNode('textbox');
             node.style.position = 'absolute';
-            /*
-             if (document.documentMode === 8) {                 // IE 8
-             node.setAttribute('class', el.visProp.cssclass);
-             } else {
-             node.setAttribute(document.all ? 'className' : 'class', el.visProp.cssclass);
-             }
-             */
-            el.rendNodeText = document.createTextNode('');
+            el.rendNodeText = this.container.ownerDocument.createTextNode('');
             node.appendChild(el.rendNodeText);
             this.appendChildPrim(node, 9);
+            node.style.filter = "progid:DXImageTransform.Microsoft.Matrix(M11='1.0', sizingMethod='auto expand', enabled = false) progid:DXImageTransform.Microsoft.Alpha(opacity = 100, enabled = false)";
+
             return node;
         },
 
         // documented in AbstractRenderer
         updateInternalText: function (el) {
-            var v,
-                content = el.plaintext;
+            var v, content = el.plaintext,
+                m = this.joinTransforms(el, el.transformations),
+                offset = [0, 0],
+                maxX, maxY, minX, minY, i,
+                node = el.rendNode,
+                p = [],
+                ev_ax = el.getAnchorX(),
+                ev_ay = el.getAnchorY();
 
             if (!isNaN(el.coords.scrCoords[1] + el.coords.scrCoords[2])) {
-                if (el.visProp.anchorx === 'right') {
-                    el.rendNode.style.right = Math.floor(el.board.canvasWidth - el.coords.scrCoords[1]) + 'px';
-                    el.rendNode.style.left = 'auto';
-                } else if (el.visProp.anchorx === 'middle') {
-                    el.rendNode.style.left = Math.floor(el.coords.scrCoords[1] - 0.5 * el.size[0]) + 'px';
-                    el.rendNode.style.right = 'auto';
-                } else {
-                    el.rendNode.style.left = Math.floor(el.coords.scrCoords[1]) + 'px';
-                    el.rendNode.style.right = 'auto';
+                // Horizontal
+                if (ev_ax === 'right') {
+                    offset[0] = 1;
+                } else if (ev_ax === 'middle') {
+                    offset[0] = 0.5;
+                } // default (ev_ax === 'left') offset[0] = 0;
+
+                // Vertical
+                if (ev_ay === 'bottom') {
+                    offset[1] = 1;
+                } else if (ev_ay === 'middle') {
+                    offset[1] = 0.5;
+                } // default (ev_ay === 'top') offset[1] = 0;
+
+                // Compute maxX, maxY, minX, minY
+                p[0] = Mat.matVecMult(m, [1,
+                                          el.coords.scrCoords[1] - offset[0] * el.size[0],
+                                          el.coords.scrCoords[2] + (1 - offset[1]) * el.size[1] + this.vOffsetText]);
+                p[0][1] /= p[0][0];
+                p[0][2] /= p[0][0];
+                p[1] = Mat.matVecMult(m, [1,
+                                          el.coords.scrCoords[1] + (1 - offset[0]) * el.size[0],
+                                          el.coords.scrCoords[2] + (1 - offset[1]) * el.size[1] + this.vOffsetText]);
+                p[1][1] /= p[1][0];
+                p[1][2] /= p[1][0];
+                p[2] = Mat.matVecMult(m, [1,
+                                          el.coords.scrCoords[1] + (1 - offset[0]) * el.size[0],
+                                          el.coords.scrCoords[2] - offset[1] * el.size[1] + this.vOffsetText]);
+                p[2][1] /= p[2][0];
+                p[2][2] /= p[2][0];
+                p[3] = Mat.matVecMult(m, [1,
+                                          el.coords.scrCoords[1] - offset[0] * el.size[0],
+                                          el.coords.scrCoords[2] - offset[1] * el.size[1] + this.vOffsetText]);
+                p[3][1] /= p[3][0];
+                p[3][2] /= p[3][0];
+                maxX = p[0][1];
+                minX = p[0][1];
+                maxY = p[0][2];
+                minY = p[0][2];
+
+                for (i = 1; i < 4; i++) {
+                    maxX = Math.max(maxX, p[i][1]);
+                    minX = Math.min(minX, p[i][1]);
+                    maxY = Math.max(maxY, p[i][2]);
+                    minY = Math.min(minY, p[i][2]);
                 }
 
-                if (el.visProp.anchory === 'top') {
-                    el.rendNode.style.top = Math.floor(el.coords.scrCoords[2] + this.vOffsetText) + 'px';
-                } else if (el.visProp.anchory === 'middle') {
-                    el.rendNode.style.top = Math.floor(el.coords.scrCoords[2] - 0.5 * el.size[1] + this.vOffsetText) + 'px';
-                } else {
-                    el.rendNode.style.top = Math.floor(el.coords.scrCoords[2] - el.size[1] + this.vOffsetText) + 'px';
+                // Horizontal
+                v = offset[0] === 1 ? Math.floor(el.board.canvasWidth - maxX) : Math.floor(minX);
+                if (el.visPropOld.left !== (ev_ax + v)) {
+                    if (offset[0] === 1) {
+                        el.rendNode.style.right = v + 'px';
+                        el.rendNode.style.left = 'auto';
+                    } else {
+                        el.rendNode.style.left = v + 'px';
+                        el.rendNode.style.right = 'auto';
+                    }
+                    el.visPropOld.left = ev_ax + v;
                 }
+
+                // Vertical
+                v = offset[1] === 1 ? Math.floor(el.board.canvasHeight - maxY) : Math.floor(minY);
+                if (el.visPropOld.top !== (ev_ay + v)) {
+                    if (offset[1] === 1) {
+                        el.rendNode.style.bottom = v + 'px';
+                        el.rendNode.style.top = 'auto';
+                    } else {
+                        el.rendNode.style.top = v + 'px';
+                        el.rendNode.style.bottom = 'auto';
+                    }
+                    el.visPropOld.top = ev_ay + v;
+                }
+
             }
 
             if (el.htmlStr !== content) {
@@ -232,14 +295,19 @@ define([
                 el.htmlStr = content;
             }
 
-            this.transformImage(el, el.transformations);
+            //this.transformImage(el, el.transformations);
+            node.filters.item(0).M11 = m[1][1];
+            node.filters.item(0).M12 = m[1][2];
+            node.filters.item(0).M21 = m[2][1];
+            node.filters.item(0).M22 = m[2][2];
+            node.filters.item(0).enabled = true;
         },
 
         /* **************************
          *    Image related stuff
          * **************************/
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         drawImage: function (el) {
             // IE 8: Bilder ueber data URIs werden bis 32kB unterstuetzt.
             var node;
@@ -249,31 +317,32 @@ define([
             this._setAttr(node, 'id', this.container.id + '_' + el.id);
 
             this.container.appendChild(node);
-            this.appendChildPrim(node, el.visProp.layer);
+            this.appendChildPrim(node, Type.evaluate(el.visProp.layer));
 
             // Adding the rotation filter. This is always filter item 0:
             // node.filters.item(0), see transformImage
+            // Also add the alpha filter. This is always filter item 1
+            // node.filters.item(1), see setObjectFillColor and setObjectSTrokeColor
             //node.style.filter = node.style['-ms-filter'] = "progid:DXImageTransform.Microsoft.Matrix(M11='1.0', sizingMethod='auto expand')";
-            node.style.filter = "progid:DXImageTransform.Microsoft.Matrix(M11='1.0', sizingMethod='auto expand')";
+            node.style.filter = "progid:DXImageTransform.Microsoft.Matrix(M11='1.0', sizingMethod='auto expand') progid:DXImageTransform.Microsoft.Alpha(opacity = 100, enabled = false)";
             el.rendNode = node;
             this.updateImage(el);
         },
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         transformImage: function (el, t) {
             var m, s, maxX, maxY, minX, minY, i, nt,
                 node = el.rendNode,
                 p = [],
                 len = t.length;
 
-            if (el.type === Const.OBJECT_TYPE_TEXT) {
-                el.updateSize();
-            }
             if (len > 0) {
+                /*
                 nt = el.rendNode.style.filter.toString();
                 if (!nt.match(/DXImageTransform/)) {
                     node.style.filter = "progid:DXImageTransform.Microsoft.Matrix(M11='1.0', sizingMethod='auto expand') " + nt;
                 }
+                */
 
                 m = this.joinTransforms(el, t);
                 p[0] = Mat.matVecMult(m, el.coords.scrCoords);
@@ -306,10 +375,11 @@ define([
                 node.filters.item(0).M12 = m[1][2];
                 node.filters.item(0).M21 = m[2][1];
                 node.filters.item(0).M22 = m[2][2];
+                node.filters.item(0).enabled = true;
             }
         },
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         updateImageURL: function (el) {
             var url = Type.evaluate(el.url);
 
@@ -320,7 +390,7 @@ define([
          * Render primitive objects
          * **************************/
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         appendChildPrim: function (node, level) {
             // For trace nodes
             if (!Type.exists(level)) {
@@ -333,18 +403,18 @@ define([
             return node;
         },
 
-        // already documented in JXG.AbstractRenderer
-        appendNodesToElement: function (element, type) {
+        // Already documented in JXG.AbstractRenderer
+        appendNodesToElement: function (el, type) {
             if (type === 'shape' || type === 'path' || type === 'polygon') {
-                element.rendNodePath = this.getElementById(element.id + '_path');
+                el.rendNodePath = this.getElementById(el.id + '_path');
             }
-            element.rendNodeFill = this.getElementById(element.id + '_fill');
-            element.rendNodeStroke = this.getElementById(element.id + '_stroke');
-            element.rendNodeShadow = this.getElementById(element.id + '_shadow');
-            element.rendNode = this.getElementById(element.id);
+            el.rendNodeFill = this.getElementById(el.id + '_fill');
+            el.rendNodeStroke = this.getElementById(el.id + '_stroke');
+            el.rendNodeShadow = this.getElementById(el.id + '_shadow');
+            el.rendNode = this.getElementById(el.id);
         },
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         createPrim: function (type, id) {
             var node, pathNode,
                 fillNode = this.createNode('fill'),
@@ -383,22 +453,24 @@ define([
             return node;
         },
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         remove: function (node) {
             if (Type.exists(node)) {
                 node.removeNode(true);
             }
         },
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         makeArrows: function (el) {
-            var nodeStroke;
+            var nodeStroke,
+                ev_fa = Type.evaluate(el.visProp.firstarrow),
+                ev_la = Type.evaluate(el.visProp.lastarrow);
 
-            if (el.visPropOld.firstarrow === el.visProp.firstarrow && el.visPropOld.lastarrow === el.visProp.lastarrow) {
+            if (el.visPropOld.firstarrow === ev_fa && el.visPropOld.lastarrow === ev_la) {
                 return;
             }
 
-            if (el.visProp.firstarrow) {
+            if (ev_fa) {
                 nodeStroke = el.rendNodeStroke;
                 this._setAttr(nodeStroke, 'startarrow', 'block');
                 this._setAttr(nodeStroke, 'startarrowlength', 'long');
@@ -409,7 +481,7 @@ define([
                 }
             }
 
-            if (el.visProp.lastarrow) {
+            if (ev_la) {
                 nodeStroke = el.rendNodeStroke;
                 this._setAttr(nodeStroke, 'id', this.container.id + '_' + el.id + "stroke");
                 this._setAttr(nodeStroke, 'endarrow', 'block');
@@ -420,11 +492,11 @@ define([
                     this._setAttr(nodeStroke, 'endarrow', 'none');
                 }
             }
-            el.visPropOld.firstarrow = el.visProp.firstarrow;
-            el.visPropOld.lastarrow = el.visProp.lastarrow;
+            el.visPropOld.firstarrow = ev_fa;
+            el.visPropOld.lastarrow = ev_la;
         },
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         updateEllipsePrim: function (node, x, y, rx, ry) {
             node.style.left = Math.floor(x - rx) + 'px';
             node.style.top =  Math.floor(y - ry) + 'px';
@@ -432,7 +504,7 @@ define([
             node.style.height = Math.floor(Math.abs(ry) * 2) + 'px';
         },
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         updateLinePrim: function (node, p1x, p1y, p2x, p2y, board) {
             var s, r = this.resolution;
 
@@ -442,7 +514,7 @@ define([
             }
         },
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         updatePathPrim: function (node, pointString, board) {
             var x = board.canvasWidth,
                 y = board.canvasHeight;
@@ -455,7 +527,7 @@ define([
             this._setAttr(node, 'path', pointString.join(""));
         },
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         updatePathStringPoint: function (el, size, type) {
             var s = [],
                 mround = Math.round,
@@ -520,7 +592,7 @@ define([
             return s;
         },
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         updatePathStringPrim: function (el) {
             var i, scr,
                 pStr = [],
@@ -530,7 +602,6 @@ define([
                 symbl = ' l ',
                 symbc = ' c ',
                 nextSymb = symbm,
-                isNotPlot = (el.visProp.curvetype !== 'plot'),
                 len = Math.min(el.numberPoints, 8192); // otherwise IE 7 crashes in hilbert.html
 
             if (el.numberPoints <= 0) {
@@ -539,9 +610,11 @@ define([
             len = Math.min(len, el.points.length);
 
             if (el.bezierDegree === 1) {
+                /*
                 if (isNotPlot && el.board.options.curve.RDPsmoothing) {
-                    el.points = Numerics.RamerDouglasPeuker(el.points, 1.0);
+                    el.points = Numerics.RamerDouglasPeucker(el.points, 1.0);
                 }
+                */
 
                 for (i = 0; i < len; i++) {
                     scr = el.points[i].scrCoords;
@@ -590,24 +663,24 @@ define([
             return pStr;
         },
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         updatePathStringBezierPrim: function (el) {
             var i, j, k, scr, lx, ly,
                 pStr = [],
-                f = el.visProp.strokewidth,
+                f = Type.evaluate(el.visProp.strokewidth),
                 r = this.resolution,
                 mround = Math.round,
                 symbm = ' m ',
                 symbl = ' c ',
                 nextSymb = symbm,
-                isNoPlot = (el.visProp.curvetype !== 'plot'),
+                isNoPlot = (Type.evaluate(el.visProp.curvetype) !== 'plot'),
                 len = Math.min(el.numberPoints, 8192); // otherwise IE 7 crashes in hilbert.html
 
             if (el.numberPoints <= 0) {
                 return '';
             }
             if (isNoPlot && el.board.options.curve.RDPsmoothing) {
-                el.points = Numerics.RamerDouglasPeuker(el.points, 1.0);
+                el.points = Numerics.RamerDouglasPeucker(el.points, 1.0);
             }
             len = Math.min(len, el.points.length);
 
@@ -654,7 +727,7 @@ define([
             return pStr;
         },
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         updatePolygonPrim: function (node, el) {
             var i,
                 len = el.vertices.length,
@@ -692,7 +765,7 @@ define([
             this.updatePathPrim(node, pStr, el.board);
         },
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         updateRectPrim: function (node, x, y, w, h) {
             node.style.left = Math.floor(x) + 'px';
             node.style.top = Math.floor(y) + 'px';
@@ -710,7 +783,7 @@ define([
          *  Set Attributes
          * **************************/
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         setPropertyPrim: function (node, key, val) {
             var keyVml = '',
                 v;
@@ -733,21 +806,37 @@ define([
             }
         },
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
+        display: function(el, val) {
+            if (el && el.rendNode) {
+                el.visPropOld.visible = val;
+                if (val) {
+                    el.rendNode.style.visibility = "inherit";
+                } else {
+                    el.rendNode.style.visibility = "hidden";
+                }
+            }
+        },
+
+        // Already documented in JXG.AbstractRenderer
         show: function (el) {
+            JXG.deprecated('Board.renderer.show()', 'Board.renderer.display()');
+
             if (el && el.rendNode) {
                 el.rendNode.style.visibility = "inherit";
             }
         },
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         hide: function (el) {
+            JXG.deprecated('Board.renderer.hide()', 'Board.renderer.display()');
+
             if (el && el.rendNode) {
                 el.rendNode.style.visibility = "hidden";
             }
         },
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         setDashStyle: function (el, visProp) {
             var node;
             if (visProp.dash >= 0) {
@@ -756,27 +845,29 @@ define([
             }
         },
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         setGradient: function (el) {
-            var nodeFill = el.rendNodeFill;
+            var nodeFill = el.rendNodeFill,
+                ev_g = Type.evaluate(el.visProp.gradient);
 
-            if (el.visProp.gradient === 'linear') {
+            if (ev_g === 'linear') {
                 this._setAttr(nodeFill, 'type', 'gradient');
-                this._setAttr(nodeFill, 'color2', el.visProp.gradientsecondcolor);
-                this._setAttr(nodeFill, 'opacity2', el.visProp.gradientsecondopacity);
-                this._setAttr(nodeFill, 'angle', el.visProp.gradientangle);
-            } else if (el.visProp.gradient === 'radial') {
+                this._setAttr(nodeFill, 'color2', Type.evaluate(el.visProp.gradientsecondcolor));
+                this._setAttr(nodeFill, 'opacity2', Type.evaluate(el.visProp.gradientsecondopacity));
+                this._setAttr(nodeFill, 'angle', Type.evaluate(el.visProp.gradientangle));
+            } else if (ev_g === 'radial') {
                 this._setAttr(nodeFill, 'type', 'gradientradial');
-                this._setAttr(nodeFill, 'color2', el.visProp.gradientsecondcolor);
-                this._setAttr(nodeFill, 'opacity2', el.visProp.gradientsecondopacity);
-                this._setAttr(nodeFill, 'focusposition', el.visProp.gradientpositionx * 100 + '%,' + el.visProp.gradientpositiony * 100 + '%');
+                this._setAttr(nodeFill, 'color2', Type.evaluate(el.visProp.gradientsecondcolor));
+                this._setAttr(nodeFill, 'opacity2', Type.evaluate(el.visProp.gradientsecondopacity));
+                this._setAttr(nodeFill, 'focusposition', Type.evaluate(el.visProp.gradientpositionx) * 100 + '%,' +
+                            Type.evaluate(el.visProp.gradientpositiony) * 100 + '%');
                 this._setAttr(nodeFill, 'focussize', '0,0');
             } else {
                 this._setAttr(nodeFill, 'type', 'solid');
             }
         },
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         setObjectFillColor: function (el, color, opacity) {
             var rgba = Type.evaluate(color), c, rgbo,
                 o = Type.evaluate(opacity), oo,
@@ -811,21 +902,29 @@ define([
                     }
                 }
                 if (el.type === Const.OBJECT_TYPE_IMAGE) {
+                    /*
                     t = el.rendNode.style.filter.toString();
                     if (t.match(/alpha/)) {
                         el.rendNode.style.filter = t.replace(/alpha\(opacity *= *[0-9\.]+\)/, 'alpha(opacity = ' + (oo * 100) + ')');
                     } else {
                         el.rendNode.style.filter += ' alpha(opacity = ' + (oo * 100) + ')';
                     }
+                    */
+                    if (node.filters.length > 1) {
+                        // Why am I sometimes seeing node.filters.length==0 here when I move the pointer around near [0,0]?
+                        // Setting axes:true shows text labels!
+                        node.filters.item(1).opacity = Math.round(oo * 100); // Why does setObjectFillColor not use Math.round?
+                        node.filters.item(1).enabled = true;
+                    }
                 }
             }
             el.visPropOld.fillcolor = rgba;
             el.visPropOld.fillopacity = o;
         },
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         setObjectStrokeColor: function (el, color, opacity) {
-            var rgba = Type.evaluate(color), c, rgbo,
+            var rgba = Type.evaluate(color), c, rgbo, t,
                 o = Type.evaluate(opacity), oo,
                 node = el.rendNode, nodeStroke;
 
@@ -848,9 +947,24 @@ define([
                     c = rgbo[0];
                     oo = o * rgbo[1];
                 }
-                if (el.type === Const.OBJECT_TYPE_TEXT) {
-                    oo = Math.round(oo * 100);
-                    node.style.filter = ' alpha(opacity = ' + oo + ')';
+                if (el.elementClass === Const.OBJECT_CLASS_TEXT) {
+                    //node.style.filter = ' alpha(opacity = ' + oo + ')';
+                    /*
+                    t = node.style.filter.toString();
+                    if (t.match(/alpha/)) {
+                        node.style.filter =
+                        t.replace(/alpha\(opacity *= *[0-9\.]+\)/, 'alpha(opacity = ' + oo + ')');
+                    } else {
+                        node.style.filter += ' alpha(opacity = ' + oo + ')';
+                    }
+                    */
+                    if (node.filters.length > 1) {
+                        // Why am I sometimes seeing node.filters.length==0 here when I move the pointer around near [0,0]?
+                        // Setting axes:true shows text labels!
+                        node.filters.item(1).opacity = Math.round(oo * 100);
+                        node.filters.item(1).enabled = true;
+                    }
+
                     node.style.color = c;
                 } else {
                     if (c !== false) {
@@ -868,7 +982,7 @@ define([
             el.visPropOld.strokeopacity = o;
         },
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         setObjectStrokeWidth: function (el, width) {
             var w = Type.evaluate(width),
                 node;
@@ -881,21 +995,27 @@ define([
             this.setPropertyPrim(node, 'stroked', 'true');
 
             if (Type.exists(w)) {
+
                 this.setPropertyPrim(node, 'stroke-width', w);
+                if (w === 0 && Type.exists(el.rendNodeStroke)) {
+                    this._setAttr(node, 'stroked', 'false');
+                }
             }
 
             el.visPropOld.strokewidth = w;
+
         },
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         setShadow: function (el) {
-            var nodeShadow = el.rendNodeShadow;
+            var nodeShadow = el.rendNodeShadow,
+                ev_s = Type.evaluate(el.visProp.shadow);
 
-            if (!nodeShadow || el.visPropOld.shadow === el.visProp.shadow) {
+            if (!nodeShadow || el.visPropOld.shadow === ev_s) {
                 return;
             }
 
-            if (el.visProp.shadow) {
+            if (ev_s) {
                 this._setAttr(nodeShadow, 'On', 'True');
                 this._setAttr(nodeShadow, 'Offset', '3pt,3pt');
                 this._setAttr(nodeShadow, 'Opacity', '60%');
@@ -904,19 +1024,19 @@ define([
                 this._setAttr(nodeShadow, 'On', 'False');
             }
 
-            el.visPropOld.shadow = el.visProp.shadow;
+            el.visPropOld.shadow = ev_s;
         },
 
         /* **************************
          * renderer control
          * **************************/
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         suspendRedraw: function () {
             this.container.style.display = 'none';
         },
 
-        // already documented in JXG.AbstractRenderer
+        // Already documented in JXG.AbstractRenderer
         unsuspendRedraw: function () {
             this.container.style.display = '';
         }
