@@ -1,35 +1,35 @@
 /*
-    Copyright 2008-2013
+    Copyright 2008-2022
         Matthias Ehmann,
         Michael Gerhaeuser,
         Carsten Miller,
         Bianca Valentin,
+        Andreas Walter,
         Alfred Wassermann,
         Peter Wilfahrt
 
     This file is part of JSXGraph.
 
     JSXGraph is free software dual licensed under the GNU LGPL or MIT License.
-    
+
     You can redistribute it and/or modify it under the terms of the
-    
+
       * GNU Lesser General Public License as published by
         the Free Software Foundation, either version 3 of the License, or
         (at your option) any later version
       OR
       * MIT License: https://github.com/jsxgraph/jsxgraph/blob/master/LICENSE.MIT
-    
+
     JSXGraph is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU Lesser General Public License for more details.
-    
+
     You should have received a copy of the GNU Lesser General Public License and
     the MIT License along with JSXGraph. If not, see <http://www.gnu.org/licenses/>
     and <http://opensource.org/licenses/MIT/>.
  */
 
-
 /*global JXG: true, define: true*/
 
 /*jslint nomen: true, plusplus: true*/
@@ -45,7 +45,8 @@
  * Stoyan Stefanov <sstoo@gmail.com> (see http://www.phpied.com/rgb-color-parser-in-javascript/)
  */
 
-define(['jxg', 'utils/type', 'math/math'], function (JXG, Type, Mat) {
+define(['jxg', 'utils/type', 'math/math'],
+    function (JXG, Type, Mat) {
 
     "use strict";
 
@@ -110,8 +111,8 @@ define(['jxg', 'utils/type', 'math/math'], function (JXG, Type, Mat) {
             greenyellow: 'adff2f',
             honeydew: 'f0fff0',
             hotpink: 'ff69b4',
-            indianred : 'cd5c5c',
-            indigo : '4b0082',
+            indianred: 'cd5c5c',
+            indigo: '4b0082',
             ivory: 'fffff0',
             khaki: 'f0e68c',
             lavender: 'e6e6fa',
@@ -198,9 +199,10 @@ define(['jxg', 'utils/type', 'math/math'], function (JXG, Type, Mat) {
             yellow: 'ffff00',
             yellowgreen: '9acd32'
         },
+
         // array of color definition objects
         colorDefs = [{
-            re: /^\s*rgba\(\s*(\d{1,3})\s*,\s*(\d{1,3})\s*,\s*(\d{1,3})\s*,\s*([\d\.]{1,3})\s*\)\s*$/,
+            re: /^\s*rgba\(\s*(\d{1,3})\s*,\s*(\d{1,3})\s*,\s*(\d{1,3})\s*,\s*([\d.]{1,3})\s*\)\s*$/,
             example: ['rgba(123, 234, 45, 0.5)', 'rgba(255,234,245,1.0)'],
             process: function (bits) {
                 return [
@@ -257,7 +259,7 @@ define(['jxg', 'utils/type', 'math/math'], function (JXG, Type, Mat) {
         var color_string, channels, re, processor, bits, i,
             r, g, b,
             values = color,
-            testFloat = false;
+            testFloat;
 
         if (!Type.exists(color)) {
             return [];
@@ -269,6 +271,7 @@ define(['jxg', 'utils/type', 'math/math'], function (JXG, Type, Mat) {
 
         color_string = values;
 
+        testFloat = false;
         if (Type.isArray(color_string)) {
             for (i = 0; i < 3; i++) {
                 testFloat = testFloat || /\./.test(values[i].toString());
@@ -384,14 +387,12 @@ define(['jxg', 'utils/type', 'math/math'], function (JXG, Type, Mat) {
     /**
      * Converts a valid HTML/CSS color string from the '#rrggbb' format into the 'rgb(r, g, b)' format.
      * @param {String} hex A valid HTML or CSS styled color value, e.g. '#12ab21', '#abc', or 'black'
+     * @deprecated Use {@link JXG#rgb2css} instead.
      * @returns {String} A 'rgb(r, g, b)' formatted string
      */
     JXG.hex2rgb = function (hex) {
-        var r;
-
-        r = JXG.rgbParser(hex);
-
-        return 'rgb(' + r[0] + ', ' + r[1] + ', ' + r[2] + ')';
+        JXG.deprecated('JXG.hex2rgb()', 'JXG.rgb2css()');
+        return JXG.rgb2css(hex);
     };
 
     /**
@@ -402,7 +403,7 @@ define(['jxg', 'utils/type', 'math/math'], function (JXG, Type, Mat) {
      * @param {Number} H value between 0 and 360
      * @param {Number} S value between 0.0 (shade of gray) to 1.0 (pure color)
      * @param {Number} V value between 0.0 (black) to 1.0 (white)
-     * @return {String} RGB color string
+     * @returns {String} RGB color string
      */
     JXG.hsv2rgb = function (H, S, V) {
         var R, G, B, f, i, hTemp, p, q, t;
@@ -435,36 +436,36 @@ define(['jxg', 'utils/type', 'math/math'], function (JXG, Type, Mat) {
             t = V * (1.0 - (S * (1.0 - f)));
 
             switch (i) {
-            case 0:
-                R = V;
-                G = t;
-                B = p;
-                break;
-            case 1:
-                R = q;
-                G = V;
-                B = p;
-                break;
-            case 2:
-                R = p;
-                G = V;
-                B = t;
-                break;
-            case 3:
-                R = p;
-                G = q;
-                B = V;
-                break;
-            case 4:
-                R = t;
-                G = p;
-                B = V;
-                break;
-            case 5:
-                R = V;
-                G = p;
-                B = q;
-                break;
+                case 0:
+                    R = V;
+                    G = t;
+                    B = p;
+                    break;
+                case 1:
+                    R = q;
+                    G = V;
+                    B = p;
+                    break;
+                case 2:
+                    R = p;
+                    G = V;
+                    B = t;
+                    break;
+                case 3:
+                    R = p;
+                    G = q;
+                    B = V;
+                    break;
+                case 4:
+                    R = t;
+                    G = p;
+                    B = V;
+                    break;
+                case 5:
+                    R = V;
+                    G = p;
+                    B = q;
+                    break;
             }
         }
 
@@ -537,7 +538,6 @@ define(['jxg', 'utils/type', 'math/math'], function (JXG, Type, Mat) {
         return [h, s, v];
     };
 
-
     /**
      * Converts a color from the RGB color space into the LMS space. Input can be any valid HTML/CSS color definition.
      * @param {String,Array,Number} color A valid HTML or CSS styled color value, e.g. '#12ab21', '#abc', 'black',
@@ -551,7 +551,9 @@ define(['jxg', 'utils/type', 'math/math'], function (JXG, Type, Mat) {
     JXG.rgb2LMS = function (color, ag, ab) {
         var r, g, b, l, m, s, ret,
             // constants
-            matrix = [[0.05059983, 0.08585369, 0.00952420], [0.01893033, 0.08925308, 0.01370054], [0.00292202, 0.00975732, 0.07145979]];
+            matrix = [[0.05059983, 0.08585369, 0.00952420],
+                [0.01893033, 0.08925308, 0.01370054],
+                [0.00292202, 0.00975732, 0.07145979]];
 
         r = JXG.rgbParser(color, ag, ab);
         g = r[1];
@@ -586,7 +588,9 @@ define(['jxg', 'utils/type', 'math/math'], function (JXG, Type, Mat) {
     JXG.LMS2rgb = function (l, m, s) {
         var r, g, b, ret,
             // constants
-            matrix = [[30.830854, -29.832659, 1.610474], [-6.481468, 17.715578, -2.532642], [-0.375690, -1.199062, 14.273846]],
+            matrix = [[30.830854, -29.832659, 1.610474],
+                [-6.481468, 17.715578, -2.532642],
+                [-0.375690, -1.199062, 14.273846]],
 
             // re-gamma, inspired by GIMP modules/display-filter-color-blind.c:
             // Copyright (C) 2002-2003 Michael Natterer <mitch@gimp.org>,
@@ -728,57 +732,57 @@ define(['jxg', 'utils/type', 'math/math'], function (JXG, Type, Mat) {
         deficiency = deficiency.toLowerCase();
 
         switch (deficiency) {
-        case "protanopia":
-            a1 = -0.06150039994295001;
-            b1 = 0.08277001656812001;
-            c1 = -0.013200141220000003;
-            a2 = 0.05858939668799999;
-            b2 = -0.07934519995360001;
-            c2 = 0.013289415272000003;
-            inflection = 0.6903216543277437;
+            case "protanopia":
+                a1 = -0.06150039994295001;
+                b1 = 0.08277001656812001;
+                c1 = -0.013200141220000003;
+                a2 = 0.05858939668799999;
+                b2 = -0.07934519995360001;
+                c2 = 0.013289415272000003;
+                inflection = 0.6903216543277437;
 
-            tmp = s / m;
+                tmp = s / m;
 
-            if (tmp < inflection) {
-                l = -(b1 * m + c1 * s) / a1;
-            } else {
-                l = -(b2 * m + c2 * s) / a2;
-            }
-            break;
-        case "tritanopia":
-            a1 = -0.00058973116217;
-            b1 = 0.007690316482;
-            c1 = -0.01011703519052;
-            a2 = 0.025495080838999994;
-            b2 = -0.0422740347;
-            c2 = 0.017005316784;
-            inflection = 0.8349489908460004;
+                if (tmp < inflection) {
+                    l = -(b1 * m + c1 * s) / a1;
+                } else {
+                    l = -(b2 * m + c2 * s) / a2;
+                }
+                break;
+            case "tritanopia":
+                a1 = -0.00058973116217;
+                b1 = 0.007690316482;
+                c1 = -0.01011703519052;
+                a2 = 0.025495080838999994;
+                b2 = -0.0422740347;
+                c2 = 0.017005316784;
+                inflection = 0.8349489908460004;
 
-            tmp = m / l;
+                tmp = m / l;
 
-            if (tmp < inflection) {
-                s = -(a1 * l + b1 * m) / c1;
-            } else {
-                s = -(a2 * l + b2 * m) / c2;
-            }
-            break;
-        default:
-            a1 = -0.06150039994295001;
-            b1 = 0.08277001656812001;
-            c1 = -0.013200141220000003;
-            a2 = 0.05858939668799999;
-            b2 = -0.07934519995360001;
-            c2 = 0.013289415272000003;
-            inflection = 0.5763833686400911;
+                if (tmp < inflection) {
+                    s = -(a1 * l + b1 * m) / c1;
+                } else {
+                    s = -(a2 * l + b2 * m) / c2;
+                }
+                break;
+            default:
+                a1 = -0.06150039994295001;
+                b1 = 0.08277001656812001;
+                c1 = -0.013200141220000003;
+                a2 = 0.05858939668799999;
+                b2 = -0.07934519995360001;
+                c2 = 0.013289415272000003;
+                inflection = 0.5763833686400911;
 
-            tmp = s / l;
+                tmp = s / l;
 
-            if (tmp < inflection) {
-                m = -(a1 * l + c1 * s) / b1;
-            } else {
-                m = -(a2 * l + c2 * s) / b2;
-            }
-            break;
+                if (tmp < inflection) {
+                    m = -(a1 * l + c1 * s) / b1;
+                } else {
+                    m = -(a2 * l + c2 * s) / b2;
+                }
+                break;
         }
 
         rgb = JXG.LMS2rgb(l, m, s);
@@ -796,5 +800,221 @@ define(['jxg', 'utils/type', 'math/math'], function (JXG, Type, Mat) {
         return color;
     };
 
+    /**
+     * Determines highlight color to a given color. Done by reducing (or increasing) the opacity.
+     * @param {String} color HTML RGBA string containing the HTML color code.
+     * @returns {String} Returns a HTML RGBA color string
+     */
+    JXG.autoHighlight = function (colstr) {
+        var col = JXG.rgba2rgbo(colstr),
+            c = col[0],
+            opa = col[1];
+
+        if (colstr.charAt(0) === '#') {
+            if (opa < 0.3) {
+                opa *= 1.8;
+            } else {
+                opa *= 0.4;
+            }
+
+            return JXG.rgbo2rgba(c, opa);
+        }
+
+        return colstr;
+    };
+
+    /**
+     * Calculate whether a light or a dark color is needed as a contrast.
+     * Especially useful to determine whether white or black font goes
+     * better with a given background color.
+     * @param {String} hexColor HEX value of color.
+     * @param {String} [darkColor="#000000"] HEX string for a dark color.
+     * @param {String} [lightColor="#ffffff"] HEX string for a light color.
+     * @param {Number} [threshold=8]
+     * @returns {String} Returns darkColor or lightColor.
+     */
+    JXG.contrast = function (hexColor, darkColor, lightColor, threshold) {
+        var rgb,
+            black = '#000000',
+            rgbBlack,
+            l1, l2,
+            contrastRatio;
+
+        darkColor = darkColor || '#000000';
+        lightColor = lightColor || '#ffffff';
+        threshold = threshold || 7;
+
+        // hexColor RGB
+        rgb = JXG.rgbParser(hexColor);
+
+        // Black RGB
+        rgbBlack = JXG.rgbParser(black);
+
+        // Calc contrast ratio
+        l1 = 0.2126 * Math.pow(rgb[0] / 255, 2.2) +
+            0.7152 * Math.pow(rgb[1] / 255, 2.2) +
+            0.0722 * Math.pow(rgb[2] / 255, 2.2);
+
+        l2 = 0.2126 * Math.pow(rgbBlack[0] / 255, 2.2) +
+            0.7152 * Math.pow(rgbBlack[1] / 255, 2.2) +
+            0.0722 * Math.pow(rgbBlack[2] / 255, 2.2);
+
+        if (l1 > l2) {
+            contrastRatio = Math.floor((l1 + 0.05) / (l2 + 0.05));
+        } else {
+            contrastRatio = Math.floor((l2 + 0.05) / (l1 + 0.05));
+        }
+        contrastRatio = contrastRatio - 1;
+
+        // If contrast is more than threshold, return darkColor
+        if (contrastRatio > threshold) {
+            return darkColor;
+        }
+        // if not, return lightColor.
+        return lightColor;
+    };
+
+    /**
+     * Use the color scheme of JSXGraph up to version 1.3.2.
+     * This method has to be called before JXG.JSXGraph.initBoard();
+     *
+     * @see JXG.palette
+     * @see JXG.paletteWong
+     *
+     * @example
+     *
+     * JXG.setClassicColors();
+     * var board = JXG.JSXGraph.initBoard('jxgbox', {boundingbox: [-5, 5, 5,-5]});
+     *
+     */
+    JXG.setClassicColors = function() {
+        JXG.Options.elements.strokeColor = 'blue';
+        JXG.Options.elements.fillColor = 'red';
+        JXG.Options.hatch.strokeColor = 'blue';
+        JXG.Options.angle.fillColor = '#ff7f00';
+        JXG.Options.angle.highlightFillColor = '#ff7f00';
+        JXG.Options.angle.strokeColor = '#ff7f00';
+        JXG.Options.angle.label.strokeColor = 'blue';
+        JXG.Options.arc.strokeColor = 'blue';
+        JXG.Options.circle.center.fillColor = 'red';
+        JXG.Options.circle.center.strokeColor = 'blue';
+        JXG.Options.circumcircle.strokeColor = 'blue';
+        JXG.Options.circumcircle.center.fillColor = 'red';
+        JXG.Options.circumcircle.center.strokeColor = 'blue';
+        JXG.Options.circumcirclearc.strokeColor = 'blue';
+        JXG.Options.circumcirclesector.strokeColor = 'blue';
+        JXG.Options.circumcirclesector.fillColor = 'green';
+        JXG.Options.circumcirclesector.highlightFillColor = 'green';
+        JXG.Options.conic.strokeColor = 'blue';
+        JXG.Options.curve.strokeColor = 'blue';
+        JXG.Options.incircle.strokeColor = 'blue';
+        JXG.Options.incircle.center.fillColor = 'red';
+        JXG.Options.incircle.center.strokeColor = 'blue';
+        JXG.Options.inequality.fillColor = 'red';
+        JXG.Options.integral.fillColor = 'red';
+        JXG.Options.integral.curveLeft.color = 'red';
+        JXG.Options.integral.curveRight.color = 'red';
+        JXG.Options.line.strokeColor = 'blue';
+        JXG.Options.point.fillColor = 'red';
+        JXG.Options.point.strokeColor = 'red';
+        JXG.Options.polygon.fillColor = 'green';
+        JXG.Options.polygon.highlightFillColor = 'green';
+        JXG.Options.polygon.vertices.strokeColor = 'red';
+        JXG.Options.polygon.vertices.fillColor = 'red';
+        JXG.Options.regularpolygon.fillColor = 'green';
+        JXG.Options.regularpolygon.highlightFillColor = 'green';
+        JXG.Options.regularpolygon.vertices.strokeColor = 'red';
+        JXG.Options.regularpolygon.vertices.fillColor = 'red';
+        JXG.Options.riemannsum.fillColor = 'yellow';
+        JXG.Options.sector.fillColor = 'green';
+        JXG.Options.sector.highlightFillColor = 'green';
+        JXG.Options.semicircle.center.fillColor = 'red';
+        JXG.Options.semicircle.center.strokeColor = 'blue';
+        JXG.Options.slopetriangle.fillColor = 'red';
+        JXG.Options.slopetriangle.highlightFillColor = 'red';
+        JXG.Options.turtle.arrow.strokeColor = 'blue';
+    };
+
+    JXG.extend(JXG, /** @lends JXG */ {
+        /**
+         * Bang Wong color palette,
+         * optimized for various type
+         * of color blindness.
+         * It contains values for
+         * <ul>
+         * <li> 'black'
+         * <li> 'orange'
+         * <li> 'skyblue'
+         * <li> 'bluishgreen'
+         * <li> 'yellow'
+         * <li> 'darkblue'
+         * <li> 'vermillion'
+         * <li> 'reddishpurple'
+         * </ul>
+         *
+         * As substitutes for standard colors, it contains the following aliases:
+         *
+         * <ul>
+         * <li> black (= #000000)
+         * <li> blue (= darkblue)
+         * <li> green (= bluishgreen)
+         * <li> purple (= reddishpurple)
+         * <li> red (= vermillion)
+         * <li> white (= #ffffff)
+         * </ul>
+         *
+         * See <a href="https://www.nature.com/articles/nmeth.1618">Bang Wong: "Points of view: Color blindness"</a>
+         * and
+         * <a href="https://davidmathlogic.com/colorblind/">https://davidmathlogic.com/colorblind/</a>.
+         *
+         * @name JXG.paletteWong
+         * @type Object
+         * @see JXG.palette
+         * @example
+         * var p = board.create('line', [[-1, 1], [2, -3]], {strokeColor: JXG.paletteWong.yellow});
+         */
+        paletteWong: {
+            black: '#000000',
+            orange: '#E69F00',
+            skyblue: '#56B4E9',
+            bluishgreen: '#009E73',
+            yellow: '#F0E442',
+            darkblue: '#0072B2',
+            vermillion: '#D55E00',
+            reddishpurple: '#CC79A7',
+
+            blue: '#0072B2',
+            red: '#D55E00',   // vermillion
+            green: '#009E73', // bluishgreen
+            purple: '#CC79A7', // reddishpurple
+            white: '#ffffff'
+        }
+    });
+
+    /**
+     * Default color palette.
+     * Contains at least color values for
+     * <ul>
+     * <li> black
+     * <li> blue
+     * <li> green
+     * <li> purple
+     * <li> red
+     * <li> white
+     * <li> yellow
+     * </ul>
+     *
+     * @name JXG.palette
+     * @type Object
+     * @default JXG.paletteWong
+     * @see JXG.paletteWong
+     *
+     * @example
+     *
+     * var p = board.create('line', [[-1, 1], [2, -3]], {strokeColor: JXG.palette.yellow});
+     *
+     */
+    JXG.palette = JXG.paletteWong;
+
     return JXG;
-});
\ No newline at end of file
+});
