@@ -1,9 +1,10 @@
 /*
-    Copyright 2008-2013
+    Copyright 2008-2022
         Matthias Ehmann,
         Michael Gerhaeuser,
         Carsten Miller,
         Bianca Valentin,
+        Andreas Walter,
         Alfred Wassermann,
         Peter Wilfahrt
 
@@ -29,7 +30,6 @@
     and <http://opensource.org/licenses/MIT/>.
  */
 
-
 /*global JXG: true, define: true, window: true, document: true, navigator: true, module: true, global: true, self: true, require: true*/
 /*jslint nomen: true, plusplus: true*/
 
@@ -46,15 +46,78 @@
 
 define(['jxg', 'utils/type'], function (JXG, Type) {
 
-    "use strict";
+    'use strict';
+
+    JXG.extendConstants(JXG, /** @lends JXG */{
+        /**
+         * Determines the property that stores the relevant information in the event object.
+         * @type String
+         * @default 'touches'
+         * @private
+         */
+        touchProperty: 'touches',
+    });
 
     JXG.extend(JXG, /** @lends JXG */ {
         /**
-         * Determines the property that stores the relevant information in the event object.
-         * @type {String}
-         * @default 'touches'
+         * Determines whether evt is a touch event.
+         * @param evt {Event}
+         * @returns {Boolean}
          */
-        touchProperty: 'touches',
+        isTouchEvent: function (evt) {
+            return JXG.exists(evt[JXG.touchProperty]);
+        },
+
+        /**
+         * Determines whether evt is a pointer event.
+         * @param evt {Event}
+         * @returns {Boolean}
+         */
+        isPointerEvent: function (evt) {
+            return JXG.exists(evt.pointerId);
+        },
+
+        /**
+         * Determines whether evt is neither a touch event nor a pointer event.
+         * @param evt {Event}
+         * @returns {Boolean}
+         */
+        isMouseEvent: function (evt) {
+            return !JXG.isTouchEvent(evt)&&!JXG.isPointerEvent(evt);
+        },
+
+        /**
+         * Determines the number of touch points in a touch event.
+         * For other events, -1 is returned.
+         * @param evt {Event}
+         * @returns {Number}
+         */
+        getNumberOfTouchPoints: function (evt) {
+            var n = -1;
+
+            if (JXG.isTouchEvent(evt)) {
+                n = evt[JXG.touchProperty].length;
+            }
+
+            return n;
+        },
+
+        /**
+         * Checks whether an mouse, pointer or touch event evt is the first event of a multitouch event.
+         * Attention: When two or more pointer device types are being used concurrently,
+         *            it is only checked whether the passed event is the first one of its type!
+         * @param evt {Event}
+         * @returns {boolean}
+         */
+        isFirstTouch: function (evt) {
+            var touchPoints = JXG.getNumberOfTouchPoints(evt);
+
+            if (JXG.isPointerEvent(evt)) {
+                return evt.isPrimary;
+            }
+
+            return (touchPoints === 1);
+        },
 
         /**
          * A document/window environment is available.
@@ -63,6 +126,24 @@ define(['jxg', 'utils/type'], function (JXG, Type) {
          */
         isBrowser: typeof window === 'object' && typeof document === 'object',
 
+        /**
+         * Features of ECMAScript 6+ are available.
+         * @type Boolean
+         * @default false
+         */
+        supportsES6: function () {
+            var testMap;
+            /* jshint ignore:start */
+            try {
+                // This would kill the old uglifyjs: testMap = (a = 0) => a;
+                new Function('(a = 0) => a');
+                return true;
+            } catch (err) {
+                return false;
+            }
+            /* jshint ignore:end */
+        },
+
         /**
          * Detect browser support for VML.
          * @returns {Boolean} True, if the browser supports VML.
@@ -85,14 +166,14 @@ define(['jxg', 'utils/type'], function (JXG, Type) {
          * @returns {Boolean} True, if the browser supports HTML canvas.
          */
         supportsCanvas: function () {
-            var c,
-                hasCanvas = false;
+            var c, hasCanvas = false;
 
             if (this.isNode()) {
                 try {
                     c = (typeof module === 'object' ? module.require('canvas') : require('canvas'));
-                    hasCanvas = true;
-                } catch (err) { }
+                    hasCanvas = !!c;
+                } catch (err) {
+                }
             }
 
             return hasCanvas || (this.isBrowser && !!document.createElement('canvas').getContext);
@@ -105,7 +186,7 @@ define(['jxg', 'utils/type'], function (JXG, Type) {
         isNode: function () {
             // this is not a 100% sure but should be valid in most cases
 
-                // we are not inside a browser
+            // we are not inside a browser
             return !this.isBrowser && (
                 // there is a module object (plain node, no requirejs)
                 (typeof module === 'object' && !!module.exports) ||
@@ -124,10 +205,15 @@ define(['jxg', 'utils/type'], function (JXG, Type) {
 
         /**
          * Checks if the environments supports the W3C Pointer Events API {@link http://www.w3.org/Submission/pointer-events/}
-         * @return {Boolean}
+         * @returns {Boolean}
          */
         supportsPointerEvents: function () {
-            return JXG.isBrowser && window.navigator && (window.navigator.msPointerEnabled || window.navigator.pointerEnabled);
+            return !!(this.isBrowser && window.navigator &&
+                (window.PointerEvent ||             // Chrome/Edge/IE11+
+                    window.navigator.pointerEnabled || // IE11+
+                    window.navigator.msPointerEnabled  // IE10-
+                )
+            );
         },
 
         /**
@@ -135,7 +221,7 @@ define(['jxg', 'utils/type'], function (JXG, Type) {
          * @returns {Boolean} True, if the browser supports touch events.
          */
         isTouchDevice: function () {
-            return this.isBrowser && document.documentElement.hasOwnProperty('ontouchstart');
+            return this.isBrowser && window.ontouchstart !== undefined;
         },
 
         /**
@@ -167,15 +253,15 @@ define(['jxg', 'utils/type'], function (JXG, Type) {
          * @returns {Boolean}
          */
         isWebkitApple: function () {
-            return this.isApple() && (navigator.userAgent.search(/Mobile\/[0-9A-Za-z\.]*Safari/) > -1);
+            return this.isApple() && (navigator.userAgent.search(/Mobile\/[0-9A-Za-z.]*Safari/) > -1);
         },
 
         /**
          * Returns true if the run inside a Windows 8 "Metro" App.
-         * @return {Boolean}
+         * @returns {Boolean}
          */
         isMetroApp: function () {
-            return typeof window === 'object' && window.clientInformation && window.clientInformation.appName && window.clientInformation.appName.indexOf('MSAppHost') > -1;
+            return typeof window === 'object' && window.clientInformation && window.clientInformation.appVersion && window.clientInformation.appVersion.indexOf('MSAppHost') > -1;
         },
 
         /**
@@ -205,7 +291,7 @@ define(['jxg', 'utils/type'], function (JXG, Type) {
          * @type Number
          */
         ieVersion: (function () {
-            var undef, div, all,
+            var div, all,
                 v = 3;
 
             if (typeof document !== 'object') {
@@ -219,7 +305,7 @@ define(['jxg', 'utils/type'], function (JXG, Type) {
                 div.innerHTML = '<!--[if gt IE ' + (++v) + ']><' + 'i><' + '/i><![endif]-->';
             } while (all[0]);
 
-            return v > 4 ? v : undef;
+            return v > 4 ? v : undefined;
 
         }()),
 
@@ -228,28 +314,39 @@ define(['jxg', 'utils/type'], function (JXG, Type) {
          * @param {String} elementId The HTML id of an HTML DOM node.
          * @returns {Object} An object with the two properties width and height.
          */
-        getDimensions: function (elementId) {
+        getDimensions: function (elementId, doc) {
             var element, display, els, originalVisibility, originalPosition,
-                originalDisplay, originalWidth, originalHeight;
+                originalDisplay, originalWidth, originalHeight, style,
+                pixelDimRegExp = /\d+(\.\d*)?px/;
 
-            if (!JXG.isBrowser || elementId === null) {
+            if (!this.isBrowser || elementId === null) {
                 return {
                     width: 500,
                     height: 500
                 };
             }
 
+            doc = doc || document;
             // Borrowed from prototype.js
-            element = document.getElementById(elementId);
+            element = doc.getElementById(elementId);
             if (!Type.exists(element)) {
-                throw new Error("\nJSXGraph: HTML container element '" + elementId + "' not found.");
+                throw new Error('\nJSXGraph: HTML container element \'' + elementId + '\' not found.');
             }
 
             display = element.style.display;
 
             // Work around a bug in Safari
             if (display !== 'none' && display !== null) {
-                return {width: element.offsetWidth, height: element.offsetHeight};
+                if (element.clientWidth > 0 && element.clientHeight > 0) {
+                    return {width: element.clientWidth, height: element.clientHeight};
+                }
+
+                // a parent might be set to display:none; try reading them from styles
+                style = window.getComputedStyle ? window.getComputedStyle(element) : element.style;
+                return {
+                    width: pixelDimRegExp.test(style.width) ? parseFloat(style.width) : 0,
+                    height: pixelDimRegExp.test(style.height) ? parseFloat(style.height) : 0
+                };
             }
 
             // All *Width and *Height properties give 0 on elements with display set to none,
@@ -336,7 +433,7 @@ define(['jxg', 'utils/type'], function (JXG, Type) {
             i = Type.indexOf(owner['x_internal' + type], fn, 'origin');
 
             if (i === -1) {
-                JXG.debug('no such event function in internal list: ' + fn);
+                JXG.debug('removeEvent: no such event function in internal list: ' + fn);
                 return;
             }
 
@@ -383,9 +480,10 @@ define(['jxg', 'utils/type'], function (JXG, Type) {
          * Cross browser mouse / touch coordinates retrieval relative to the board's top left corner.
          * @param {Object} [e] The browsers event object. If omitted, <tt>window.event</tt> will be used.
          * @param {Number} [index] If <tt>e</tt> is a touch event, this provides the index of the touch coordinates, i.e. it determines which finger.
+         * @param {Object} [doc] The document object.
          * @returns {Array} Contains the position as x,y-coordinates in the first resp. second component.
          */
-        getPosition: function (e, index) {
+        getPosition: function (e, index, doc) {
             var i, len, evtTouches,
                 posx = 0,
                 posy = 0;
@@ -394,9 +492,15 @@ define(['jxg', 'utils/type'], function (JXG, Type) {
                 e = window.event;
             }
 
+            doc = doc || document;
             evtTouches = e[JXG.touchProperty];
 
-            if (Type.exists(index)) {
+            // touchend events have their position in "changedTouches"
+            if (Type.exists(evtTouches) && evtTouches.length === 0) {
+                evtTouches = e.changedTouches;
+            }
+
+            if (Type.exists(index) && Type.exists(evtTouches)) {
                 if (index === -1) {
                     len = evtTouches.length;
 
@@ -406,17 +510,17 @@ define(['jxg', 'utils/type'], function (JXG, Type) {
                             break;
                         }
                     }
+
                 } else {
                     e = evtTouches[index];
                 }
             }
 
-            if (e.pageX || e.pageY) {
-                posx = e.pageX;
-                posy = e.pageY;
-            } else if (e.clientX || e.clientY) {
-                posx = e.clientX + document.body.scrollLeft + document.documentElement.scrollLeft;
-                posy = e.clientY + document.body.scrollTop + document.documentElement.scrollTop;
+            // Scrolling is ignored.
+            // e.clientX is supported since IE6
+            if (e.clientX) {
+                posx = e.clientX;
+                posy = e.clientY;
             }
 
             return [posx, posy];
@@ -473,6 +577,7 @@ define(['jxg', 'utils/type'], function (JXG, Type) {
                 }
                 o = o.offsetParent;
             }
+
             return [l, t];
         },
 
@@ -483,11 +588,12 @@ define(['jxg', 'utils/type'], function (JXG, Type) {
          * @returns The value of the CSS property and <tt>undefined</tt> if it is not set.
          */
         getStyle: function (obj, stylename) {
-            var r;
+            var r,
+                doc = obj.ownerDocument;
 
             // Non-IE
-            if (window.getComputedStyle) {
-                r = document.defaultView.getComputedStyle(obj, null).getPropertyValue(stylename);
+            if (doc.defaultView && doc.defaultView.getComputedStyle) {
+                r = doc.defaultView.getComputedStyle(obj, null).getPropertyValue(stylename);
                 // IE
             } else if (obj.currentStyle && JXG.ieVersion >= 9) {
                 r = obj.currentStyle[stylename];
@@ -568,27 +674,49 @@ define(['jxg', 'utils/type'], function (JXG, Type) {
                     }
                 }
             }
+
+            // Zoom is used by reveal.js
+            if (Type.exists(obj.style.zoom)) {
+                str = obj.style.zoom;
+                if (str !== '') {
+                    cPos[0] *= parseFloat(str);
+                    cPos[1] *= parseFloat(str);
+                }
+            }
+
             return cPos;
         },
 
         /**
          * Scaling CSS transformations applied to the div element containing the JSXGraph constructions
-         * are determined. Not implemented are 'rotate', 'skew', 'skewX', 'skewY'.
-         * @returns {Array} 3x3 transformation matrix. See {@link JXG.Board#updateCSSTransforms}.
+         * are determined. In IE prior to 9, 'rotate', 'skew', 'skewX', 'skewY' are not supported.
+         * @returns {Array} 3x3 transformation matrix without translation part. See {@link JXG.Board#updateCSSTransforms}.
          */
         getCSSTransformMatrix: function (obj) {
             var i, j, str, arrstr, start, len, len2, arr,
+                st,
+                doc = obj.ownerDocument,
                 t = ['transform', 'webkitTransform', 'MozTransform', 'msTransform', 'oTransform'],
                 mat = [[1, 0, 0],
                     [0, 1, 0],
                     [0, 0, 1]];
 
-            // Take the first transformation matrix
-            len = t.length;
-            for (i = 0, str = ''; i < len; i++) {
-                if (Type.exists(obj.style[t[i]])) {
-                    str = obj.style[t[i]];
-                    break;
+            // This should work on all browsers except IE 6-8
+            if (doc.defaultView && doc.defaultView.getComputedStyle) {
+                st = doc.defaultView.getComputedStyle(obj, null);
+                str = st.getPropertyValue('-webkit-transform') ||
+                    st.getPropertyValue('-moz-transform') ||
+                    st.getPropertyValue('-ms-transform') ||
+                    st.getPropertyValue('-o-transform') ||
+                    st.getPropertyValue('transform');
+            } else {
+                // Take the first transformation matrix
+                len = t.length;
+                for (i = 0, str = ''; i < len; i++) {
+                    if (Type.exists(obj.style[t[i]])) {
+                        str = obj.style[t[i]];
+                        break;
+                    }
                 }
             }
 
@@ -608,7 +736,6 @@ define(['jxg', 'utils/type'], function (JXG, Type) {
                         mat = [[1, 0, 0],
                             [0, arr[0], arr[1]],
                             [0, arr[2], arr[3]]];
-                        // Missing are rotate, skew, skewX, skewY
                     } else if (str.indexOf('scaleX') === 0) {
                         mat[1][1] = arr[0];
                     } else if (str.indexOf('scaleY') === 0) {
@@ -619,6 +746,19 @@ define(['jxg', 'utils/type'], function (JXG, Type) {
                     }
                 }
             }
+
+            // CSS style zoom is used by reveal.js
+            // Recursively search for zoom style entries.
+            // This is necessary for reveal.js on webkit.
+            // It fails if the user does zooming
+            if (Type.exists(obj.style.zoom)) {
+                str = obj.style.zoom;
+                if (str !== '') {
+                    mat[1][1] *= parseFloat(str);
+                    mat[2][2] *= parseFloat(str);
+                }
+            }
+
             return mat;
         },
 
@@ -651,6 +791,106 @@ define(['jxg', 'utils/type'], function (JXG, Type) {
                 };
 
             window.setTimeout(timerFun, 1);
+        },
+
+        /**
+         * Calculate the scale factor and vertical shift for the JSXGraph div
+         * in full screen mode.
+         *
+         * @param {Object} obj Reference to a DOM node.
+         * @returns Object {scale: number, vshift: number}
+         * @see JXG.Board#fullscreenListener
+         * @private
+         */
+        _getScaleFactors: function (node) {
+            var width = node.getBoundingClientRect().width,
+                height = node.getBoundingClientRect().height,
+
+                // Determine the maximum scale factor.
+                r_w = window.screen.width / width,
+                r_h = window.screen.height / height,
+
+                // Determine the vertical shift to place the div in the center of the screen
+                vshift = (window.screen.height - height) * 0.5,
+
+                // Scaling factor: if not supplied, it's taken as large as possible
+                scale = Math.min(r_w, r_h);
+
+            // Adapt vshift and scale for landscape on tablets
+            if (window.matchMedia && window.matchMedia('(orientation:landscape)').matches &&
+                window.screen.width < window.screen.height) {
+                // Landscape on iOS: it returns 'landscape', but still width < height.
+                r_w = window.screen.height / width;
+                r_h = window.screen.width / height;
+                scale = Math.min(r_w, r_h);
+                vshift = (window.screen.width - height) * 0.5;
+            }
+            scale *= 0.85;
+
+            return { scale: scale, vshift: vshift, width: width };
+        },
+
+        /**
+         * Scale and vertically shift a DOM element (usually a JSXGraph div)
+         * inside of a parent DOM
+         * element which is set to fullscreen.
+         * This is realized with a CSS transformation.
+         *          *
+         * @param  {String} wrap_id  id of the parent DOM element which is in fullscreen mode
+         * @param  {String} inner_id id of the DOM element which is scaled and shifted
+         * @param  {Number} scale    Scaling factor
+         * @param  {Number} vshift   Vertical shift (in pixel)
+         *
+         * @private
+         * @see JXG.Board#toFullscreen
+         * @see JXG.Board#fullscreenListener
+         *
+         */
+        scaleJSXGraphDiv: function (wrap_id, inner_id, scale, vshift) {
+            var len = document.styleSheets.length, style,
+
+                pseudo_keys = [':fullscreen', ':-webkit-full-screen', ':-moz-full-screen', ':-ms-fullscreen'],
+                len_pseudo = pseudo_keys.length, i,
+
+                // CSS rules to center the inner div horizontally and vertically.
+                rule_inner = '{margin:0 auto;transform:matrix(' + scale + ',0,0,' + scale + ',0,' + vshift + ');}',
+
+                // A previously installed CSS rule to center the JSXGraph div has to
+                // be searched and removed again.
+                regex = new RegExp('.*#' + wrap_id + ':.*full.*screen.*#' + inner_id + '.*auto;.*transform:.*matrix');
+
+            if (len === 0) {
+                // In case there is not a single CSS rule defined at all.
+                style = document.createElement('style');
+                // WebKit hack :(
+                style.appendChild(document.createTextNode(''));
+                // Add the <style> element to the page
+                document.body.appendChild(style);
+                len = document.styleSheets.length;
+            }
+
+            // Remove a previously installed CSS rule.
+            if (document.styleSheets[len - 1].cssRules.length > 0 &&
+                regex.test(document.styleSheets[len - 1].cssRules[0].cssText) &&
+                document.styleSheets[len - 1].deleteRule) {
+
+                document.styleSheets[len - 1].deleteRule(0);
+            }
+
+            // Install a CSS rule to center the JSXGraph div at the first position of the list.
+            for (i = 0; i < len_pseudo; i++) {
+                try {
+                    document.styleSheets[len - 1].insertRule('#' + wrap_id + pseudo_keys[i] + ' #' + inner_id + rule_inner, 0);
+                    break;
+                } catch (err) {
+                    // console.log('JXG.scaleJSXGraphDiv: Could not add CSS rule "' + pseudo_keys[i] + '".');
+                    // console.log('One possible reason could be that the id of the JSXGraph container does not start with a letter.');
+                }
+            }
+            if (i === len_pseudo) {
+                console.log('JXG.scaleJSXGraphDiv: Could not add any CSS rule.');
+                console.log('One possible reason could be that the id of the JSXGraph container does not start with a letter.');
+            }
         }
     });
 
