@@ -1,9 +1,10 @@
 /*
-    Copyright 2008-2013
+    Copyright 2008-2022
         Matthias Ehmann,
         Michael Gerhaeuser,
         Carsten Miller,
         Bianca Valentin,
+        Andreas Walter,
         Alfred Wassermann,
         Peter Wilfahrt
 
@@ -29,7 +30,6 @@
     and <http://opensource.org/licenses/MIT/>.
  */
 
-
 /*global JXG: true, define: true, html_sanitize: true*/
 /*jslint nomen: true, plusplus: true*/
 
@@ -39,16 +39,17 @@
  */
 
 /**
- * @fileoverview type.js contains several functions to help deal with javascript's weak types. This file mainly consists
- * of detector functions which verify if a variable is or is not of a specific type and converter functions that convert
- * variables to another type or normalize the type of a variable.
+ * @fileoverview type.js contains several functions to help deal with javascript's weak types.
+ * This file mainly consists of detector functions which verify if a variable is or is not of
+ * a specific type and converter functions that convert variables to another type or normalize
+ * the type of a variable.
  */
 
 define([
     'jxg', 'base/constants'
 ], function (JXG, Const) {
 
-    "use strict";
+    'use strict';
 
     JXG.extend(JXG, /** @lends JXG */ {
         /**
@@ -58,7 +59,7 @@ define([
          * @returns {Boolean}
          */
         isId: function (board, s) {
-            return typeof s === 'string' && !!board.objects[s];
+            return (typeof s === 'string') && !!board.objects[s];
         },
 
         /**
@@ -87,7 +88,7 @@ define([
          * @returns {Boolean} True, if v is of type string.
          */
         isString: function (v) {
-            return typeof v === "string";
+            return typeof v === 'string';
         },
 
         /**
@@ -96,7 +97,7 @@ define([
          * @returns {Boolean} True, if v is of type number.
          */
         isNumber: function (v) {
-            return typeof v === "number";
+            return typeof v === 'number' || Object.prototype.toString.call(v) === '[Object Number]';
         },
 
         /**
@@ -105,7 +106,7 @@ define([
          * @returns {Boolean} True, if v is a function.
          */
         isFunction: function (v) {
-            return typeof v === "function";
+            return typeof v === 'function';
         },
 
         /**
@@ -120,7 +121,7 @@ define([
             if (Array.isArray) {
                 r = Array.isArray(v);
             } else {
-                r = (v !== null && typeof v === "object" && typeof v.splice === 'function' && typeof v.join === 'function');
+                r = (v !== null && typeof v === 'object' && typeof v.splice === 'function' && typeof v.join === 'function');
             }
 
             return r;
@@ -131,7 +132,7 @@ define([
          * @param v
          */
         isObject: function (v) {
-            return typeof v === 'object' && !JXG.isArray(v);
+            return typeof v === 'object' && !this.isArray(v);
         },
 
         /**
@@ -140,24 +141,93 @@ define([
          * @returns {Boolean} True, if v is of type JXG.Point.
          */
         isPoint: function (v) {
-            if (typeof v === 'object') {
+            if (v !== null && typeof v === 'object') {
                 return (v.elementClass === Const.OBJECT_CLASS_POINT);
             }
 
             return false;
         },
 
+        /**
+         * Checks if a given variable is a reference of a JSXGraph Point element or an array of length at least two or
+         * a function returning an array of length two or three.
+         * @param {JXG.Board} board
+         * @param v A variable of any type.
+         * @returns {Boolean} True, if v is of type JXG.Point.
+         */
+        isPointType: function (board, v) {
+            var val, p;
+
+            if (this.isArray(v)) {
+                return true;
+            }
+            if (this.isFunction(v)) {
+                val = v();
+                if (this.isArray(val) && val.length > 1) {
+                    return true;
+                }
+            }
+            p = board.select(v);
+            return this.isPoint(p);
+        },
+
+        /**
+         * Checks if a given variable is a reference of a JSXGraph transformation element or an array
+         * of JSXGraph transformation elements.
+         * @param v A variable of any type.
+         * @returns {Boolean} True, if v is of type JXG.Transformation.
+         */
+        isTransformationOrArray: function (v) {
+            if (v !== null) {
+                if (this.isArray(v) && v.length > 0) {
+                    return this.isTransformationOrArray(v[0]);
+                }
+                if (typeof v === 'object') {
+                    return (v.type === Const.OBJECT_TYPE_TRANSFORMATION);
+                }
+            }
+            return false;
+        },
+
         /**
          * Checks if a given variable is neither undefined nor null. You should not use this together with global
          * variables!
          * @param v A variable of any type.
+         * @param {Boolean} [checkEmptyString=false] If set to true, it is also checked whether v is not equal to ''.
          * @returns {Boolean} True, if v is neither undefined nor null.
          */
-        exists: (function (undef) {
-            return function (v) {
-                return !(v === undef || v === null);
-            };
-        }()),
+        exists: function (v, checkEmptyString) {
+            /* eslint-disable eqeqeq */
+            var result = !(v == undefined || v === null);
+            /* eslint-enable eqeqeq */
+            checkEmptyString = checkEmptyString || false;
+
+            if (checkEmptyString) {
+                return result && v !== '';
+            }
+            return result;
+        },
+        // exists: (function (undef) {
+        //     return function (v, checkEmptyString) {
+        //         var result = !(v === undef || v === null);
+
+        //         checkEmptyString = checkEmptyString || false;
+
+        //         if (checkEmptyString) {
+        //             return result && v !== '';
+        //         }
+        //         return result;
+        //     };
+        // }()),
+
+        /**
+         * Checks if v is an empty object or empty.
+         * @param v {Object|Array}
+         * @returns {boolean} True, if v is an empty object or array.
+         */
+        isEmpty: function(v) {
+            return Object.keys(v).length === 0;
+        },
 
         /**
          * Handle default parameters.
@@ -166,7 +236,7 @@ define([
          * @returns <tt>d</tt>, if <tt>v</tt> is undefined or null.
          */
         def: function (v, d) {
-            if (JXG.exists(v)) {
+            if (this.exists(v)) {
                 return v;
             }
 
@@ -179,7 +249,7 @@ define([
          * @returns {Boolean} String typed boolean value converted to boolean.
          */
         str2Bool: function (s) {
-            if (!JXG.exists(s)) {
+            if (!this.exists(s)) {
                 return true;
             }
 
@@ -187,7 +257,7 @@ define([
                 return s;
             }
 
-            if (JXG.isString(s)) {
+            if (this.isString(s)) {
                 return (s.toLowerCase() === 'true');
             }
 
@@ -205,7 +275,7 @@ define([
          * to evaluate.
          */
         createEvalFunction: function (board, param, n) {
-            var f = [], i, str;
+            var f = [], i;
 
             for (i = 0; i < n; i++) {
                 f[i] = JXG.createFunction(param[i], board, '', true);
@@ -218,7 +288,7 @@ define([
 
         /**
          * Convert a String, number or function into a function.
-         * @param term A variable of type string, function or number.
+         * @param {String|Number|Function} term A variable of type string, function or number.
          * @param {JXG.Board} board Reference to a JSXGraph board. It is required to resolve dependencies given
          * by a GEONE<sub>X</sub>T string, thus it must be a valid reference only in case one of the param
          * values is of type string.
@@ -231,7 +301,7 @@ define([
         createFunction: function (term, board, variableName, evalGeonext) {
             var f = null;
 
-            if ((!JXG.exists(evalGeonext) || evalGeonext) && JXG.isString(term)) {
+            if ((!this.exists(evalGeonext) || evalGeonext) && this.isString(term)) {
                 // Convert GEONExT syntax into  JavaScript syntax
                 //newTerm = JXG.GeonextParser.geonext2JS(term, board);
                 //return new Function(variableName,'return ' + newTerm + ';');
@@ -239,14 +309,14 @@ define([
                 //term = JXG.GeonextParser.replaceNameById(term, board);
                 //term = JXG.GeonextParser.geonext2JS(term, board);
                 f = board.jc.snippet(term, true, variableName, true);
-            } else if (JXG.isFunction(term)) {
+            } else if (this.isFunction(term)) {
                 f = term;
-            } else if (JXG.isNumber(term)) {
+            } else if (this.isNumber(term)) {
                 /** @ignore */
                 f = function () {
                     return term;
                 };
-            } else if (JXG.isString(term)) {
+            } else if (this.isString(term)) {
                 // In case of string function like fontsize
                 /** @ignore */
                 f = function () {
@@ -261,6 +331,70 @@ define([
             return f;
         },
 
+        /**
+         *  Test if the parents array contains existing points. If instead parents contains coordinate arrays or
+         *  function returning coordinate arrays
+         *  free points with these coordinates are created.
+         *
+         * @param {JXG.Board} board Board object
+         * @param {Array} parents Array containing parent elements for a new object. This array may contain
+         *    <ul>
+         *      <li> {@link JXG.Point} objects
+         *      <li> {@link JXG.GeometryElement#name} of {@link JXG.Point} objects
+         *      <li> {@link JXG.GeometryElement#id} of {@link JXG.Point} objects
+         *      <li> Coordinates of points given as array of numbers of length two or three, e.g. [2, 3].
+         *      <li> Coordinates of points given as array of functions of length two or three. Each function returns one coordinate, e.g.
+         *           [function(){ return 2; }, function(){ return 3; }]
+         *      <li> Function returning coordinates, e.g. function() { return [2, 3]; }
+         *    </ul>
+         *  In the last three cases a new point will be created.
+         * @param {String} attrClass Main attribute class of newly created points, see {@link JXG#copyAttributes}
+         * @param {Array} attrArray List of subtype attributes for the newly created points. The list of subtypes is mapped to the list of new points.
+         * @returns {Array} List of newly created {@link JXG.Point} elements or false if not all returned elements are points.
+         */
+        providePoints: function (board, parents, attributes, attrClass, attrArray) {
+            var i, j,
+                len,
+                lenAttr = 0,
+                points = [], attr, val;
+
+            if (!this.isArray(parents)) {
+                parents = [parents];
+            }
+            len = parents.length;
+            if (this.exists(attrArray)) {
+                lenAttr = attrArray.length;
+            }
+            if (lenAttr === 0) {
+                attr = this.copyAttributes(attributes, board.options, attrClass);
+            }
+
+            for (i = 0; i < len; ++i) {
+                if (lenAttr > 0) {
+                    j = Math.min(i, lenAttr - 1);
+                    attr = this.copyAttributes(attributes, board.options, attrClass, attrArray[j]);
+                }
+                if (this.isArray(parents[i]) && parents[i].length > 1) {
+                    points.push(board.create('point', parents[i], attr));
+                    points[points.length - 1]._is_new = true;
+                } else if (this.isFunction(parents[i])) {
+                    val = parents[i]();
+                    if (this.isArray(val) && (val.length > 1)) {
+                        points.push(board.create('point', [parents[i]], attr));
+                        points[points.length - 1]._is_new = true;
+                    }
+                } else {
+                    points.push(board.select(parents[i]));
+                }
+
+                if (!this.isPoint(points[i])) {
+                    return false;
+                }
+            }
+
+            return points;
+        },
+
         /**
          * Generates a function which calls the function fn in the scope of owner.
          * @param {Function} fn Function to call.
@@ -280,7 +414,7 @@ define([
          * @returns If <tt>val</tt> is a function, it is evaluated and the result is returned. Otherwise <tt>val</tt> is returned.
          */
         evaluate: function (val) {
-            if (JXG.isFunction(val)) {
+            if (this.isFunction(val)) {
                 return val();
             }
 
@@ -296,7 +430,7 @@ define([
          * <tt>-1</tt> if the value was not found.
          */
         indexOf: function (array, value, sub) {
-            var i, s = JXG.exists(sub);
+            var i, s = this.exists(sub);
 
             if (Array.indexOf && !s) {
                 return array.indexOf(value);
@@ -366,8 +500,12 @@ define([
             }
 
             for (i = 0; i < arr.length; i++) {
-                isArray = JXG.isArray(arr[i]);
+                isArray = this.isArray(arr[i]);
 
+                if (!this.exists(arr[i])) {
+                    arr[i] = '';
+                    continue;
+                }
                 for (j = i + 1; j < arr.length; j++) {
                     if (isArray && JXG.cmpArrays(arr[i], arr[j])) {
                         arr[i] = [];
@@ -380,14 +518,14 @@ define([
             j = 0;
 
             for (i = 0; i < arr.length; i++) {
-                isArray = JXG.isArray(arr[i]);
+                isArray = this.isArray(arr[i]);
 
                 if (!isArray && arr[i] !== '') {
                     ret[j] = arr[i];
-                    j += 1;
+                    j++;
                 } else if (isArray && arr[i].length !== 0) {
                     ret[j] = (arr[i].slice(0));
-                    j += 1;
+                    j++;
                 }
             }
 
@@ -451,7 +589,11 @@ define([
             }
 
             for (i = 0; i < a1.length; i++) {
-                if (a1[i] !== a2[i]) {
+                if (this.isArray(a1[i]) && this.isArray(a2[i])) {
+                    if (!this.cmpArrays(a1[i], a2[i])) {
+                        return false;
+                    }
+                } else if (a1[i] !== a2[i]) {
                     return false;
                 }
             }
@@ -487,36 +629,119 @@ define([
         trunc: function (n, p) {
             p = JXG.def(p, 0);
 
-            /*jslint bitwise: true*/
+            return this.toFixed(n, p);
+        },
 
-            if (p === 0) {
-                n = ~n;
-                n = ~n;
-            } else {
-                n = n.toFixed(p);
+        /**
+         * Decimal adjustment of a number.
+         * From https://developer.mozilla.org/de/docs/Web/JavaScript/Reference/Global_Objects/Math/round
+         *
+         * @param    {String}    type    The type of adjustment.
+         * @param    {Number}    value    The number.
+         * @param    {Number}    exp        The exponent (the 10 logarithm of the adjustment base).
+         * @returns    {Number}            The adjusted value.
+         *
+         * @private
+         */
+        _decimalAdjust: function (type, value, exp) {
+            // If the exp is undefined or zero...
+            if (exp === undefined || +exp === 0) {
+                return Math[type](value);
             }
 
-            return n;
+            value = +value;
+            exp = +exp;
+            // If the value is not a number or the exp is not an integer...
+            if (isNaN(value) || !(typeof exp === 'number' && exp % 1 === 0)) {
+                return NaN;
+            }
+
+            // Shift
+            value = value.toString().split('e');
+            value = Math[type](+(value[0] + 'e' + (value[1] ? (+value[1] - exp) : -exp)));
+
+            // Shift back
+            value = value.toString().split('e');
+            return +(value[0] + 'e' + (value[1] ? (+value[1] + exp) : exp));
+        },
+
+        /**
+         * Round a number to given number of decimal digits.
+         *
+         * Example: JXG._toFixed(3.14159, -2) gives 3.14
+         * @param  {Number} value Number to be rounded
+         * @param  {Number} exp   Number of decimal digits given as negative exponent
+         * @return {Number}       Rounded number.
+         *
+         * @private
+         */
+        _round10: function (value, exp) {
+            return this._decimalAdjust('round', value, exp);
+        },
+
+        /**
+         * "Floor" a number to given number of decimal digits.
+         *
+         * Example: JXG._toFixed(3.14159, -2) gives 3.14
+         * @param  {Number} value Number to be floored
+         * @param  {Number} exp   Number of decimal digits given as negative exponent
+         * @return {Number}       "Floored" number.
+         *
+         * @private
+         */
+        _floor10: function (value, exp) {
+            return this._decimalAdjust('floor', value, exp);
+        },
+
+        /**
+         * "Ceil" a number to given number of decimal digits.
+         *
+         * Example: JXG._toFixed(3.14159, -2) gives 3.15
+         * @param  {Number} value Number to be ceiled
+         * @param  {Number} exp   Number of decimal digits given as negative exponent
+         * @return {Number}       "Ceiled" number.
+         *
+         * @private
+         */
+        _ceil10: function (value, exp) {
+            return this._decimalAdjust('ceil', value, exp);
+        },
+
+        /**
+         * Replacement of the default toFixed() method.
+         * It does a correct rounding (independent of the browser) and
+         * returns "0.00" for toFixed(-0.000001, 2) instead of "-0.00" which
+         * is returned by JavaScript's toFixed()
+         *
+         * @memberOf JXG
+         * @param  {Number} num    Number tp be rounded
+         * @param  {Number} digits Decimal digits
+         * @return {String}        Rounded number is returned as string
+         */
+        toFixed: function (num, digits) {
+            return this._round10(num, -digits).toFixed(digits);
         },
 
         /**
          * Truncate a number <tt>val</tt> automatically.
+         * @memberOf JXG
          * @param val
          * @returns {Number}
          */
         autoDigits: function (val) {
-            var x = Math.abs(val);
+            var x = Math.abs(val),
+                str;
 
             if (x > 0.1) {
-                x = val.toFixed(2);
+                str = this.toFixed(val, 2);
             } else if (x >= 0.01) {
-                x = val.toFixed(4);
+                str = this.toFixed(val, 4);
             } else if (x >= 0.0001) {
-                x = val.toFixed(6);
+                str = this.toFixed(val, 6);
             } else {
-                x = val;
+                str = val;
             }
-            return x;
+            return str;
         },
 
         /**
@@ -569,7 +794,7 @@ define([
          */
         cloneAndCopy: function (obj, obj2) {
             var r,
-                cObj = function () {};
+                cObj = function () { return undefined; };
 
             cObj.prototype = obj;
 
@@ -584,13 +809,12 @@ define([
             /*jslint forin:false*/
             /*jshint forin:false*/
 
-
             return cObj;
         },
 
         /**
          * Recursively merges obj2 into obj1. Contrary to {@link JXG#deepCopy} this won't create a new object
-         * but instead will
+         * but instead will overwrite obj1.
          * @param {Object} obj1
          * @param {Object} obj2
          * @returns {Object}
@@ -637,7 +861,7 @@ define([
          * @returns {Object} copy of obj or merge of obj and obj2.
          */
         deepCopy: function (obj, obj2, toLower) {
-            var c, i, prop, j, i2;
+            var c, i, prop, i2;
 
             toLower = toLower || false;
 
@@ -646,15 +870,19 @@ define([
             }
 
             // missing hasOwnProperty is on purpose in this function
-            /*jslint forin:true*/
-            /*jshint forin:false*/
-
             if (this.isArray(obj)) {
                 c = [];
                 for (i = 0; i < obj.length; i++) {
                     prop = obj[i];
                     if (typeof prop === 'object') {
-                        c[i] = this.deepCopy(prop);
+                        // We certainly do not want to recurse into a JSXGraph object.
+                        // This would for sure result in an infinite recursion.
+                        // As alternative we copy the id of the object.
+                        if (this.exists(prop.board)) {
+                            c[i] = prop.id;
+                        } else {
+                            c[i] = this.deepCopy(prop);
+                        }
                     } else {
                         c[i] = prop;
                     }
@@ -662,41 +890,45 @@ define([
             } else {
                 c = {};
                 for (i in obj) {
-                    i2 = toLower ? i.toLowerCase() : i;
-
-                    prop = obj[i];
-                    if (typeof prop === 'object') {
-                        c[i2] = this.deepCopy(prop);
-                    } else {
-                        c[i2] = prop;
+                    if (obj.hasOwnProperty(i)) {
+                        i2 = toLower ? i.toLowerCase() : i;
+                        prop = obj[i];
+                        if (prop !== null && typeof prop === 'object') {
+                            if (this.exists(prop.board)) {
+                                c[i2] = prop.id;
+                            } else {
+                                c[i2] = this.deepCopy(prop);
+                            }
+                        } else {
+                            c[i2] = prop;
+                        }
                     }
                 }
 
                 for (i in obj2) {
-                    i2 = toLower ? i.toLowerCase() : i;
+                    if (obj2.hasOwnProperty(i)) {
+                        i2 = toLower ? i.toLowerCase() : i;
 
-                    prop = obj2[i];
-                    if (typeof prop === 'object') {
-                        if (JXG.isArray(prop) || !JXG.exists(c[i2])) {
-                            c[i2] = this.deepCopy(prop);
+                        prop = obj2[i];
+                        if (typeof prop === 'object') {
+                            if (this.isArray(prop) || !this.exists(c[i2])) {
+                                c[i2] = this.deepCopy(prop);
+                            } else {
+                                c[i2] = this.deepCopy(c[i2], prop, toLower);
+                            }
                         } else {
-                            c[i2] = this.deepCopy(c[i2], prop, toLower);
+                            c[i2] = prop;
                         }
-                    } else {
-                        c[i2] = prop;
                     }
                 }
             }
 
-            /*jslint forin:false*/
-            /*jshint forin:true*/
-
             return c;
         },
 
         /**
          * Generates an attributes object that is filled with default values from the Options object
-         * and overwritten by the user speciified attributes.
+         * and overwritten by the user specified attributes.
          * @param {Object} attributes user specified attributes
          * @param {Object} options defaults options
          * @param {String} s variable number of strings, e.g. 'slider', subtype 'point1'.
@@ -716,7 +948,6 @@ define([
                     'integral': 1
                 };
 
-
             len = arguments.length;
             if (len < 3 || primitives[s]) {
                 // default options from Options.elements
@@ -734,7 +965,7 @@ define([
             o = options;
             isAvail = true;
             for (i = 2; i < len; i++) {
-                if (JXG.exists(o[arguments[i]])) {
+                if (this.exists(o[arguments[i]])) {
                     o = o[arguments[i]];
                 } else {
                     isAvail = false;
@@ -749,7 +980,7 @@ define([
             o = attributes;
             isAvail = true;
             for (i = 3; i < len; i++) {
-                if (JXG.exists(o[arguments[i]])) {
+                if (this.exists(o[arguments[i]])) {
                     o = o[arguments[i]];
                 } else {
                     isAvail = false;
@@ -760,25 +991,51 @@ define([
                 this.extend(a, o, null, true);
             }
 
+            if (arguments[2] === 'board') {
+                // For board attributes we are done now.
+                return a;
+            }
+
             // Special treatment of labels
             o = options;
             isAvail = true;
             for (i = 2; i < len; i++) {
-                if (JXG.exists(o[arguments[i]])) {
+                if (this.exists(o[arguments[i]])) {
                     o = o[arguments[i]];
                 } else {
                     isAvail = false;
                     break;
                 }
             }
-            if (isAvail && JXG.exists(o.label)) {
-                a.label =  JXG.deepCopy(o.label, a.label);
+            if (isAvail && this.exists(o.label)) {
+                a.label = JXG.deepCopy(o.label, a.label);
             }
             a.label = JXG.deepCopy(options.label, a.label);
 
             return a;
         },
 
+        /**
+         * Copy all prototype methods from object "superObject" to object
+         * "subObject". The constructor of superObject will be available
+         * in subObject as subObject.constructor[constructorName].
+         * @param {Object} subObj A JavaScript object which receives new methods.
+         * @param {Object} superObj A JavaScript object which lends its prototype methods to subObject
+         * @returns {String} constructorName Under this name the constructor of superObj will be available
+         * in subObject.
+         * @private
+         */
+        copyPrototypeMethods: function (subObject, superObject, constructorName) {
+            var key;
+
+            subObject.prototype[constructorName] = superObject.prototype.constructor;
+            for (key in superObject.prototype) {
+                if (superObject.prototype.hasOwnProperty(key)) {
+                    subObject.prototype[key] = superObject.prototype[key];
+                }
+            }
+        },
+
         /**
          * Converts a JavaScript object into a JSON string.
          * @param {Object} obj A JavaScript object, functions will be ignored.
@@ -802,42 +1059,42 @@ define([
             }
 
             switch (typeof obj) {
-            case 'object':
-                if (obj) {
-                    list = [];
+                case 'object':
+                    if (obj) {
+                        list = [];
 
-                    if (JXG.isArray(obj)) {
-                        for (i = 0; i < obj.length; i++) {
-                            list.push(JXG.toJSON(obj[i], noquote));
-                        }
-
-                        return '[' + list.join(',') + ']';
-                    }
-
-                    for (prop in obj) {
-                        if (obj.hasOwnProperty(prop)) {
-                            try {
-                                val = JXG.toJSON(obj[prop], noquote);
-                            } catch (e2) {
-                                val = '';
+                        if (this.isArray(obj)) {
+                            for (i = 0; i < obj.length; i++) {
+                                list.push(JXG.toJSON(obj[i], noquote));
                             }
 
-                            if (noquote) {
-                                list.push(prop + ':' + val);
-                            } else {
-                                list.push('"' + prop + '":' + val);
+                            return '[' + list.join(',') + ']';
+                        }
+
+                        for (prop in obj) {
+                            if (obj.hasOwnProperty(prop)) {
+                                try {
+                                    val = JXG.toJSON(obj[prop], noquote);
+                                } catch (e2) {
+                                    val = '';
+                                }
+
+                                if (noquote) {
+                                    list.push(prop + ':' + val);
+                                } else {
+                                    list.push('"' + prop + '":' + val);
+                                }
                             }
                         }
-                    }
 
-                    return '{' + list.join(',') + '} ';
-                }
-                return 'null';
-            case 'string':
-                return '\'' + obj.replace(/(["'])/g, '\\$1') + '\'';
-            case 'number':
-            case 'boolean':
-                return obj.toString();
+                        return '{' + list.join(',') + '} ';
+                    }
+                    return 'null';
+                case 'string':
+                    return '\'' + obj.replace(/(["'])/g, '\\$1') + '\'';
+                case 'number':
+                case 'boolean':
+                    return obj.toString();
             }
 
             return '0';
@@ -850,19 +1107,24 @@ define([
          */
         clearVisPropOld: function (el) {
             el.visPropOld = {
+                cssclass: '',
+                cssdefaultstyle: '',
+                cssstyle: '',
+                fillcolor: '',
+                fillopacity: '',
+                firstarrow: false,
+                fontsize: -1,
+                lastarrow: false,
+                left: -100000,
+                linecap: '',
+                shadow: false,
                 strokecolor: '',
                 strokeopacity: '',
                 strokewidth: '',
-                fillcolor: '',
-                fillopacity: '',
-                shadow: false,
-                firstarrow: false,
-                lastarrow: false,
-                cssclass: '',
-                fontsize: -1,
-                left: -100000,
-                right: 100000,
-                top: -100000
+                tabindex: -100000,
+                transitionduration: 0,
+                top: -100000,
+                visible: null
             };
 
             return el;
@@ -904,7 +1166,7 @@ define([
          * @returns {String}
          */
         unescapeHTML: function (str) {
-            // this regex is NOT insecure. We are replacing everything found with ''
+            // This regex is NOT insecure. We are replacing everything found with ''
             /*jslint regexp:true*/
             return str.replace(/<\/?[^>]+>/gi, '').replace(/&amp;/g, '&').replace(/&lt;/g, '<').replace(/&gt;/g, '>');
         },
@@ -932,7 +1194,7 @@ define([
             }
 
             if (str[0] === '.' || str[0] === ',') {
-                str = "0" + str;
+                str = '0' + str;
             }
 
             return str;
@@ -1003,10 +1265,11 @@ define([
          * @returns {String}
          */
         trim: function (str) {
-            str = str.replace(/^\s+/, '');
-            str = str.replace(/\s+$/, '');
-
-            return str;
+            // str = str.replace(/^\s+/, '');
+            // str = str.replace(/\s+$/, '');
+            //
+            // return str;
+            return str.replace(/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g, '');
         },
 
         /**
@@ -1017,7 +1280,7 @@ define([
          */
         sanitizeHTML: function (str, caja) {
             if (typeof html_sanitize === 'function' && caja) {
-                return html_sanitize(str, function () { return; }, function (id) { return id; });
+                return html_sanitize(str, function () { return undefined; }, function (id) { return id; });
             }
 
             if (str) {
@@ -1030,11 +1293,11 @@ define([
         /**
          * If <tt>s</tt> is a slider, it returns the sliders value, otherwise it just returns the given value.
          * @param {*} s
-         * @retusn {*} s.Value() if s is an element of type slider, s otherwise
+         * @returns {*} s.Value() if s is an element of type slider, s otherwise
          */
         evalSlider: function (s) {
-            if (s.type === Const.OBJECT_TYPE_GLIDER && typeof s.Value === 'function') {
-                s = s.Value();
+            if (s && s.type === Const.OBJECT_TYPE_GLIDER && typeof s.Value === 'function') {
+                return s.Value();
             }
 
             return s;
