@@ -1,5 +1,5 @@
 /*
-    Copyright 2008-2013
+    Copyright 2008-2022
         Matthias Ehmann,
         Michael Gerhaeuser,
         Carsten Miller,
@@ -7,25 +7,29 @@
         Alfred Wassermann,
         Peter Wilfahrt
 
-    This file is part of JSXGraph.
+    This file is part of JSXGraph and JSXCompressor.
 
     JSXGraph is free software dual licensed under the GNU LGPL or MIT License.
-    
+    JSXCompressor is free software dual licensed under the GNU LGPL or Apache License.
+
     You can redistribute it and/or modify it under the terms of the
-    
+
       * GNU Lesser General Public License as published by
         the Free Software Foundation, either version 3 of the License, or
         (at your option) any later version
       OR
       * MIT License: https://github.com/jsxgraph/jsxgraph/blob/master/LICENSE.MIT
-    
+      OR
+      * Apache License Version 2.0
+
     JSXGraph is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU Lesser General Public License for more details.
-    
-    You should have received a copy of the GNU Lesser General Public License and
-    the MIT License along with JSXGraph. If not, see <http://www.gnu.org/licenses/>
+
+    You should have received a copy of the GNU Lesser General Public License, Apache
+    License, and the MIT License along with JSXGraph. If not, see
+    <http://www.gnu.org/licenses/>, <https://www.apache.org/licenses/LICENSE-2.0.html>,
     and <http://opensource.org/licenses/MIT/>.
  */
 
@@ -124,7 +128,7 @@ define(['jxg'], function (JXG) {
      * @see http://www.cs.tut.fi/~albert
      */
     JXG.Util.Unzip = function (barray) {
-        var gpflags, crc, SIZE, fileout, flens, fmax, skipdir,
+        var gpflags, crc, SIZE, fileout, flens, fmax,
             outputArr = [],
             output = '',
             debug = false,
@@ -168,29 +172,38 @@ define(['jxg'], function (JXG) {
         function readBit() {
             var carry;
 
-            bits++;
-            carry = (bb & 1);
-            bb >>= 1;
-
-            if (bb === 0) {
-                bb = readByte();
+            try {   // Prevent problems on iOS7 with >>
+                bits++;
                 carry = (bb & 1);
-                bb = (bb >> 1) | 0x80;
-            }
+                bb >>= 1;
 
-            return carry;
+                if (bb === 0) {
+                    bb = readByte();
+                    carry = (bb & 1);
+                    bb = (bb >> 1) | 0x80;
+                }
+
+                return carry;
+            } catch (e) {
+                throw e;
+            }
         }
 
         function readBits(a) {
             var res = 0,
                 i = a;
 
-            while (i--) {
-                res = (res << 1) | readBit();
-            }
+            // Prevent problems on iOS7 with >>
+            try {
+                while (i--) {
+                    res = (res << 1) | readBit();
+                }
 
-            if (a) {
-                res = bitReverse[res] >> (8 - a);
+                if (a) {
+                    res = bitReverse[res] >> (8 - a);
+                }
+            } catch (e) {
+                throw e;
             }
 
             return res;
@@ -563,110 +576,140 @@ define(['jxg'], function (JXG) {
             return 0;
         }
 
+
+        /**
+         * nextFile:
+         * Extract the next file from the compressed archive.
+         * Calls skipdir() to proceed recursively.
+         *
+         * @return {Boolean}  false if the end of files' data section has baseElement
+         * reached. Then, then all recursive functions are stopped immediately.
+         *
+         */
         function nextFile() {
             var i, c, extralen, filelen, size, compSize, crc, method,
                 tmp = [];
 
-            outputArr = [];
-            modeZIP = false;
-            tmp[0] = readByte();
-            tmp[1] = readByte();
+            // Prevent problems on iOS7 with >>
+            try {
+                outputArr = [];
+                modeZIP = false;
+                tmp[0] = readByte();
+                tmp[1] = readByte();
 
-            //GZIP
-            if (tmp[0] === 0x78 && tmp[1] === 0xda) {
-                deflateLoop();
-                unzipped[files] = [outputArr.join(''), 'geonext.gxt'];
-                files++;
-            }
+                //GZIP
+                if (tmp[0] === 0x78 && tmp[1] === 0xda) {
+                    deflateLoop();
+                    unzipped[files] = [outputArr.join(''), 'geonext.gxt'];
+                    files++;
+                }
 
-            //GZIP
-            if (tmp[0] === 0x1f && tmp[1] === 0x8b) {
-                skipdir();
-                unzipped[files] = [outputArr.join(''), 'file'];
-                files++;
-            }
+                //GZIP
+                if (tmp[0] === 0x1f && tmp[1] === 0x8b) {
+                    skipdir();
+                    unzipped[files] = [outputArr.join(''), 'file'];
+                    files++;
+                }
 
-            //ZIP
-            if (tmp[0] === 0x50 && tmp[1] === 0x4b) {
-                modeZIP = true;
-                tmp[2] = readByte();
-                tmp[3] = readByte();
+                //ZIP
+                if (tmp[0] === 0x50 && tmp[1] === 0x4b) {
+                    modeZIP = true;
+                    tmp[2] = readByte();
+                    tmp[3] = readByte();
 
-                if (tmp[2] === 0x03 && tmp[3] === 0x04) {
-                    //MODE_ZIP
-                    tmp[0] = readByte();
-                    tmp[1] = readByte();
+                    if (tmp[2] === 0x03 && tmp[3] === 0x04) {
+                        //MODE_ZIP
+                        tmp[0] = readByte();
+                        tmp[1] = readByte();
 
-                    gpflags = readByte();
-                    gpflags |= (readByte() << 8);
+                        gpflags = readByte();
+                        gpflags |= (readByte() << 8);
 
-                    method = readByte();
-                    method |= (readByte() << 8);
+                        method = readByte();
+                        method |= (readByte() << 8);
 
-                    readByte();
-                    readByte();
-                    readByte();
-                    readByte();
+                        readByte();
+                        readByte();
+                        readByte();
+                        readByte();
 
-                    crc = readByte();
-                    crc |= (readByte() << 8);
-                    crc |= (readByte() << 16);
-                    crc |= (readByte() << 24);
+                        crc = readByte();
+                        crc |= (readByte() << 8);
+                        crc |= (readByte() << 16);
+                        crc |= (readByte() << 24);
 
-                    compSize = readByte();
-                    compSize |= (readByte() << 8);
-                    compSize |= (readByte() << 16);
-                    compSize |= (readByte() << 24);
+                        compSize = readByte();
+                        compSize |= (readByte() << 8);
+                        compSize |= (readByte() << 16);
+                        compSize |= (readByte() << 24);
 
-                    size = readByte();
-                    size |= (readByte() << 8);
-                    size |= (readByte() << 16);
-                    size |= (readByte() << 24);
+                        size = readByte();
+                        size |= (readByte() << 8);
+                        size |= (readByte() << 16);
+                        size |= (readByte() << 24);
 
-                    filelen = readByte();
-                    filelen |= (readByte() << 8);
+                        filelen = readByte();
+                        filelen |= (readByte() << 8);
 
-                    extralen = readByte();
-                    extralen |= (readByte() << 8);
+                        extralen = readByte();
+                        extralen |= (readByte() << 8);
 
-                    i = 0;
-                    nameBuf = [];
+                        i = 0;
+                        nameBuf = [];
 
-                    while (filelen--) {
-                        c = readByte();
-                        if (c === '/' | c === ':') {
-                            i = 0;
-                        } else if (i < NAMEMAX - 1) {
-                            nameBuf[i++] = String.fromCharCode(c);
+                        while (filelen--) {
+                            c = readByte();
+                            if (c === '/' | c === ':') {
+                                i = 0;
+                            } else if (i < NAMEMAX - 1) {
+                                nameBuf[i++] = String.fromCharCode(c);
+                            }
+                        }
+
+                        if (!fileout) {
+                            fileout = nameBuf;
+                        }
+
+                        i = 0;
+                        while (i < extralen) {
+                            c = readByte();
+                            i++;
+                        }
+
+                        SIZE = 0;
+                        if (method === 8) {
+                            deflateLoop();
+                            unzipped[files] = new Array(2);
+                            unzipped[files][0] = outputArr.join('');
+                            unzipped[files][1] = nameBuf.join('');
+                            files++;
+                        }
+
+                        if (skipdir()) {
+                            // We are beyond the files' data in the zip archive.
+                            // Let's get out immediately...
+                            return false;
                         }
                     }
-
-                    if (!fileout) {
-                        fileout = nameBuf;
-                    }
-
-                    i = 0;
-                    while (i < extralen) {
-                        c = readByte();
-                        i++;
-                    }
-
-                    SIZE = 0;
-
-                    if (method === 8) {
-                        deflateLoop();
-                        unzipped[files] = new Array(2);
-                        unzipped[files][0] = outputArr.join('');
-                        unzipped[files][1] = nameBuf.join('');
-                        files++;
-                    }
-
-                    skipdir();
+                    return true;
                 }
+            } catch (e) {
+                throw e;
             }
+            return false;
         }
 
-        skipdir = function () {
+
+        /**
+         * Test if the end of the files' data part of the archive has baseElement
+         * reached. If not, uncompressing is resumed.
+         *
+         * @return {Boolean}  true if the end of the files' data sections have
+         * been reached.
+         *
+         * @private
+         */
+        function skipdir() {
             var crc, compSize, size, os, i, c,
                 tmp = [];
 
@@ -676,6 +719,11 @@ define(['jxg'], function (JXG) {
                 tmp[2] = readByte();
                 tmp[3] = readByte();
 
+                // signature for data descriptor record: 0x08074b50
+                // 12 bytes:
+                //  crc 4 bytes
+                //  compressed size 4 bytes
+                // uncompressed size 4 bytes
                 if (tmp[0] === 0x50 &&
                         tmp[1] === 0x4b &&
                         tmp[2] === 0x07 &&
@@ -700,14 +748,20 @@ define(['jxg'], function (JXG) {
             }
 
             if (modeZIP) {
-                nextFile();
+                if (nextFile()) {
+                    // A file has been decompressed, we have to proceed
+                    return false;
+                }
             }
 
             tmp[0] = readByte();
             if (tmp[0] !== 8) {
-                return;
+                // It seems, we are beyond the files' data in the zip archive.
+                // We'll skip the rest..
+                return true;
             }
 
+            // There is another file in the zip file. We proceed...
             gpflags = readByte();
 
             readByte();
@@ -770,9 +824,16 @@ define(['jxg'], function (JXG) {
             size |= (readByte() << 24);
 
             if (modeZIP) {
-                nextFile();
+                if (nextFile()) {
+                    // A file has been decompressed, we have to proceed
+                    return false;
+                }
             }
-        };
+
+            // We are here in non-ZIP-files only,
+            // In that case the eturn value doesn't matter
+            return false;
+        }
 
         JXG.Util.Unzip.prototype.unzipFile = function (name) {
             var i;
@@ -795,4 +856,4 @@ define(['jxg'], function (JXG) {
     };
 
     return JXG.Util;
-});
\ No newline at end of file
+});
